#!/usr/bin/perl -w

# Application Checker
# Manual Test Page Module (tests_manual.pl)
#
# Copyright (C) 2007-2010 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use ManualTests;
use SimpleJournal;

my $error = '';

if (!$_GET{'test_run'}) {
	$error = 'Test run is not specified!';
}

my $testrun_dir = $CONFIG{'RESULTS_DIR'}.'/'.$_GET{'test_run'};
if (!$error) {
	if (!-d $testrun_dir) {
		$error = "Test run directory '$testrun_dir' does not exist!";
	}
}

if (!$error) {
	if ($_POST{'finish'}) {
		my $journal = new SimpleJournal($testrun_dir.'/test_journals/manual_tests.simplejournal');
		if (!$journal->{'ERROR'}) {
			$journal->test_name('Manual Tests');

			foreach my $test_id (sort keys %_POST) {
				next if ($test_id eq 'finish');
				$journal->test_case_start($test_id);
				$journal->test_case_result(($_POST{$test_id} eq 'yes') ? 'passed' : 'failed');
			}
			$journal->close();

			my $app_name = '';
			if (open(INFO_FILE, $testrun_dir.'/INFO')) {
				while (<INFO_FILE>) {
					s/[\r\n]//g;
					if (m/^Application:\s*(.*)/i) {
						$app_name = $1;
						last;
					}
				}
				close(INFO_FILE);
			}

			system(esc_cmdline("«$CONFIG{'UTILS_DIR'}/report.pl» «$testrun_dir» -n «$app_name» -o «$_GET{'test_run'}.tgz» >/dev/null 2>&1"));

			# Update TAR.GZ
			my $full_result_archive = "$testrun_dir/$_GET{'test_run'}.tgz";
			if (-e $full_result_archive) {
				unlink($full_result_archive);
				if (-f $full_result_archive) {
					die "Failed to remove old archive.\n";
				}
			}
			system(esc_cmdline("(cd «$CONFIG{'RESULTS_DIR'}» && tar czf «$_GET{'test_run'}.tgz» «$_GET{'test_run'}/» --force-local && mv «$_GET{'test_run'}.tgz» «$_GET{'test_run'}/») >/dev/null 2>&1"));

			print 'HTTP/1.0 302 Moved' . CRLF . 'Location: tests_results.pl?details='.esc_url($_GET{'test_run'}).'&tab=cert' . CRLF . CRLF;
			exit;
		}
		else {
			$error = $journal->{'ERROR'};
		}
	}
}

print 'HTTP/1.0 200 OK' . CRLF;
print 'Content-type: text/html' . CRLF . CRLF;

print_header("Manual Tests", "manual-tests");

my $fips;
if (!$error) {
	read_manual_tests("$SERVER_PARAM{'APP_DATA'}/results/$_GET{'test_run'}/ManualTests");
	$fips = read_fips("$SERVER_PARAM{'APP_DATA'}/results/$_GET{'test_run'}/test_journals");

	my $app_name = '<i>unknown application</i>';
	if (open(FILE, $testrun_dir.'/INFO')) {
		while (<FILE>) {
			if (m/Application: (.*)$/) {
				$app_name = esc_html($1);
				last;
			}
		}
		close(FILE);
	}

	my $journal = new SimpleJournalParser($testrun_dir.'/test_journals/manual_tests.simplejournal');
	my $journal_data = $journal->{'TESTCASES'};

	my @manual_test_ids = ();
	my $contents = '';
	my $res = <<DATA;
<h1>Manual Tests for <span class="title-highlight">$app_name</span></h1>

DATA

	if (scalar(@spec_manual_tests_sort) > 0) {
		$contents .= "  <li><a href=\"#manual_tests\"><b>$appchecker_branch{'STD'} Specific Tests</b></a></li>\n";
		$res .= "<a name=\"manual_tests\"></a><h2>$appchecker_branch{'STD'} Specific Tests</h2>\n\n";
		foreach my $test_id (@spec_manual_tests_sort) {
			push @manual_test_ids, $test_id;
			my $optional = ($spec_manual_tests{$test_id}->{'OPTIONAL'} ? '<span style="color: blue;"><i>Optional</i></span> ' : '');
			$sel_yes = (($journal_data->{$test_id} and ($journal_data->{$test_id} eq 'yes')) ? ' checked="checked"' : '');
			$sel_no = (($journal_data->{$test_id} and ($journal_data->{$test_id} eq 'no')) ? ' checked="checked"' : '');
			$res .= <<DATA;
<blockquote class="list">
  $optional$spec_manual_tests{$test_id}->{'MESSAGE'}<br />
  Is this requirement met in your application?
  <table cellpadding="0" cellspacing="0" border="0" id="$test_id-bkgd" style="margin-top: 0.5em;">
    <tr>
      <td style="color: #00a000; font-size: larger; font-weight: bold;"><label><input type="radio" name="$test_id" id="$test_id-yes" value="yes" onclick="javascript:clearBkgd('$test_id');"$sel_yes />&nbsp;Yes</label></td>
      <td style="width: 15px;"><img src="images/environment-spacer.gif" alt="" width="15" height="1" /></td>
      <td style="color: #c00000; font-size: larger; font-weight: bold;"><label><input type="radio" name="$test_id" id="$test_id-no" value="no" onclick="javascript:clearBkgd('$test_id');"$sel_no />&nbsp;No</label></td>
    </tr>
  </table>
</blockquote>

DATA
		}
	}

	foreach my $category (keys %$fips) {
		my $testcases = $fips->{$category}->{'TESTCASES'};
		if (scalar(keys %$testcases) > 0) {
			my $cat_title = ($fips->{$category}->{'TITLE'} or "Unknown category ($category)");
			$contents .= "  <li><a href=\"#$category\"><b>$cat_title</b></a></li>\n";
			$res .= "<a name=\"$category\"></a><h2>$cat_title</h2>\n\n";
			foreach my $testcase_id (keys %$testcases) {
				push @manual_test_ids, $testcase_id;
				$sel_yes = (($journal_data->{$testcase_id} and ($journal_data->{$testcase_id} eq 'yes')) ? ' checked="checked"' : '');
				$sel_no = (($journal_data->{$testcase_id} and ($journal_data->{$testcase_id} eq 'no')) ? ' checked="checked"' : '');
				$res .= <<DATA;
<blockquote class="list">
  <code>$testcases->{$testcase_id}->{'FILEPATH'}</code><br />
  $testcases->{$testcase_id}->{'MESSAGE'}<br />
  Please, check this test case manually, then mark <b>Yes</b> if the application is $appchecker_branch{'STD'} compliant regarding this issue, or <b>No</b> otherwise.
  <table cellpadding="0" cellspacing="0" border="0" id="$testcase_id-bkgd" style="margin-top: 0.5em;">
    <tr>
      <td style="color: #00a000; font-size: larger; font-weight: bold;"><label><input type="radio" name="$testcase_id" id="$testcase_id-yes" value="yes" onclick="javascript:clearBkgd('$testcase_id');"$sel_yes />&nbsp;Yes</label></td>
      <td style="width: 15px;"><img src="images/environment-spacer.gif" alt="" width="15" height="1" /></td>
      <td style="color: #c00000; font-size: larger; font-weight: bold;"><label><input type="radio" name="$testcase_id" id="$testcase_id-no" value="no" onclick="javascript:clearBkgd('$testcase_id');"$sel_no />&nbsp;No</label></td>
    </tr>
  </table>
</blockquote>
DATA
			}
		}
	}

	my $js_init_tests = join(', ', map("'$_'", @manual_test_ids));
	print <<DATA;
<h3>Test Categories</h3>
<div style="background-color: #f0f0ff; border: dashed 1px gray; padding: 3px; margin: 0.2em 0 1.5em 0; width: 20em;">
<ol>
$contents</ol>
</div>

<form method="post" name="manual_tests" id="manual_tests" action="">
$res

<input type="submit" name="finish" id="finish" value="Finish" style="width: 7em;" onclick="javascript:return onFormSubmit();" />
<input type="reset" name="reset" id="reset" value="Reset" style="width: 7em;" onclick="javascript:return onFormReset();" />
</form>

<script language="javascript" type="text/javascript">
// <![CDATA[
var manual_test_ids = new Array($js_init_tests);

function onFormSubmit() {
	var i;
	var elem_yes;
	var elem_no;
	var elem;
	var unchecked = 0;
	for (i = 0; i < manual_test_ids.length; ++i) {
		elem_yes = document.getElementById(manual_test_ids[i] + '-yes');
		elem_no = document.getElementById(manual_test_ids[i] + '-no');
		if (elem_yes && elem_no) {
			if (!elem_yes.checked && !elem_no.checked) {
				++unchecked;
				elem = document.getElementById(manual_test_ids[i] + '-bkgd');
				if (elem) {
					elem.style.backgroundColor = '#ffa0a0';
				}
			}
		}
		else {
			++unchecked;
		}
	}

	if (unchecked > 0) {
		var single_plural = ((unchecked == 1) ? 'is 1 question' : 'are ' + unchecked + ' questions');
		return confirm('There ' + single_plural + ' remained unanswered.\\nAre you sure you wish to store this incomplete result?');
	}
	else {
		return true;
	}
}

function clearBkgd(id) {
	elem = document.getElementById(id + '-bkgd');
	if (elem) {
		elem.style.backgroundColor = '';
	}
}

function onFormReset() {
	if (confirm('Are you sure you wish to clear your answers?')) {
		var i;
		for (i = 0; i < manual_test_ids.length; ++i) {
			clearBkgd(manual_test_ids[i]);
		}
		return true;
	}
	else {
		return false;
	}
}
// ]]>
</script>
DATA
}
else {
	print "<h3><font color=\"red\" size=\"+1\">Error!</font></h3>\n$error\n";
}

print_footer();
