#!/usr/bin/perl -w

# Application Checker
# Results Page Module (Tests_results.pm)
#
# Copyright (C) 2007-2010 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Tests_results;

use strict;
use Templates;
use Analysis; # utils/Analysis.pm

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
	&read_results_info
	&html_title &html_results_tabs
	&load_analysis_data
	&get_distr_sel
	&Summary
    &D_tab
		&distr_details
	&C_tab
		&com_details &com_common_details &com_distr_details
	&L_tab
		&lib_details &lib_distr_list
	&I_tab
		&int_details &int_distr_list
	&Dyn_tab
);

our @EXPORT_OK = qw(&tab_link);

# Colorscheme:
#  #CCFFCC - green background
#  #FEFECC - yellow background
#  #FEECCC - dark yellow background
#  #FFCCCC - dark red background
#  #FFDDDD - red background
#
#  'red'   - red foreground
#  #A04020 - yellow-red foreground
#  #907000 - yellow foreground
#  'green' - green foreground
#  'blue'  - blue foreground

#-----------------------------------------------------------------------
# Misc stuff:

sub tab_link ($) {
	my ($tab) = @_;
	
	return '?details='.esc_url($_GET{'details'}).'&tab='.esc_url($tab);
}

#-----------------------------------------------------------------------

# Reads information about the tested application
sub read_results_info($) {
	my ($res_dir) = @_;

	my $app_name = "";
	if (open(FILE, "$res_dir/INFO")) {
		while (<FILE>) {
			if (m/Application: (.*)$/) {
				$app_name = $1;
				last;
			}
		}
		close(FILE);
	}
	my $host_platform = read_file("$res_dir/test_logs/host_platform");
	$host_platform =~ s/[\r\n]//g;
	my $spec_version = read_file("$res_dir/test_logs/spec_version");
	if ($spec_version eq '') {
		if ($appchecker_branch{'STD'} eq 'LSB') {
			$spec_version = (read_file("$res_dir/test_logs/lsb_version") or '5.0');
		}
		else {
			$spec_version = (read_file("$res_dir/test_logs/moblin_version") or '2.0');
		}
	}
	$spec_version =~ s/[\r\n]//g;

	return ($app_name, $host_platform, $spec_version);
}
#-----------------------------------------------------------------------

# Renders main title of the results page.
sub html_title {
	my ($app_name, $architecture, $res_dir) = @_;

	my $res = '';
	if (! -f "$res_dir/test_logs/ftp_upload") {
		my $href_name = esc_url($_GET{'details'});
		$res .= <<DATA;
<table border="0" cellpadding="0" cellspacing="0" class="upload-suggest" onclick="javascript:document.location='upload_data.pl?details=$href_name';">
  <tr>
    <td valign="middle"><img src="images/upload_data.png" width="16" height="16" alt="" /></td>
    <td><img src="images/environment-spacer.gif" width="3" height="1" alt="" /></td>
    <td>Please <a href="upload_data.pl?details=$href_name" onclick="javascript:event.cancelBubble=true;"><b>upload</b></a> info about your application to the <a href="$appchecker_branch{'NAVIGATOR'}app.php" target="_blank" onclick="javascript:event.cancelBubble=true;">$appchecker_branch{'STD'} Navigator</a>.</td>
  </tr>
</table>
<script type="text/javascript" language="javascript">
// <![CDATA[
preload_images(new Array('themes/$appchecker_branch{'DIR'}/images/ajax_progress.gif'));
// ]]>
</script>
DATA
	$res .= '<h1>Analysis Results for <span class="title-highlight">'.esc_html($app_name).'</span> on <span class="title-highlight">'.esc_html($architecture).'</span></h1>'."\n\n";

	}
	return $res;
}
#-----------------------------------------------------------------------

my @res_tab_ids = ('distrs', 'com', 'libs', 'ints', 'cert', 'dynchk');
my %res_tab_names = (
	'distrs' => 'Distribution Compatibility',
	'com'    => 'App Components',
	'libs'   => 'External Libraries',
	'ints'   => 'External Interfaces',
	'cert'   => (($appchecker_branch{'STD'} eq 'LSB') ? 'LSB Certification' : 'Certification'),
	'dynchk' => 'DynChk Tests',
);

# Customizable tab bar
sub html_results_tabs {
	my ($selected) = @_;

	my $nl = "\n";
	my $res = "";

	# Table start
	$res .= '<table cellpadding="0" cellspacing="0" border="0" width="100%"'
		.' style="white-space: nowrap; font-weight: bold; text-align: center; font-size: 120%; margin-bottom: 5pt;">'.$nl;
	$res .= '<tr>'.$nl;

	# Left spacer
	$res .= '<td class="res-tab-delimiter" style="width: 10px; height: 2em;">';
	$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
	$res .= '</td>'.$nl;

	my $tabs_num = scalar(@res_tab_ids);

	foreach my $id (@res_tab_ids) {
		my $name = $res_tab_names{$id};

		# Tab
		if ( $id eq $selected ) {
			$res .= '<td class="res-tab-selected">';
		} else {
			$res .= '<td class="res-tab-highlight">';
			$res .= '<a class="res-tab-link" href="'.esc_html(tab_link($id)).'">';
		}
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$res .= '<img src="images/res-'.$id.'.png" width="16" height="16" alt="" />&nbsp;';
		$name = esc_html($name);
		$name =~ s/\s/&nbsp;/g;
		$res .= $name;
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$res .= '</a>' if $id ne $selected;
		$res .= '</td>';

		# Spacer
		$res .= '<td class="res-tab-delimiter" style="width: 10px;">';
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$res .= '</td>'.$nl;
	}

	$res .= '<td class="res-tab-delimiter" style="width: 100%;">';
	$res .= '<img src="images/environment-spacer.gif" width="1" height="1" alt="" />';
	$res .= '</td>';

	$res .= '</tr>';

	# Bottom line
	$res .= '<tr>';
	$res .= '<td colspan="'.(2 * $tabs_num+2).'" class="tab-footer" style="height: 10px;">';
	$res .= '<img src="images/environment-spacer.gif" width="1" height="1" alt="" />';
	$res .= '</td>';
	$res .= '</tr>';

	$res .= '</table>'.$nl;

	return $res;
}
#-----------------------------------------------------------------------

# Returns distribution selection passed by POST.
# Struct: $distr_sel->{$Did} = $checked;
sub get_distr_sel {
	my $distr_sel = {};
	
	if ( $_COOKIE{"ds"} ) {
		# User's selection
		for my $Did ( split /-/, $_COOKIE{"ds"} ) {
			next if !$Analysis::all_distrs->{$Did};
			$distr_sel->{$Did} = 1;
		}
		for my $Did ( keys %$Analysis::all_distrs ) {
			if ( !$distr_sel->{$Did} ) {
				$distr_sel->{$Did} = 0;
			}
		}
	} else {
		# Default values
		for my $Did ( keys %$Analysis::all_distrs ) {
			$distr_sel->{$Did} = 1;
		}
	}
	
	return $distr_sel;
}
#-----------------------------------------------------------------------

# Uses Analysis module to load all the analysis results.
sub load_analysis_data {
	my ($res_dir) = @_;
	
	# Load distribution list (contains distribution name, version, vendor, release year)
	Analysis::load_distrs( $res_dir."/Distributions" )
		or return 'Failed to read file "'.esc_html($res_dir).'/Distributions".<br/>This may happen if the Analyzer failed.';
	
	my $err = Analysis::load_main( $res_dir."/analysis_data" );
	if ( $err ) {
		return esc_html($err).'<br/>Failed to read file "'.esc_html($res_dir).'/analysis_data"';
	}

	Analysis::complete_structures()
		or return "Mailformed data was loaded.";
	
	Analysis::unneeded_libs();
	
	return;
}
#-----------------------------------------------------------------------

# Renders a control for selecting distribution subset
sub distr_selector {
	my ($distr_sel) = @_;

	my $nl = "\n";
	my $res = '';

	my $sel_distrs = scalar grep {$_} values %$distr_sel;
	my $total_distrs = scalar keys %$distr_sel;

	my %distrs = ();
	foreach my $did (keys %$distr_sel) {
		my $distr = $Analysis::all_distrs->{$did};
		$distrs{$did} = {
			'NAME'   => $distr->{'FULLNAME'},
			'YEAR'   => $distr->{'YEAR'},
			'FAMILY' => $distr->{'NAME'},
			'VENDOR' => $distr->{'VENDOR'}
		};
	}

	my @groups = (
		{
			'NAME'        => 'no',
			'DESCRIPTION' => '* No grouping *',
			'SORT_FUNC'   => undef,
			'ICON_FUNC'   => undef
		},
		{
			'NAME'        => 'FAMILY',
			'DESCRIPTION' => 'By distro family',
			'SORT_FUNC'   => sub {lc($a) cmp lc($b)},
			'ICON_FUNC'   => sub($) {'images/distros/'.lc($_[0]).'.png'}
		},
		{
			'NAME'        => 'VENDOR',
			'DESCRIPTION' => 'By vendor',
			'SORT_FUNC'   => sub {lc($a) cmp lc($b)},
			'ICON_FUNC'   => sub($) {'images/vendors/'.lc($_[0]).'.png'}
		},
		{
			'NAME'        => 'YEAR',
			'DESCRIPTION' => 'By release year',
			'SORT_FUNC'   => sub {$b <=> $a},
			'ICON_FUNC'   => undef
		}
	);
	my $num_groups = scalar(@groups);

	my @distr_groups = ();
	my @group_sel = ();
	foreach (my $i = 1; $i < $num_groups; ++$i) {
		my $group = $groups[$i];
		my $gname = $group->{'NAME'};
		$distr_groups[$i] = {};
		$group_sel[$i] = {};
		foreach my $did (keys %distrs) {
			my $group_value = $distrs{$did}->{$gname};
			if (defined($group_sel[$i]->{$group_value})) {
				if (!$distr_sel->{$did}) {
					$group_sel[$i]->{$group_value} = 0;
				}
			}
			else {
				$group_sel[$i]->{$group_value} = $distr_sel->{$did};
			}
			if (!defined($distr_groups[$i]->{$group_value})) {
				$distr_groups[$i]->{$group_value} = [];
			}
			push @{$distr_groups[$i]->{$group_value}}, $did;
		}
	}

	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[
var num_distrs = $sel_distrs;
var num_groups = $num_groups;

var distros_checked = new Array();
var distro_to_group = new Array();
var group_to_distros = new Array();

HEREDOC

	foreach my $did (sort {$a <=> $b} keys %$distr_sel) {
		$res .= "distros_checked[$did] = ".$distr_sel->{$did}.";\n";
	}
	$res .= "\n";

	for (my $i = 1; $i < $num_groups; ++$i) {
		$res .= "// Group $i: ".$groups[$i]->{'NAME'}."\n";
		$res .= "distro_to_group[$i] = new Array();\n";
		foreach my $did (sort {$a <=> $b} keys %distrs) {
			my $group_value = $distrs{$did}->{$groups[$i]->{'NAME'}};
			$res .= "distro_to_group[$i][$did] = '".esc_id($group_value)."';\n";
		}
	}
	$res .= "\n";

	for (my $i = 1; $i < $num_groups; ++$i) {
		$res .= "// Group $i: ".$groups[$i]->{'NAME'}."\n";
		$res .= "group_to_distros[$i] = new Array();\n";
		foreach my $group_value (sort {lc $a cmp lc $b} keys %{$distr_groups[$i]}) {
			my $arr = join(', ', @{$distr_groups[$i]->{$group_value}});
			$res .= "group_to_distros[$i]['".esc_id($group_value)."'] = [$arr];\n";
		}
	}
	$res .= "\n";

	my $cur_ds = "";
	my @selected_distrs = grep {$distr_sel->{$_}} keys %$distr_sel;
	# Left $cur_ds = "" if all distributions are selected.
	if ( @selected_distrs < keys %$distr_sel ) {
		$cur_ds = join "-", sort {$a<=>$b} @selected_distrs;
	}

	$res .= <<HEREDOC;
function update_sel_distrs() {
	var sel_distrs = document.getElementById('sel-distrs');
	if ( sel_distrs ) {
		sel_distrs.innerHTML = num_distrs;
		if ( num_distrs < $total_distrs ) {
			sel_distrs.style.color = 'red';
		} else {
			sel_distrs.style.color = '';
		}
	}
}

function my_reset() {
	var inputs = document.getElementsByTagName('input');
	for (var i = 0; i < inputs.length; i++) {
		if ((inputs[i].type == 'checkbox') && ((inputs[i].id.indexOf('distr-') == 0) || (inputs[i].id.indexOf('group-') == 0))){
			inputs[i].checked = 'checked';
		}
	}
	for (var did in distros_checked) {
		distros_checked[did] = 1;
	}
	num_distrs = $total_distrs;
	update_sel_distrs();
}

function checkGroupCheckbox(group_idx, group_value) {
	// Do not check the 'no' grouping
	if (group_idx == 0)
		return;
	for (var i=0; i<group_to_distros[group_idx][group_value].length; ++i) {
		var did = group_to_distros[group_idx][group_value][i];
		if (!document.getElementById('distr-' + group_idx + '-' + did).checked) {
			document.getElementById('group-' + group_idx + '-' + group_value).checked = false;
			return;
		}
	}
	document.getElementById('group-' + group_idx + '-' + group_value).checked = true;
}

function onDistrToggle(elem) {
	distrToggle(elem.id.substr(elem.id.lastIndexOf('-') + 1), elem.checked, 0);
	update_sel_distrs();
}

function distrToggle(did, checked, not_check_group) {
	if (distros_checked[did] == checked)
		return;
	distros_checked[did] = checked;
	if (checked)
		++num_distrs;
	else
		--num_distrs;
	for (var i=0; i<num_groups; ++i) {
		document.getElementById('distr-' + i + '-' + did).checked = checked;
		if ((i > 0) && (i != not_check_group))
			checkGroupCheckbox(i, distro_to_group[i][did]);
	}
}

function onGroupToggle(elem, group_idx, group_value) {
	// Do not check the 'no' grouping
	if (group_idx == 0)
		return;
	var checked = elem.checked;
	for (var i=0; i<group_to_distros[group_idx][group_value].length; ++i) {
		var did = group_to_distros[group_idx][group_value][i];
		distrToggle(did, checked, group_idx);
	}
	update_sel_distrs();
}

function switch_grouping(id) {
	for (var i=0; i<num_groups; ++i) {
		var div = document.getElementById('distr-grouping-' + i);
		if (div)
			div.style.display = ((i == id) ? '' : 'none');
	}
}

var cur_ds = '$cur_ds';

function distr_sel_apply() {
	var distrsel_form = document.getElementById('distrsel');
	if ( !distrsel_form ) return;
	
	var distrs = "";
	var all_distrs_selected = 1;
	
	var inputs = distrsel_form.getElementsByTagName('input');
	for (var i = 0; i < inputs.length; i++) {
		if ( inputs[i].name.match(/^distr-0-\\d+\$/) ) {
			if ( !inputs[i].checked ) {
				all_distrs_selected = 0;
				continue;
			}
			var did = inputs[i].name.replace(/^distr-0-/, "");
			if ( distrs ) distrs += "-";
			distrs += did;
		}
	}
	if ( all_distrs_selected ) distrs = "";
	
	cur_ds = distrs;
	setCookie('ds', distrs, null);
	
	onFilter();
}
// ]]>
</script>
HEREDOC

	# Toggler
	$res .= '<div>';
	$res .= '<a href="" onclick="javascript:return toggle(\'dseldiv\');"><b><u>Select distributions&hellip;</u></b></a> (<b><span id="sel-distrs">'.($sel_distrs==$total_distrs ? $sel_distrs : '<span style="color:red">'.$sel_distrs.'</span>').'</span></b> of <b>'.$total_distrs.'</b> selected)';
	$res .= '</div>';
	
	# Print selector
	$res .= '<div id="dseldiv" style="display: none;">';

	# Print group options
	$res .= <<DATA;
<table style="border: solid 1px gray; background: #e0e0e0; padding: 2px; margin: 0.5em 0;"><tr><td>
Group: <select name="distr-grouping" id="distr-grouping" onchange="javascript:switch_grouping(this.value);" style="width: 12.5em;">
DATA
	foreach (my $i = 0; $i < $num_groups; ++$i) {
		$res .= "  <option value=\"$i\"".(($i == 0) ? ' selected="selected"' : '').">"
				.esc_html($groups[$i]->{'DESCRIPTION'})."</option>\n";
	}
	$res .= <<DATA;
</select>
</td></tr></table>
DATA

	# Print DIVs for every grouping
	for (my $i = 1; $i < $num_groups; ++$i) {
		$res .= '<div id="distr-grouping-'.$i.'" style="border: solid 1px; padding: 3pt; margin-bottom: 3pt; height: 200pt; width: 200pt; overflow: auto; display: none; white-space: nowrap;">'."\n";
		foreach my $group_value (sort {$groups[$i]->{'SORT_FUNC'}()} keys %{$distr_groups[$i]}) {
			my $group_value_id = esc_id($group_value);
			my $group_value_html = esc_html($group_value);
			my $g_checked = ($group_sel[$i]->{$group_value} ? ' checked="checked"' : '');
			my $g_icon = (defined($groups[$i]->{'ICON_FUNC'}) ? '&nbsp;<img src="'.esc_html($groups[$i]->{'ICON_FUNC'}($group_value)).'" width="16" height="16" alt="" />' : '');
			$res .= <<DATA;
  <label><input type="checkbox" name="group-$i-$group_value_id" id="group-$i-$group_value_id"$g_checked onclick="javascript:onGroupToggle(this, $i, '$group_value_id');" />$g_icon&nbsp;$group_value_html</label><br />
DATA
			foreach my $did (sort {lc($distrs{$a}->{'NAME'}) cmp lc($distrs{$b}->{'NAME'})} @{$distr_groups[$i]->{$group_value}}) {
				my $d_checked = ($distr_sel->{$did} ? ' checked="checked"' : '');
				my $d_icon = '&nbsp;<img src="'.esc_html('images/distros/'.lc($distrs{$did}->{'FAMILY'}).'.png').'" width="16" height="16" alt="" />';
				my $distr_name_html = esc_html($distrs{$did}->{'NAME'});
				$res .= <<DATA;
  &nbsp;&nbsp;&nbsp;&nbsp;<label><input type="checkbox" name="distr-$i-$did" id="distr-$i-$did"$d_checked onclick="javascript:onDistrToggle(this);" />$d_icon&nbsp;$distr_name_html</label><br />
DATA
			}
		}
		$res .= "</div>\n";
	}

	# Print DIV with plain list
	$res .= <<DATA;
<form name="distrsel" id="distrsel" method="post" action="">
<div id="distr-grouping-0" style="border: solid 1px; padding: 3pt; margin-bottom: 3pt; height: 200pt; width: 200pt; overflow: auto; white-space: nowrap;">
DATA
	foreach my $did (sort {lc($distrs{$a}->{'NAME'}) cmp lc($distrs{$b}->{'NAME'})} keys %distrs) {
		my $d_checked = ($distr_sel->{$did} ? ' checked="checked"' : '');
		my $d_icon = '&nbsp;<img src="'.esc_html('images/distros/'.lc($distrs{$did}->{'FAMILY'}).'.png').'" width="16" height="16" alt="" />';
		my $distr_name_html = esc_html($distrs{$did}->{'NAME'});
		$res .= <<DATA;
  <label><input type="checkbox" name="distr-0-$did" id="distr-0-$did"$d_checked onclick="javascript:onDistrToggle(this);" />$d_icon&nbsp;$distr_name_html</label><br />
DATA
	}
	$res .= <<DATA;
</div>
<input type="button" value="Apply" style="width: 7em;" onclick="distr_sel_apply();"/>
 &nbsp; &nbsp; 
<input type="button" value="Reset" style="width: 7em;" onclick="my_reset();" />
</form>
</div>
DATA

	# Print hidden form with selected distros
	$res .= '<form name="hiddensel" action="" method="post" style="display:none">'.$nl;
	foreach my $Did ( keys %$distr_sel ) {
		$res .= '<input type="hidden" name="distr-0-'.$Did.'"'
				.' value="'.($distr_sel->{$Did}?1:0).'" />'.$nl;
	}
	$res .= '<input type="hidden" name="distrsel" value="1" />'.$nl;
	$res .= '</form>'.$nl;

	return $res;
}

sub parse_app_component {
	my ($s) = @_;
	
	my $disp_path = $s;
	$disp_path =~ s/\|//g;
	
	$s =~ s/^.*\.rpm\|([^|]+)/RPM:$1/;
	$s =~ s/^.*\.deb\|([^|]+)/DEB:$1/;
	$s =~ s/^.*\.(tar)\|([^|]+)/TAR:$2/;
	$s =~ s/^.*\.(tar\.gz|tgz)\|([^|]+)/TGZ:$2/;
	$s =~ s/^.*\.(tar\.bz2|tbz)\|([^|]+)/TBZ:$2/;
	$s =~ s/^PKG:.*\|([^|]+)/PKG:$1/;
	$s =~ s/\|//g;
	while ( length($s) > 50 ) { # shrink long paths via ellipsis
		$s =~ s{^((\w+:)?)(\.\.\.)?/[^/]+/}{$1.../} or last;
	}
	
	return ($s, $disp_path);
}

sub no_binary_div {
	my $res = "";
	
	$res .= '<div style="background:#fec;border:1px solid black;padding:10pt;margin-top:20pt;text-align:center;font-size:large">';
	$res .= $_[0] || 'This application contains no binary files (ELF executables or libraries).<br />Please see the '.$res_tab_names{'cert'}.' tab for the results of script analysis (Perl, Python'.(($appchecker_branch{'STD'} eq 'LSB') ? ', Shell' : '').' scripts).';
	$res .= '</div>';

	return $res;
}
#-----------------------------------------------------------------------

sub libs_data {
	my ($filt_distr, $filt_com) = @_;
	
	my $libs = {};
	# $libs->{$soname} => { # per column
	#  COMPONENTS->{$com_name} => 1,
	#  INTERFACES->{$fullname} => 1,
	#  DISTRS->{$Did} => 1,
	# };
	
	my $nonfilt = { LIBS => {} }; # Non-filtered information
	
	# Gather the NEEDED libs of all components.
	foreach my $com_obj ( values %$Analysis::app_components ) {
		# Non-filtered information
		foreach my $soname ( @{$com_obj->{NEEDED_LIBS}} ) {
			$nonfilt->{LIBS}{$soname} = 1;
		}
		
		if ( $filt_com && $com_obj->{NAME} ne $filt_com ) { next; } # component filter
		
		foreach my $soname ( @{$com_obj->{NEEDED_LIBS}} ) {
			$libs->{$soname}{COMPONENTS}{$com_obj->{NAME}} = 1; # COMPONENTS field
		}
	}
	
	foreach my $com_obj ( values %$Analysis::app_components ) {
		# Filter
		if ( $filt_com && $com_obj->{NAME} ne $filt_com ) { next; } # component filter
		
		foreach my $Did ( keys %$Analysis::all_distrs ) {
			if ($filt_distr && !$filt_distr->{$Did}) { next; } # distribution filter
			
			my $com_distr = $com_obj->{$Did};
			my $com_distr_libs = $com_distr->{LIBS};
			
			# Presence in distributions
			foreach my $soname ( @{$com_obj->{NEEDED_LIBS}} ) {
				next if !$com_distr_libs->{$soname};
				
				if ( $com_distr_libs->{$soname}{FOUND} ) {
					unless ( defined $libs->{$soname}{DISTRS}{$Did} ) { # do not set 1 if there is already 0 or 1
						$libs->{$soname}{DISTRS}{$Did} = 1; # DISTRS
					}
				} else {
					$libs->{$soname}{DISTRS}{$Did} = 0; # DISTRS
				}
			}
			# Gather interfaces
			foreach my $fullname ( keys %{$com_obj->{NEEDED_INTS}} ) {
				my $soname = $com_distr->{PROBABLE_LIB}{$fullname}
						|| $com_distr->{FOUND_INTS}{$fullname}
						|| "";
				
				if ( !$soname ) {
					# Do nothing
				}
				elsif ( $libs->{$soname} ) { # $soname is in NEEDED list
					$libs->{$soname}{INTERFACES}{$fullname} = 1;
				}
				else {
					Analysis::for_deps_tree $com_distr_libs, $soname, sub {
						my ($lib) = @_;
						if ( $libs->{$lib} ) { # $soname is in NEEDED list
							$libs->{$lib}{INTERFACES_MORE}{$fullname} = 1;
							return 0; # skip children
						}
						return;
					};
				}
				# TODO: calc interfaces that are not in any NEEDED library.
			}
		}
	}
	
	if ( wantarray ) {
		return ($libs, $nonfilt);
	}
	return $libs;
}

# Library compatibility table
sub Dyn_tab {
	my ($summary_fn, $details) = @_;

	my $res = "";
	
	#my $link = tab_link('dynchk');
	$res .= <<HEREDOC;
<div id="ajax_loading" style="display: none;"><div align="center">
Please wait&hellip;<br />
<img src="images/environment-spacer.gif" width="1" height="5" alt="" /><br />
<img src="themes/lsb/images/ajax_progress.gif" width="16" height="16" alt="" /><br />
<img src="images/environment-spacer.gif" width="1" height="7" alt="" /><br />
<input type="button" name="ajax_abort" id="ajax_abort" value="Abort" onclick="javascript:onAbort();" />
</div></div>

<script type="text/javascript" language="javascript">
// <![CDATA[


function call_dynchk() {
    var cmd = document.getElementById('cmd_text');
    if (cmd.value == "") {
	return false;
    }
    var t = cmd.value;
    var details = (RegExp(name + '=' + '(.+?)(&|\$)').exec(location.search)||[,null])[1]
    ajax_call_get("action=call_dynchk&command_line=" + t + "&details=" + details);
    cmd.value = "";
}

function dynchk_rm_table_row(report_name, row_id) {
    var cmd = document.getElementById('cmd_text');
    var details = (RegExp(name + '=' + '(.+?)(&|\$)').exec(location.search)||[,null])[1]
    var url = "action=delete_report&report_name=" + report_name +
					"&details=" + details;
    ajax_call_get(url);
    var tbl = document.getElementById('dynchk_reports');
    var html = "<a href='javascript:dynchk_rm_table_row("+'"'+
	    report_name+'"'+");'><b>X</b></a>";

    for (var i = 0, row; row = tbl.rows[i]; i++) {
      col = row.cells[2];

      if (col.innerHTML == html)
      {
	tbl.deleteRow(i);
      }
    }
}

function ajaxProcessResult(responseXML) {
	
  var deleted = responseXML.getElementsByTagName('deleted')[0];
  if (deleted)
  {
    return;
  }
  var tbl = document.getElementById('dynchk_reports');
  var lastRow = tbl.rows.length;
  // if there's no header row in the table, then iteration = lastRow + 1
  var iteration = lastRow;
  var row = tbl.insertRow(lastRow);
  

  var report = responseXML.getElementsByTagName('report')[0];
  var cmd = report.getElementsByTagName('cmd')[0].childNodes[0].nodeValue;
  var href = report.getElementsByTagName('href')[0].childNodes[0].nodeValue;
  var report_name = report.getElementsByTagName('report_name')[0].childNodes[0].nodeValue;
  var errors_count = report.getElementsByTagName('errors_count')[0].childNodes[0].nodeValue;

  // left cell
  var cellLeft = row.insertCell(0);
  //var textNode = document.createTextNode(iteration);
  var report_cmd = document.createElement("text");
  report_cmd.innerHTML = "<strong>"+cmd+"</strong>";
  cellLeft.appendChild(report_cmd);
  
  // middle cell
  var cellMiddle = row.insertCell(1);
  //var report_data = document.createElement("text");
  //report_data.innerHTML = href;
  //cellMiddle.appendChild(report_data);
  //row.cells[2].bgColor = '#777777';
  row.cells[1].innerHTML = href;
  
  if (errors_count > 0) {
    row.cells[1].bgColor = '#ffc0c0';
  }
  else {
    row.cells[1].bgColor = '#ccffcc';
  }
  row.cells[1].align = "center";

  // right cell
  var cellRight = row.insertCell(2);
  row.cells[2].innerHTML = "<a href='javascript:dynchk_rm_table_row("+'"'+
	    report_name+'"'+");'><b>X</b></a>";
}
// ]]>
</script>
HEREDOC
	$res .= <<DATA;


<form action="tableaddrow_nw.html" method="get">
<p>
<label>Command to execute:</label>
<input type="text" value="" id="cmd_text" onkeydown="if (event.keyCode == 13) { javascript:call_dynchk(); return false; }" />
<input type="button" value="run" onclick="javascript:call_dynchk();" />
</p>

<table class="wikitable" id="dynchk_reports">

  <tr>
    <th style="text-align: left;">Command</th>
    <th>Unique errors / Total errors</th>
    <th>x</th>
  </tr>

DATA

	my $r = open RFILE, "<", $summary_fn;
	my $color = '#ccffcc';
	if ($r) {
            my $line_id = 0;
	    while (<RFILE>) {
		$res .= "\t<tr>\n\t<th style=\"text-align: left;\">";
		my @values = split(' ', $_);
		my ($n, $up, $p, @cmd) = @values;
		$res .= "@cmd</th>\n";
		my $url = "\"/tests_results.pl?details=$details&tab=dynchk&report=$n\"";
		if ($p > 0) {
		    $color = "#ffc0c0";
		}
		else {
		    $color = "#ccffcc";
		}
		$res .= "<td align=\"center\" bgcolor=\"$color\";><a href=$url>$up / $p</a></td>\n";
		$res .= "<td><a href='javascript:dynchk_rm_table_row(\"$n\");'><b>X</b></a></td>\n</tr>\n";
		$line_id += 1;
	    }
	}
	close RFILE;

	print "</table>\n</form>\n";

	return $res;
}
#-----------------------------------------------------------------------


# Library compatibility table
sub L_tab {
	my ($selected_lsb_ver, $distr_sel, $filt_com, $filt_show) = @_;

	my $res = "";
	my $nl = "\n";
	
	my $filtered = $filt_show || $filt_com || scalar grep {!$_} values %$distr_sel;
	
	my ($libs, $nonfilt_info) = libs_data( $distr_sel, $filt_com );
	
	# Check if the table is empty
	if ( !$filtered && !%$Analysis::app_components ) {
		$res .= no_binary_div();
		return $res;
	}
	if ( !$filtered && !%$libs ) {
		$res .= no_binary_div("The application needs no external libraries.");
		return $res;
	}

	$res .= <<HEREDOC;
The table below lists all external libraries required by your Application (based on DT_NEEDED ELF section). Distribution presence and $appchecker_branch{'STD'} compliance statuses are provided for each library. You can adjust distributions of interest and the report will be updated appropriately.<br /><br />
HEREDOC

	my $link = tab_link('libs');
	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[
function toggle( id )
{
	var v = document.getElementById( id );
	if ( v ) {
		v.style.display = (v.style.display == 'none' ? '' : 'none');
	}
	return false;
}

function popup_win( link )
{
	var w = window.open( link, 'details',
			'toolbar=0,menubar=0,scrollbars=1,height=750,width=850,dependent=yes,dialog=yes,resizable=1'
		);
	w.moveTo(100,0);
	w.focus();

	return false;
}
// ]]>
</script>
HEREDOC
	
	my ($table, $row_n) = L_table($libs, $selected_lsb_ver, $distr_sel, $filt_com, $filt_show);
	
	# Filters
	my $filt = "";
	
	$filt .= distr_selector( $distr_sel ); # Distribution selector
	
	$filt .= '<form name="filter" method="get" action="">';
	$filt .= '<table><tr><td>';
	
	$filt .= '<table cellspacing="0" class="bartable"><tr>';
	
	# "Component" filter
	$filt .= '<td style="border-right:0"><b>Component:</b></td>'
		.'<td>'
		.'<select name="filt_com" id="filt_com" onchange="onFilter()" style="width: 20em;">'.$nl
		.'<option value="">All</option>'.$nl;
	foreach my $com ( sort keys %$Analysis::app_components ) {
		my ($disp_name, $disp_path) = parse_app_component($com);
		my $selected = "";
		if ( $filt_com && $com eq $filt_com ) {
			$selected = ' selected="selected"';
		}
		$filt .= '<option value="'.esc_html($com).'"'
					.$selected.'>'.esc_html($disp_name).'</option>'.$nl;
	}
	$filt .= '</select>'.$nl
		.'</td>'; # </component>
	
	# spacer
	$filt .= '<td style="padding:0.2em;border:0;background:white"></td>';
	
	# "Show" filter
	$filt .= '<td style="border-right:0"><b>Show:</b></td>'
		.'<td>'
		.'<select name="filt_show" id="filt_show" onchange="onFilter()" style="width: 12.5em;">'.$nl;
	my @filt_show_values = ( ""=>"All", "problems"=>"Compatibility problems",
			"unneeded"=>"Unneeded libraries", "uncommon"=>"Uncommon libraries",
			"non-std"=>"Non-$appchecker_branch{'STD'}", "deprecated"=>"Deprecated" ); # yes, this IS a LIST.
	while ( @filt_show_values ) {
		my $key = shift @filt_show_values;
		my $val = shift @filt_show_values;
		my $selected = "";
		if ( $filt_show && $key eq $filt_show ) {
			$selected = ' selected="selected"'; 
		}
		$filt .= '<option value="'.esc_html($key).'"'
					.$selected.'>'.esc_html($val).'</option>'.$nl;
	}
	$filt .= '</select>'.$nl
		.'</td>'; # </show filter>
	
	# "Apply" button
	#$filt .= '<td style="padding:0.2em;border:0;background:white"></td>';
	#$filt .= '<td onclick="return onFilter()" style="cursor:pointer"><b>Apply &gt;</b></td>';
	
	# TODO: highlight changed filters
	
	$filt .= '</tr></table>';
	$filt .= '</td>';

	# Filter status
	my $all_n = scalar keys %{$nonfilt_info->{LIBS}};
	$filt .= '<td style="padding-left:20pt">';
	$filt .= '<span id="lines_count">';
	if ( $filtered ) {
		$filt .= '<span style="color:red"><b>'.$row_n.'</b></span> of ';
	}
	$filt .= '</span>';
	$filt .= $all_n.' librar'.($all_n!=1?'ies':'y').' shown.';
	$filt .= '</td></tr></table>'; # </filters>
	$filt .= '</form>';
	
	## Ajax stuff
	$res .= show_error_dlg(''); # placeholder for an error box
	
	$res .= ajax_wait(); # ajax hourglass

	my $filt_com_js = esc_js($filt_com);
	my $filt_show_js = esc_js($filt_show);
	my $ajax_link_js = esc_js('details='.esc_url($_GET{'details'}));
	my $details_js = esc_js($_GET{"details"});
	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[

timeout_show_progress = 0;

var cur_lfc = '$filt_com_js';
var cur_lfs = '$filt_show_js';

function onFilter() {
	var filt_com = document.getElementById('filt_com');
	var filt_show = document.getElementById('filt_show');
	
	if ( filt_com ) {
		cur_lfc = filt_com.value;
		setCookie('lfc', cur_lfc, null);
		filt_com.blur();
	}
	if ( filt_show ) {
		cur_lfs = filt_show.value;
		setCookie('lfs', cur_lfs, null);
		filt_show.blur();
	}
	setCookie('lf_for', '$details_js', null);
	
	var l_table_wrap = document.getElementById('l_table_wrap');
	
	if ( l_table_wrap ) {
		if ( ajax_req_busy) { onAbort(); }
		ajax_call_get('action=results_table&table=libs&$ajax_link_js');
	} else {
		setTimeout("onFilter()", 500);
	}
	
	return false;
}

function ajaxProcessResult(responseXML) {
	
	var l_table_html;
	if (responseXML.getElementsByTagName('l_table').length > 0) {
		l_table_html = responseXML.getElementsByTagName('l_table')[0].childNodes[0].nodeValue;
	}
	
	var l_table_wrap = document.getElementById('l_table_wrap');
	
	if ( l_table_html && l_table_wrap ) {
		l_table_wrap.innerHTML = l_table_html;
	} else {
		return -1;
	}
	
	var lines_count = document.getElementById('lines_count');
	if ( lines_count ) {
		var row_n;
		if (responseXML.getElementsByTagName('row_n').length > 0) {
			row_n = responseXML.getElementsByTagName('row_n')[0].childNodes[0].nodeValue;
			lines_count.innerHTML = '<span style="color:red"><b>'+row_n+'</b></span> of&nbsp;';
		} else {
			lines_count.innerHTML = '';
		}
	}
}

// Cookies on focus:
if (typeof(document.addEventListener) != 'undefined') {
 document.addEventListener('focus', doFocus, false); 
} else if (typeof(window.attachEvent) != 'undefined') {
 window.attachEvent('onfocus', doFocus);
}

function doFocus(event) {
	setCookie('ds', cur_ds, null);
	setCookie('lfc', cur_lfc, null);
	setCookie('lfs', cur_lfs, null);
	setCookie('lf_for', '$details_js', null);
}

// Force AJAX initialization
ajaxInit();

// ]]>
</script>
HEREDOC
	
	## Put the filters
	$res .= $filt;
	
	## Put the table
	$res .= '<div id="l_table_wrap">'.$nl;
	$res .= $table;
	$res .= '</div>'.$nl;
	##
	
	return $res;
}
#-----------------------------------------------------------------------

sub L_table {
	my ($libs, $selected_lsb_ver, $distr_sel, $filt_com, $filt_show) = @_;
	
	my $filt_distr = scalar grep {!$_} values %$distr_sel;
	my $filtered = $filt_show || $filt_com || $filt_distr;
	
	my $table = "";
	my $nl = "\n";

	# Table Header
	$table .= '<table id="l_table" class="wikitable" width="100%">'.$nl;
	$table .= '<tr>'
		.'<th>Library</th>'
		.'<th>Dependent<br />Application Components</th>'
		.'<th>Interfaces<br />Actually Used</th>'
		.'<th>Presence in<br />'.($filt_distr ? 'Selected ' : '').'Distributions</th>'
		.'<th id="std_col_header">In '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'?</th>'
		.'<th>More Info</th>'
		.'</tr>'.$nl;
	
	my $row_n = 0; # Count the rows

	# Table Body
	foreach my $soname ( sort {lc $a cmp lc $b} keys %$libs ) {
		my $lib_obj = $libs->{$soname};

		my $row = "";
		
		$row .= '<tr>'.$nl;

		## Library
		my $navi_link = $appchecker_branch{'NAVIGATOR'}.'lib_single.php'
				.'?cmd=list-by-name&Lname='.esc_url(Analysis::soname_to_libname($soname));
		$row .= '<td><b><a href="'.esc_html($navi_link).'" target="_blank">'.esc_html($soname).'</a></b></td>'.$nl;

		# App Modules: preparation
		my $app_components_html = "";
		my $app_comp_n = 0;
		if ( $lib_obj->{COMPONENTS} ) {
			foreach my $com ( sort keys %{$lib_obj->{COMPONENTS}} ) {
				my ($disp_name, $disp_path) = parse_app_component($com);
				
				$app_components_html .= '<div class="indentwrap" title="'.esc_html($disp_path).'">'.esc_html($disp_name).'</div>';
				
				$app_comp_n++;
			}
		}
		
		## App modules expandable list
		$row .= '<td>';
		my $div_id = "cl".$row_n;
		my $plu = ( $app_comp_n > 1 ? 's' : '' );
		$row .= '<a href="" onclick="return toggle(\''.$div_id.'\')">'
					.$app_comp_n.' component'.$plu.' (&hellip;)'
				.'</a>';
		$row .= '<div id="'.$div_id.'" style="display:none">';
		$row .= $app_components_html;
		$row .= '</div>';
		$row .= '</td>'.$nl;

		# Interfaces used
		my $ints_used = 0;
		my $ints_used_more = 0;
		
		# prepare presence data, it is used when coloring number of interfaces
		my $total = scalar grep {$_} values %$distr_sel;
		my $present = 0;
		if ( $lib_obj->{DISTRS} ) {
			$present = scalar grep { $distr_sel->{$_} && $lib_obj->{DISTRS}{$_} } keys %$distr_sel;
		}
		
		if ( $lib_obj->{INTERFACES} ) {
			$ints_used = scalar keys %{$lib_obj->{INTERFACES}};
		}
		if ( $lib_obj->{INTERFACES_MORE} ) {
			$ints_used_more = scalar grep {!$lib_obj->{INTERFACES}{$_}} keys %{$lib_obj->{INTERFACES_MORE}};
		}
		$row .= '<td align="center">';
		if ( $ints_used ) {
			if ( $filt_show && $filt_show eq 'unneeded' ) { next; }
			my $link = tab_link('ints').'&filt_lib='.esc_url($soname);
			$link .= '&filt_com='.esc_url($filt_com) if $filt_com;
			$row .= '<a href="'.esc_html($link).'">'.$ints_used.'</a>'; # Blue link
		} else {
			if ( $Analysis::libs_unneeded->{$soname} ) {
				$row .= '&nbsp;&nbsp;&nbsp;&nbsp;' unless ($ints_used_more); # Add nbspaces to balance the tooltip
				$row .= '<span style="color:red"><b>'.$ints_used.'</b></span>'; # Red zero
				# Tooltip
				$row .= '&nbsp;<a href="#" onclick="javascript:show_hide_tooltip(this, \'libs_unneeded\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a>' unless ($ints_used_more);
			}
			elsif ( $Analysis::approved_libs->{$soname} ) {
				# This library is needed by another component.
				$row .= '&nbsp;&nbsp;&nbsp;&nbsp;' unless ($ints_used_more); # Add nbspaces to balance the tooltip
				$row .= '0'; # Normal zero
				# Tooltip
				$row .= '&nbsp;<a href="#" onclick="javascript:show_hide_tooltip(this, \'libs_normal_zero\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a>' unless ($ints_used_more);
			}
			else {
				if ( $filt_show && $filt_show eq 'unneeded' ) { next; }
				$row .= '<span style="color:gray">unknown</span>'; # Unknown
			}
			
			if ( $ints_used_more ) {
				my $link = tab_link('ints').'&filt_lib='.esc_url($soname.'*');
				$link .= '&filt_com='.esc_url($filt_com) if $filt_com;
				$row .= '&nbsp;(<a href="'.esc_html($link).'">'.($ints_used+$ints_used_more).'</a>';
				# Tooltip
				$row .= '&nbsp;<a href="#" onclick="javascript:show_hide_tooltip(this, \'libs_ext_ints\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a>';
				# TODO: section in the Help ^^^
				$row .= ')';
			}
		}
		$row .= '</td>'.$nl;

		## Presence in distributions

		my $bgcolor;
		if ( $present == $total ) {
			if ( $filt_show && $filt_show eq 'problems' ) { next; } # drop the row
			$bgcolor = "#CCFFCC"; # green
		} elsif ( $present > 0 ) {
			$bgcolor = "#FEFECC"; # yellow
		} else {
			$bgcolor = "#FFCCCC"; # red
		}

		$row .= '<td align="right" style="background:'.$bgcolor.'">';
		my $link = tab_link('libs').'&list='.esc_url($soname);
		my $details_link = ' (<a href="'.esc_html($link).'"'
			.' onclick="'.esc_html('return popup_win(\''.esc_js($link).'\')').'">list&hellip;</a>)';
		my $approved = $Analysis::approved_libs->{$soname};
		if ( $approved or $present == $total ) {
			if ( $filt_show && $filt_show eq 'uncommon' ) { next; } # drop the row
			$row .= $present." of ".$total;
			$row .= $details_link;
		} elsif ( $present > 0 ) {
			$row .= $present." known distro".($present > 1 ? 's' : '');
			$row .= $details_link;
		} else {
			$row .= 'Unknown library';
			# Tooltip:
			$row .= '&nbsp;<a href="tests_help.pl#libs_ints_unknown" onclick="javascript:show_hide_tooltip(this, \'libs_ints_unknown\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a>';
		}
		$row .= '</td>'.$nl;


		## LSB/Moblin status
		$row .= '<td align="center">';

		$row .= '<div>';

		my $lsb_status     = $Analysis::lsb_lib_status->{$soname}{STATUS};
		my $lsb_status_ver = $Analysis::lsb_lib_status->{$soname}{STATUS_VER};
		
		my $color = undef;

		if ( $lsb_status eq "NOT" || $lsb_status eq "WITHDRAWN" ) {
			$row .= '<span style="color:red">No</span>';
		}
		elsif ( $lsb_status eq "CANDIDATE" ) {
			if ( $lsb_status_ver eq 'Unknown' ) {
				$row .= '<span style="color:#907000">Candidate</span>';
			} else {
				$row .= '<span style="color:#907000">Candidate for '.esc_html($lsb_status_ver).'</span>';
			}
		}
		elsif ( $lsb_status eq "FUTURE" ) {
			$row .= '<span style="color:#907000">In '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'</span>';
		}
		elsif ( $lsb_status eq "OPTIONAL" ) {
			if ( $filt_show && $filt_show eq 'non-std' ) { next; } # drop the row
			$row .= '<span style="color:blue">Optional</span>';
		}
		elsif ( $lsb_status eq "IN" ) {
			if ( $filt_show && $filt_show eq 'non-std' ) { next; } # drop the row
			$row .= '<span style="color:green">Yes</span>';
		}
		elsif ( $lsb_status eq "DEPRECATED" ) {
			if ( $filt_show && $filt_show eq 'non-std' ) { next; } # drop the row
			$row .= '<span style="color:blue">Deprecated</span>';
		}
		if ( $lsb_status ne "DEPRECATED" ) {
			if ( $filt_show && $filt_show eq 'deprecated' ) { next; }
		}

		$row .= '</div>';

		$row .= '</td>'.$nl;


		## More Info

		$row .= '<td>';
		my $txt_html = 'Details&hellip;';
		if ( $Analysis::lib_attrs->{$soname} ) {
			$txt_html = '<b>More&nbsp;Info&hellip;</b>'; # MORE INFO
		}
		if ( $Analysis::libKBDistr->{$soname} ) {
			$txt_html = '<b>Known&nbsp;Issues&hellip;</b>'; # KNOWN ISSUES
		}
		my $info_link = tab_link('libs').'&lib='.esc_url($soname);
		my $info_onclick = 'onclick="'.esc_html('return popup_win(\''.esc_js($info_link).'\')').'"';
		$row .= '<a href="'.esc_html($info_link).'" '.$info_onclick.'>'.$txt_html.'</a>';
		$row .= '</td>'.$nl;

		$row .= '</tr>'.$nl;
		
		$table .= $row;
		$row_n++;
	}
	$table .= '</table>'.$nl;
	
	if ( $filtered && $row_n == 0 ) {
		$table .= '<div style="padding:10pt;text-align:center;font-size:large;color:#009">'
					.'<p>No libraries match the filter.</p>'
				.'</div>';
	}
	
	return ($table, $row_n);
}

#-----------------------------------------------------------------------

# Contents of a popup window with details on a library presence
# in a selected distribution subset.
sub lib_distr_list {
	my ($soname, $distr_sel, $filt_com) = @_;

	my $nl = "\n";
	my $res = "";

	my $libs = libs_data($distr_sel, $filt_com);
	my $lib_obj = $libs->{$soname};
	if ( !$lib_obj ) {
		$res .= "Error: there is no such library: '".esc_html($soname)."'";
		return $res;
	}
	
	my $navi_link = $appchecker_branch{'NAVIGATOR'}.'lib_single.php'
			.'?cmd=list-by-name&Lname='.esc_url(Analysis::soname_to_libname($soname));
	# Keep synchronized with lib_details()
	$res .= '<h1>Library <span class="title-highlight">'.esc_html($soname).'</span></h1>';
	$res .= '<p>Look up the <a href="'.esc_html($navi_link).'" target="_blank"><b>'.esc_html($soname).'</b> library in the '.$appchecker_branch{'STD'}.' Navigator</a>.</p>';
	
	$res .= '<hr />'.$nl;

	$res .= tooltip();
	
	# Presence in Distributions
	$res .= '<h2>Presence in Distributions</h2>'.$nl;

	my $filtered = scalar grep {!$_} values %$distr_sel;
	my $approved = $Analysis::approved_libs->{$soname};
	
	my $total_num = scalar grep {$_} values %$distr_sel;
	my $present_num = scalar grep {$_} values %{$lib_obj->{DISTRS}};
	my $missing_num = $total_num - $present_num;

	if ( $present_num > 0 ) {
		$res .= '<p>';
		$res .= 'Present in <span style="color:green">'.$present_num.'</span>';
		if ( $approved or $missing_num == 0) {
			$res .= ' and missing in <span style="color:red">'.$missing_num.'</span>';
		} else {
			$res .= ' and nothing is known about <span style="color:#907000">'.$missing_num.'</span>'; # yellow
		}
		if ( $filtered ) {
			$res .= ' of '.$total_num.' selected distributions.';
		} else {
			$res .= ' of total '.$total_num.' distributions.';
		}
		$res .= '</p>'.$nl;
	} else {
		if ( $approved ) {
			$res .= '<p>This library is missing in all the '.$total_num;
			if ( $filtered ) {
				$res .= ' selected distributions.</p>'.$nl;
			} else {
				$res .= ' analyzed distributions.</p>'.$nl;
			}
		} else {
			$res .= '<p>There is no information about this library in the database.</p>'.$nl;
		}
	}

	local *distr_list = sub {
		my ($color, @distrs) = @_;

		my $res = "";

		my $all_distrs = $Analysis::all_distrs;
		@distrs = sort {lc $all_distrs->{$a}{FULLNAME} cmp lc $all_distrs->{$b}{FULLNAME}} @distrs;
		$res .= '<ul>'.$nl;
		for my $Did ( @distrs ) {
			my $link = tab_link('distrs').'&id='.$Did;
			
			$res .= '<li><a style="color:'.$color.'" href="'.esc_html($link).'">'
						.esc_html($all_distrs->{$Did}{FULLNAME}).'</a>';
			if ( $color ne 'green' && (my $alts = $Analysis::alt_libs->{$Did}{$soname}) ) {
				$res .= ' &nbsp; &nbsp; &nbsp; <i>(see <a href="#" onclick="javascript:show_hide_tooltip(this, \''.$Did.'\', event); return false;" >alternatives</a>)</i>';
				$res .= altlib_tooltip_entry($alts, $soname, $Did, $Did);
			}
			$res .= '</li>'.$nl;
		}
		$res .= '</ul>'.$nl;

		return $res;
	};

	if ( $present_num ) {
		$res .= '<h3>Present in:</h3>'.$nl;

		my @present_distrs = grep { $distr_sel->{$_} && $lib_obj->{DISTRS}{$_} } keys %$distr_sel;
		
		$res .= distr_list('green', @present_distrs);
	}

	if ( $missing_num ) {
		my @missing_distrs = grep { $distr_sel->{$_} && !$lib_obj->{DISTRS}{$_} } keys %$distr_sel;
		if ( $approved ) {
			$res .= '<h3>Missing in:</h3>'.$nl;
			
			$res .= distr_list('red', @missing_distrs);
		} else {
			$res .= '<h3>No information for:</h3>'.$nl;

			$res .= distr_list('#907000', @missing_distrs); # yellow
		}
	}
	$res .= '<hr />'.$nl;
	
	return $res;
}

# Contents of a details popup window for Libraries table
sub lib_details {
	my ($soname, $selected_lsb_ver) = @_;

	my $nl = "\n";
	my $res = "";

	my $libs = libs_data();
	my $lib_obj = $libs->{$soname};
	
	if ( !$lib_obj ) {
		$res .= "Error: there is no such library: '".esc_html($soname)."'";
		return $res;
	}
	
	# Title
	my $navi_link = $appchecker_branch{'NAVIGATOR'}.'lib_single.php'
			.'?cmd=list-by-name&Lname='.esc_url(Analysis::soname_to_libname($soname));

	# Keep synchronized with lib_distr_list()
	$res .= '<h1>Library <span class="title-highlight">'.esc_html($soname).'</span></h1>';
	$res .= '<p>Look up the <a href="'.esc_html($navi_link).'" target="_blank"><b>'.esc_html($soname).'</b> library in the '.$appchecker_branch{'STD'}.' Navigator</a>.</p>';

	$res .= '<hr/>'.$nl;

	# Check Lib Knowledge Base
	my $kb_info = lib_KB_info($soname); # HTML data
	if ( $kb_info ) {
		$res .= '<h2>Known compatibility issues:</h2>'.$nl;
		$res .= '<p>'.$kb_info.'</p>'.$nl;

		$res .= '<hr />'.$nl;
	}

	# Check LibAttr
	my $attr = $Analysis::lib_attrs->{$soname};
	if ( $attr ) {
		foreach my $type ( sort keys %$attr ) {
			my $attr_html = "";
			my $bgcolor = '#fec';
			if ( $type eq "Recommendation" ) {
				$attr_html .= '<h3>Recommendation</h3>';
				$attr_html .= esc_html($attr->{$type});
			}
			elsif ( $type eq "StaticLinkage" ) {
				$attr_html .= '<h3>May be linked statically</h3>';
				$attr_html .= esc_html($attr->{$type});
			}
			elsif ( $type eq "Rejection" ) {
				$attr_html .= '<h3>Rejection</h3>';
				$attr_html .= esc_html($attr->{$type});
			}
			else { # Custom
				$attr_html .= esc_html($attr->{$type});
			}
			$attr_html =~ s/^\s+//; # remove unnecessary spaces
			$attr_html =~ s/\s+$//; #
			$attr_html .= "." if $attr_html =~ /[^\s\.]\s*$/; # Add a fullstop if necessary.
			
			$res .= '<div class="box" style="background:'.$bgcolor.'">'.$attr_html.'</div>';
		}

		$res .= '<hr />'.$nl;
	}

	# LSB/Moblin Status
	$res .= '<h2>'.$appchecker_branch{'STD'}.' Status:</h2>'.$nl;

	my $lsb_status = $Analysis::lsb_lib_status->{$soname}{STATUS};
	my $lsb_status_ver = $Analysis::lsb_lib_status->{$soname}{STATUS_VER};

	if ( $lsb_status eq "NOT" ) {
		$res .= '<div style="color:red">Never been in '.$appchecker_branch{'STD'}.'.</div>'.$nl;
	}
	elsif ( $lsb_status eq "FUTURE" ) {
		$res .= '<div style="color:red">Not in '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'. (Appeared in '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).')</div>'.$nl;
	}
	elsif ( $lsb_status eq "CANDIDATE" ) {
		$res .= '<div style="color:red">Not in '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'.';
		if ( $lsb_status_ver eq 'Unknown' || $lsb_status_ver eq '' ) {
			$res .= ' Candidate for future '.$appchecker_branch{'STD'}.' version.';
		} else {
			$res .= ' Candidate for '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.';
		}
		$res .= '</div>'.$nl;
	}
	elsif ( $lsb_status eq "OPTIONAL" ) {
		$res .= '<div style="color:blue">Optional since '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.</div>'.$nl;
	}
	elsif ( $lsb_status eq "IN" ) {
		$res .= '<div style="color:green">Mandatory since '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.</div>'.$nl;
	}
	elsif ( $lsb_status eq "DEPRECATED" ) {
		$res .= '<div style="color:blue">Deprecated since '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.</div>'.$nl;
	}
	elsif ( $lsb_status eq "WITHDRAWN" ) {
		$res .= '<div style="color:red">Withdrawn since '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.</div>'.$nl;
	}

	$res .= '<hr />'.$nl;

	$res .= '<h2>This library is used by the following aplication components:</h2>';

	$res .= '<ul>';
	foreach my $com ( sort keys %{$lib_obj->{COMPONENTS}} ) {
		my ($disp_name, $disp_path) = parse_app_component($com);
		$res .= '<li>';
		my $com_link = tab_link('com').'&com='.esc_url($com);
		$res .= '<a href="'.esc_html($com_link).'"'
			.' title="'.esc_html($disp_path).'"><b>'.esc_html($disp_name).'</b></a>';
		$res .= '</li>';
	}
	$res .= '</ul>';

	
	$res .= '<hr />'.$nl;

	return $res;
}

# Returns information from the Knowledge Base for specified library and
#  distribution set.
sub lib_KB_info {
	my ($soname) = @_;

	my $nl = "\n";
	my $res = "";

	my $problems = $Analysis::libKBDistr->{$soname};

	return undef if not $problems;

	foreach my $LKBid ( sort {$a <=> $b} keys %$problems ) {
		my $text = $Analysis::libKB->[$LKBid];
		$text =~ s/\n/<br\/>\n/g;

		$res .= '<div>';
		$res .= "<p>";
		$res .= esc_html($text);
		$res .= "</p>".$nl;

		my $grouped = $problems->{$LKBid};

		foreach my $i ( sort {$a <=> $b} keys %$grouped ) {
			$res .= '<div><b>Group '.$i.':</b> ';

			# Print IDs of distributions from this group
			$res .= join ", ", map {esc_html($Analysis::all_distrs->{$_}{FULLNAME})} @{$grouped->{$i}};

			$res .= '.</div>'.$nl;
		}

		$res .= '</div>'
	}

	return $res;
}

# Distribution compatibility table
sub D_tab {
	my ($selected_lsb_ver) = @_;
	
	my $res = "";
	my $nl = "\n";
	
	# Check if the table is empty
	if ( !%$Analysis::app_components ) {
		$res .= no_binary_div();
		return $res;
	}
	
	my $link = tab_link('distrs');
	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[

function popup_win( link )
{
	var w = window.open( link, 'details',
			'toolbar=0,menubar=0,scrollbars=1,height=750,width=850,dependent=yes,dialog=yes'
		);
	w.moveTo(100,0);
	w.focus();

	return false;
}
// ]]>
</script>
HEREDOC

	$res .= <<DATA;
<div id="distro-details" style="position: absolute; border: 1px #aaaaaa solid; border-left: none; background: white; padding: 0px 2px; vertical-align: bottom; display: none;">
</div>

<p>The table below shows the compatibility status of your application with the distributions analyzed by the Linux Foundation. Your Application will run on the "<font color="green">green</font>" distributions without loader problems. Compatibility with the "<font color="#c0a000">yellow</font>" distributions can be easily achieved by excluding unneeded libraries from the dependencies of your application. Making the Application compatible with the "<font color="red">red</font>" distributions may require more effort to avoid using missing libraries/interfaces or by supplying them as a part of your application package. Please note that functional correctness is not guaranteed by this analysis.</p>
DATA

	my $ext_table_display = ( $_GET{'expand'} ? '' : 'none' );
	my $ext_table = <<DATA;
<table class="wikitable" cellpadding="0" id="d_table_ext" style="display: $ext_table_display; margin: 0; border-left: none;">
<tr><th colspan="2" style="border-left: none;">Missing&nbsp;Libraries</th><th colspan="2" style="">Missing&nbsp;Interfaces</th><th rowspan="2">Comments</th></tr>
<tr>
  <th style="width: 5em; border-left: none;"><table cellpadding="0" cellspacing="0" border="0" width="100%"><tr><td align="center" style="border: none; white-space: nowrap;">Common&nbsp;<a href="tests_help.pl#libs_ints_common" onclick="javascript:show_hide_tooltip(this, 'libs_ints_common', event); return false;"><img src="themes/$appchecker_branch{'DIR'}/images/question.png" width="16" height="16" alt="?" /></a></td></tr></table></th>
  <th style="width: 5em; white-space: nowrap;">Unknown&nbsp;<a href="tests_help.pl#libs_ints_unknown" onclick="javascript:show_hide_tooltip(this, 'libs_ints_unknown', event); return false;"><img src="themes/$appchecker_branch{'DIR'}/images/question.png" width="16" height="16" alt="?" /></a></th>
  <th style="width: 5em; white-space: nowrap;">Common&nbsp;<a href="tests_help.pl#libs_ints_common" onclick="javascript:show_hide_tooltip(this, 'libs_ints_common', event); return false;"><img src="themes/$appchecker_branch{'DIR'}/images/question.png" width="16" height="16" alt="?" /></a></th>
  <th style="width: 5em; white-space: nowrap;">Unknown&nbsp;<a href="tests_help.pl#libs_ints_unknown" onclick="javascript:show_hide_tooltip(this, 'libs_ints_unknown', event); return false;"><img src="themes/$appchecker_branch{'DIR'}/images/question.png" width="16" height="16" alt="?" /></a></th>
</tr>
DATA

	my $general_table = <<DATA;
<table class="wikitable" id="d_table" style="margin: 0;">
<tr><th colspan="2">Summary</th></tr>
<tr><th>Distribution</th><th style="width: 10em;"><table cellpadding="0" cellspacing="0" border="0"><tr><td align="left" style="border: none;"><img src="images/environment-spacer.gif" width="17" height="17" alt="" /></td><td width="100%" style="border: none;">Status</td><td align="right" style="border: none;"><img src="images/status_expand.png" width="16" height="16" alt="[&gt;&gt;]" id="expand-img" style="cursor: pointer;" onclick="javascript:toggle_table();" /></td></tr></table></th></tr>
DATA

	my $distrs = {};
	foreach my $Did ( keys %{$Analysis::all_distrs} ) {
		$distrs->{$Did} = Analysis::distr_data($Did);
	}
	
	local *by_status = sub {
		my ($a, $b) = @_;
		
		return (
			$Analysis::status_range{$a->{STATUS}} <=> $Analysis::status_range{$b->{STATUS}}
			or $a->{MISSING_LIBS}   <=> $b->{MISSING_LIBS}
			or $a->{MISSING_INTS}   <=> $b->{MISSING_INTS}
			or $a->{UNNEEDED_LIBS} <=> $b->{UNNEEDED_LIBS}
			or $a->{UNKNOWN_LIBS}  <=> $b->{UNKNOWN_LIBS}
			or $a->{UNKNOWN_INTS}  <=> $b->{UNKNOWN_INTS}
			or lc $a->{FULLNAME} cmp lc $b->{FULLNAME}
		);
	};
	
	# Distributions list
	foreach my $distr ( sort {by_status($a,$b)} values %$distrs ) {
		my $Did = $distr->{ID};
		my $status = $distr->{STATUS};

		my $bgcolor;
		my $classname;
		if ( $status eq 'MISSING' ) {
			$bgcolor = "#ffdddd"; # red
			$classname = 'fail-td';
			$status = 'INCOMPATIBLE';
		}
		elsif ( $status eq 'UNKNOWN' || $status eq 'UNNEEDED' ) {
			$bgcolor = "#fefecc"; # yellow
			$classname = 'warn-td';
			$status = 'WARNINGS';
		}
		else {
			$bgcolor = "#ccffcc"; # green
			$classname = 'pass-td';
			$status = 'OK';
		}
		
		my $missing_libs   = $distr->{MISSING_LIBS};
		my $unneeded_libs = $distr->{UNNEEDED_LIBS};
		my $unknown_libs  = $distr->{UNKNOWN_LIBS};
		
		my $missing_ints  = $distr->{MISSING_INTS};
		my $unknown_ints = $distr->{UNKNOWN_INTS};
		
		# Distribution
		my $Dname_html = esc_html($distr->{FULLNAME});
		my $Dname_link = esc_url($distr->{NAME});
		$general_table .= <<DATA;
<tr id="dtr_$Did" style="background: $bgcolor;">
<td style="white-space: nowrap;"><b><a href="$appchecker_branch{'NAVIGATOR'}distr.php?cmd=list-byname&amp;Dname=$Dname_link" target="_blank">$Dname_html</a></b></td>
<td align="center" class="$classname" onmouseover="javascript:show_info(this, $missing_libs, $unneeded_libs, $missing_ints, $unknown_libs, $unknown_ints);" onmouseout="javascript:hide_info();" onclick="javascript:toggle_table();"><b>$status</b></td>
</tr>
DATA

		$ext_table .= '<tr style="background: '.$bgcolor.';">';

		# Missing libraries
		if ($missing_libs + $unneeded_libs == 0) {
			$ext_table .= '<td style="border-left: none; text-align: center; color: gray;">none</td>';
		}
		else {
			my $unneeded_libs_html = ($unneeded_libs ? ' (+<font color="red"><b>'.$unneeded_libs.'</b></font> unneeded)' : '');
			my $missing_libs_html = ($missing_libs ? '<font color="red"><b>'.$missing_libs.'</b></font>' : '0');
			my $link = tab_link('distrs').'&id='.$Did.'&show=missing_libs';
			$ext_table .= '<td style="white-space: nowrap; border-left: none;">'
				.$missing_libs_html.$unneeded_libs_html
				.' (<a href="'.esc_html($link).'" onclick="'.esc_html('return popup_win(\''.esc_js($link).'\')').'">list&hellip;</a>)'
				.'</td>';
		}

		# Unknown libraries
		if ($unknown_libs == 0) {
			$ext_table .= '<td style="border-left: none; text-align: center; color: gray;">none</td>';
		}
		else {
			my $link = tab_link('distrs').'&id='.$Did.'&show=unknown_libs';
			$ext_table .= '<td style="white-space: nowrap; border-left: none;">'
				.'<font color="red"><b>'.$unknown_libs.'</b></font>'
				.' (<a href="'.esc_html($link).'" onclick="'.esc_html('return popup_win(\''.esc_js($link).'\')').'">list&hellip;</a>)'
				.'</td>';
		}

		# Missing interfaces
		if ($missing_ints == 0) {
			$ext_table .= '<td style="border-left: none; text-align: center; color: gray;">none</td>';
		}
		else {
			my $link = tab_link('distrs').'&id='.$Did.'&show=missing_ints';
			$ext_table .= '<td style="white-space: nowrap; border-left: none;">'
				.'<font color="red"><b>'.$missing_ints.'</b></font>'
				.' (<a href="'.esc_html($link).'" onclick="'.esc_html('return popup_win(\''.esc_js($link).'\')').'">list&hellip;</a>)'
				.'</td>';
		}

		# Unknown interfaces
		if ($unknown_ints == 0) {
			$ext_table .= '<td style="border-left: none; text-align: center; color: gray;">none</td>';
		}
		else {
			my $link = tab_link('distrs').'&id='.$Did.'&show=unknown_ints';
			$ext_table .= '<td style="white-space: nowrap; border-left: none;">'
				.'<font color="red"><b>'.$unknown_ints.'</b></font>'
				.' (<a href="'.esc_html($link).'" onclick="'.esc_html('return popup_win(\''.esc_js($link).'\')').'">list&hellip;</a>)'
				.'</td>';
		}
		
		# Comments
		my $comments = '';
		if ( ($appchecker_branch{'STD'} eq 'LSB') && $distr->{COMMENTS} && $distr->{COMMENTS}{NO_GNU_HASH} ) {
			$comments = '<span style="white-space:nowrap">No .gnu.hash support <a href="" onclick="javascript:show_hide_tooltip(this, \'com_no_hash_section\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a></span>';
		}
		$ext_table .= '<td>'.$comments.'</td>'.$nl;
		
		$ext_table .= '</tr>'.$nl;
	}
	$general_table .= '</table>';
	$ext_table .= '</table>';

	$res .= <<HEREDOC;
<table cellpadding="0" cellspacing="0" border="0" style="margin: 12px 0;">
<tr>
<td>
$general_table
</td>
<td>
$ext_table
</td>
</tr>
</table>

<script type="text/javascript" language="javascript">
// <![CDATA[

// Determines absolute dynamic position of an element (right top corner).
function getPosition(elem) {
	if (!elem)
		return {x: -1, y: -1};
	var x = elem.offsetWidth;
	var y = 0;
	while (elem) {
		x += elem.offsetLeft;
		y += elem.offsetTop;
		elem = elem.offsetParent;
	}
	return {x: x, y: y};
}

var distro_details = document.getElementById('distro-details');
var table = document.getElementById('d_table');
var table_ext = document.getElementById('d_table_ext');
var expand_img = document.getElementById('expand-img');

// Fine-tuning for browsers
var dDetailsOffsetX = ((navigator.appName == 'Netscape') ? 0 : 1);
if (navigator.appName == 'Konqueror') {
	table_ext.style.borderLeft = 'solid 1px #aaaaaa';
}

function show_info(td_elem, missing_libs, unneeded_libs, missing_ints, unknown_libs, unknown_ints) {
	if (table_ext.style.display != 'none')
		return;
	var details = '';

	if (missing_libs + unneeded_libs + missing_ints > 0) {
		var tmp = ''
		if (missing_libs + unneeded_libs > 0) {
			tmp += (missing_libs ? ('<font color="red"><b>' + missing_libs + '</b></font> ' + ((missing_libs > 1) ? 'libraries' : 'library')) : '0 libraries') +
			       (unneeded_libs ? ' (+<font color="red"><b>' + unneeded_libs + '</b></font> unneeded)' : '');
		}
		if (missing_ints > 0) {
			tmp += (tmp == '' ? '' : '; ') +
			       (missing_ints ? ('<font color="red"><b>' + missing_ints + '</b></font> ' + ((missing_ints > 1) ? 'interfaces' : 'interface')) : '0 interfaces');
		}
		details += 'Missing: ' + tmp + '.';
	}
	if (unknown_libs + unknown_ints > 0) {
		var tmp = ''
		if (unknown_libs > 0) {
			tmp += (unknown_libs ? ('<font color="red"><b>' + unknown_libs + '</b></font> ' + ((unknown_libs > 1) ? 'libraries' : 'library')) : '0 libraries');
		}
		if (unknown_ints > 0) {
			tmp += (tmp == '' ? '' : '; ') +
			       (unknown_ints ? ('<font color="red"><b>' + unknown_ints + '</b></font> ' + ((unknown_ints > 1) ? 'interfaces' : 'interface')) : '0 interfaces');
		}
		details += (details == '' ? '' : ' ') + 'Unknown: ' + tmp + '.';
	}
	if (details == '')
		details = '<i>No problems detected.</i>';
	distro_details.innerHTML = details;
	var pos = getPosition(td_elem);
	distro_details.style.left = (pos.x + dDetailsOffsetX) + 'px';
	distro_details.style.top = pos.y + 'px';
	distro_details.style.height = (td_elem.offsetHeight - 1) + 'px';
	distro_details.style.display = '';
}

function hide_info() {
	distro_details.style.display = 'none';
}

function toggle_table() {
	hide_info();
	if (table_ext.style.display == 'none') {
		expand_img.setAttribute('src', 'images/status_collapse.png');
		expand_img.setAttribute('alt', '[&lt;&lt;]');
		table_ext.style.display = '';
	}
	else {
		expand_img.setAttribute('src', 'images/status_expand.png');
		expand_img.setAttribute('alt', '[&gt;&gt;]');
		table_ext.style.display = 'none';
	}
}
// ]]>
</script>
HEREDOC

	return $res;
} # end of  D_tab
#-----------------------------------------------------------------------

# Contents of a detail popup window for Distributions table
sub distr_details {
	my ($Did, $show) = @_;

	my $nl = "\n";
	my $res = "";

	my $distr = Analysis::distr_data($Did);
	if ( !$distr ) {
		$res .= "There is no distribution with ID '$Did'.";
		return $res;
	}
	
	# The header
	my $navi_link = $appchecker_branch{'NAVIGATOR'}.'distr.php'
			.'?cmd=list-byname&Dname='.esc_url($distr->{NAME});
	$res .= '<h1>Distribution <span class="title-highlight">'.esc_html($distr->{FULLNAME}).'</span></h1>'.$nl;
	$res .= '<p>Look up <a href="'.esc_html($navi_link).'" target="_blank"><b>'.esc_html($distr->{FULLNAME}).'</b> in the '.$appchecker_branch{'STD'}.' Navigator</a>.</p>'.$nl;
	$res .= '<hr/>';
	
	if ( $appchecker_branch{'STD'} eq 'LSB' ) {
		if ( !$show || $show eq 'other' ) {
			if ( $distr->{NO_GNU_HASH} ) {
				if ( grep { $_->{NO_SHT_HASH} } values %$Analysis::app_components ) {
					$res .= '<h2>Section .gnu.hash is not supported</h2>'.$nl;
					$res .= '<p>Binaries targeting this distribution must use the \'.hash\' section instead.</p>'.$nl;
					$res .= '<hr/>';
				}
			}
		}
	}
	
	# App components
	if ( %$Analysis::app_components && (!$show || $show eq 'components') ) {
		$res .= '<h2>Application components:</h2>'.$nl;
		$res .= '<p>The table below shows the compatibility information for application components with regard to '.esc_html($distr->{FULLNAME}).'.</p>'.$nl;
		$res .= '<table style="border-collapse:collapse" border="1">'.$nl;
		$res .= '<tr>'
				.'<th rowspan="1">Application component</th>'
				.'<th colspan="2">Libraries</th>'
				.'<th colspan="2">Interfaces</th>'
			.'</tr>'.$nl;
		$res .= '<tr>'
				.'<th></th>'
				.'<th>Missing</th>'
				.'<th>Unknown</th>'
				.'<th>Missing</th>'
				.'<th>Unknown</th>'
			.'</tr>'.$nl;
		foreach my $com ( sort keys %$Analysis::app_components ) {
			my ($disp_name, $disp_path) = parse_app_component($com);
			my $com_obj = $Analysis::app_components->{$com};
			Analysis::com_distr_data($com_obj, $Did, $com_obj->{LIBS_UNNEEDED});
			my $status = Analysis::com_distr_status($com_obj, $Did);
			my $bgcolor = "";
			if ( $status eq 'OK' ) {
				$bgcolor = "#ccffcc"; # green
			} elsif ( $status eq 'UNNEEDED' ) {
				$bgcolor = "#fefecc"; # yellow
			} elsif ( $status eq 'UNKNOWN' ) {
				$bgcolor = "#fefecc"; # yellow
			} elsif ( $status eq 'MISSING' ) {
				$bgcolor = "#ffdddd"; # red
			}
			my $link = tab_link('com').'&com='.esc_url($com).'&distr='.$Did;
			
			# Component Info
			my ($missing_libs, $unneeded_libs, $unknown_libs, $missing_ints, $unknown_ints) = (0) x 5;
			foreach my $req_soname ( @{$com_obj->{NEEDED_LIBS}} ) {
				my $val = $com_obj->{$Did}{LIBS}{$req_soname}{STATUS};
				if ( $val eq 'MISSING' ) { $missing_libs++; }
				if ( $val eq 'UNNEEDED' ) { $unneeded_libs++; }
				if ( $val eq 'UNKNOWN' ) { $unknown_libs++; }
			}
			foreach my $val ( values %{$com_obj->{$Did}{INT_STATUS}} ) {
				if ( $val eq 'MISSING' ) { $missing_ints++ }
				if ( $val eq 'UNKNOWN' ) { $unknown_ints++; }
			}
			
			$res .= '<tr style="background:'.$bgcolor.'">';
			$res .= '<td><a href="'.esc_html($link).'" title="'.esc_html($disp_path).'">'.esc_html($disp_name).'</a></td>';
			
			$res .= '<td>';
			if ( $missing_libs ) {
				$res .= '<span style="color:red">';
				$res .= $missing_libs.($unneeded_libs ? ' (+'.$unneeded_libs.' unneeded)' : '');
				$res .= '</span>';
			}
			elsif ( $unneeded_libs ) {
				$res .= '<span style="color:#907000">';
				$res .= $unneeded_libs.' unneeded';
				$res .= '</span>';
			}
			$res .= '</td>';
			$res .= '<td>';
			if ( $unknown_libs ) {
				$res .= '<span style="color:#907000">'.$unknown_libs.'</span>';
			}
			$res .= '</td>';
			$res .= '<td>';
			if ( $missing_ints ) {
				$res .= '<span style="color:red">'.$missing_ints.'</span>';
			}
			$res .= '</td>';
			$res .= '<td>';
			if ( $unknown_ints ) {
				$res .= '<span style="color:#907000">'.$unknown_ints.'</span>';
			}
			$res .= '</td>';
			$res .= '</tr>'.$nl;
		}
		$res .= '</table>'.$nl;
		$res .= '<hr/>'.$nl;
	}

	$res .= tooltip();

	# Libraries
	local *lib_list = sub {
		my ($color, @sonames) = @_;

		my $res = "";

		$res .= '<ul>'.$nl;
		foreach my $soname ( sort {lc $a cmp lc $b} @sonames ) {
			#my $link = tab_link('libs').'&lib='.esc_url($soname);
			$res .= '<li>';
			# Not a link, because the link won't be useful here
			$res .= '<span style="color:'.esc_html($color).'">'.esc_html($soname).'</span>';
			if ( $color ne 'green' && (my $alts = $Analysis::alt_libs->{$Did}{$soname}) ) {
				$res .= ' &nbsp; &nbsp; &nbsp; <i>(see <a href="#" onclick="javascript:show_hide_tooltip(this, \''.$soname.'\', event); return false;" >alternatives</a>)</i>';
				$res .= altlib_tooltip_entry($alts, $soname, $Did, $soname);
			}
			$res .= '</li>'.$nl;
		}
		$res .= '</ul>'.$nl;

		return $res;
	};
	
	my $hr_after_libs = 0;
	
	if ( $distr->{MISSING_LIBS} && (!$show || $show eq 'missing_libs') ) {
		$res .= '<h2>Missing Libraries:</h2>'.$nl;

		$res .= lib_list( 'red', grep {$distr->{LIBS}{$_} eq 'MISSING'} keys %{$distr->{LIBS}} );
		$hr_after_libs = 1;
	}

	if ( $distr->{UNKNOWN_LIBS} && (!$show || $show eq 'unknown_libs') ) {
		$res .= '<h2>No Information About:</h2>'.$nl;
		$res .= '<p>The Application Checker Database contains no information'
			.' about the following libraries:</p>'.$nl;
		# yellow
		$res .= lib_list('#907000', grep {$distr->{LIBS}{$_} eq 'UNKNOWN'} keys %{$distr->{LIBS}});
		$hr_after_libs = 1;
	}
	
	if ( $distr->{UNNEEDED_LIBS} && (!$show || $show eq 'unneeded_libs' || $show eq 'missing_libs') ) {
		$res .= '<h2>Unnecessary Libraries:</h2>'.$nl;
		$res .= '<p>These are libraries required by the Application,'
			.' but <b>no interfaces</b> from these libraries are actually used:</p>'.$nl;
		# yellow
		$res .= lib_list('#907000', grep {$distr->{LIBS}{$_} eq 'UNNEEDED'} keys %{$distr->{LIBS}});
		$hr_after_libs = 1;
	}
	
	if ( $distr->{OK_LIBS} && (!$show || $show eq 'present_libs') ) {
		$res .= '<h2>Present Libraries:</h2>'.$nl;

		$res .= lib_list('green', grep {$distr->{LIBS}{$_} eq 'OK'} keys %{$distr->{LIBS}});
		$hr_after_libs = 1;
	}

	$res .= '<hr/>' if $hr_after_libs;
	
	local *int_list = sub {
		my ($color, @fullnames) = @_;

		my $res = "";

		$res .= '<ul>'.$nl;
		foreach my $fullname ( sort {lc $a cmp lc $b} @fullnames ) {
			my ($Iname, $ver) = split /\@/, $fullname, 2;
			my $unmangled = $Analysis::unmangled->{$Iname} || $Iname;
			$res .= '<li>';
			# Not a link, because the link won't be useful here
			$res .= '<span style="color:'.esc_html($color).'">';
			$res .= esc_html($unmangled);
			$res .= '&nbsp;@'.esc_html($ver) if $ver;
			$res .= '</span>';
			$res .= '</li>'.$nl;
		}
		$res .= '</ul>'.$nl;

		return $res;
	};

	my $hr_after_ints = 0;
	
	if ( $distr->{MISSING_INTS} && (!$show || $show eq 'missing_ints') ) {
		$res .= '<h2>Missing Interfaces:</h2>'.$nl;

		$res .= int_list('red', grep {$distr->{INTS}{$_} eq 'MISSING'} keys %{$distr->{INTS}});
		$hr_after_ints = 1;
	}
	
	if ( $distr->{UNKNOWN_INTS} && (!$show || $show eq 'unknown_ints') ) {
		$res .= '<h2>No Information About:</h2>'.$nl;
		$res .= '<p>The Application Checker Database contains no information'
			.' about the following interfaces:</p>'.$nl;
		# yellow
		$res .= int_list('#907000', grep {$distr->{INTS}{$_} eq 'UNKNOWN'} keys %{$distr->{INTS}});
		$hr_after_ints = 1;
	}
	# OK_INTS are not shown
	
	$res .= '<hr/>' if $hr_after_ints;
	
	if ( $show ) { # if filtered...
		my $link = tab_link('distrs').'&id='.$Did;
		$res .= '<p><a href="'.esc_html($link).'"><i>More details&hellip;</i></a></p>';
	}
	
	return $res;
}

sub summary_data {
	my ($selected_lsb_ver) = @_;
	
	my $summary = {};
	
	$summary->{STATUS} = Analysis::overall_verdict();
	
	# Calculate statistics on distribution statuses
	$summary->{DISTRS}{'TOTAL'} = scalar keys %$Analysis::all_distrs;
	@{$summary->{DISTRS}}{'GOOD', 'WARN', 'BAD'} = (0)x3; # initialize the statistic
	foreach my $Did ( keys %$Analysis::all_distrs ) {
		my $status = Analysis::distr_status($Did); # OK|UNNEEDED|UNKNOWN|MISSING
		if ( $status eq 'OK' ) { $summary->{DISTRS}{GOOD}++; }
		elsif ( $status eq 'UNNEEDED' || $status eq 'UNKNOWN' ) { $summary->{DISTRS}{WARN}++; }
		elsif ( $status eq 'MISSING' ) { $summary->{DISTRS}{BAD}++; }
	}
	
	# Select the displayed status of distributions
	my $dstat = $summary->{DISTRS};
	if ( $dstat->{WARN} == 0 && $dstat->{BAD} == 0 ) {
		$dstat->{STATUS} = 'ALL';
	}
	elsif ( $dstat->{WARN} > 0 && $dstat->{BAD} == 0 ) {
		$dstat->{STATUS} = 'ALL_EXCEPT';
	}
	elsif ( $dstat->{GOOD} > 0 && $dstat->{WARN} == 0 && $dstat->{BAD} > 0 ) {
		$dstat->{STATUS} = 'SOME_GOOD';
	}
	elsif ( $dstat->{WARN} > 0 && $dstat->{BAD} > 0 ) {
		$dstat->{STATUS} = 'SOME_GOOD_EXCEPT';
	}
	elsif ( $dstat->{GOOD} == 0 && $dstat->{WARN} == 0 && $dstat->{BAD} > 0 ) {
		$dstat->{STATUS} = 'NO_ONE';
	}
	
	# Check the hash type
	if ( $appchecker_branch{'STD'} eq 'LSB' ) {
		foreach my $com_obj ( values %$Analysis::app_components ) {
			if ( $com_obj->{NO_SHT_HASH} ) {
				$summary->{NO_SHT_HASH}++;
			}
		}
	}

	# Libraries
	my $libs = libs_data();

	# initialization
	$summary->{UNNEEDED_LIBS} = 0;
	$summary->{UNAPPROVED_LIBS} = 0;
	for ( qw/NOT CANDIDATE FUTURE IN OPTIONAL DEPRECATED WITHDRAWN/ ) {
		$summary->{LSB_LIBS}{$_}{COUNT} = 0;
		$summary->{LSB_LIBS}{$_}{VERS} = {};
	}
	
	foreach my $soname ( keys %$libs ) {
		my $lib = $libs->{$soname};
		
		# Check libs without interfaces used
		if ( $Analysis::libs_unneeded->{$soname} ) {
			$summary->{UNNEEDED_LIBS}++;
		}
		# Check unapproved libs
		if ( !$Analysis::approved_libs->{$soname} ) {
			# It's ok if a library is unapproved, but is present at all distributions.
			if ( grep {!$_} values %{$lib->{DISTRS}} ) { # Check if there are missing distributions
				$summary->{UNAPPROVED_LIBS}++;
			}
		}

		# Calculate LSB statistics for libraries
		my $lsb_status = $Analysis::lsb_lib_status->{$soname}{STATUS};
		my $lsb_status_ver = $Analysis::lsb_lib_status->{$soname}{STATUS_VER};
		
		$summary->{LSB_LIBS}{$lsb_status}{COUNT}++;
		$summary->{LSB_LIBS}{$lsb_status}{VERS}{$lsb_status_ver}++ if $lsb_status_ver;
	}
	
	# Interfaces
	for ( qw/NOT CANDIDATE FUTURE IN OPTIONAL DEPRECATED WITHDRAWN/ ) {
		$summary->{LSB_INTS}{$_}{NAMES} = {};
		$summary->{LSB_INTS}{$_}{VERS} = {};
	}
	my $all_ints = {};
	foreach my $com ( keys %$Analysis::app_components ) {
		my $com_obj = $Analysis::app_components->{$com};
		foreach my $fullname ( keys %{$com_obj->{NEEDED_INTS}} ) {
			foreach my $Did ( keys %$Analysis::all_distrs ) {
				my $lib = $com_obj->{$Did}{PROBABLE_LIB}{$fullname} 
						|| $com_obj->{$Did}{FOUND_INTS}{$fullname}
						|| "";
				$all_ints->{$fullname}{$lib} = 1;
			}
		}
	}
	foreach my $fullname ( keys %$all_ints ) {
	foreach my $lib ( keys %{$all_ints->{$fullname}} ) {
		# Calculate LSB statistics for interfaces
		my $lsb_status     = $Analysis::lsb_int_status->{$fullname}{$lib}{STATUS};
		my $lsb_status_ver = $Analysis::lsb_int_status->{$fullname}{$lib}{STATUS_VER};
		
		$summary->{LSB_INTS}{$lsb_status}{NAMES}{$fullname} = 1;
		$summary->{LSB_INTS}{$lsb_status}{VERS}{$lsb_status_ver}++ if $lsb_status_ver;
	}}
	foreach my $data ( values %{$summary->{LSB_INTS}} ) {
		$data->{COUNT} = keys %{$data->{NAMES}};
		delete $data->{NAMES};
	}
	
	## Calculate the overall LSB status
	
	my $lib_lsb_status = $summary->{LSB_LIBS};
	my $int_lsb_status = $summary->{LSB_INTS};
	
	if ( $lib_lsb_status->{"NOT"}{COUNT} || $lib_lsb_status->{"WITHDRAWN"}{COUNT} ) {
		$summary->{LSB} = 'NON_LSB_LIBS';
	}
	elsif ( $int_lsb_status->{"NOT"}{COUNT} || $int_lsb_status->{"WITHDRAWN"}{COUNT} ) {
		$summary->{LSB} = 'NON_LSB_INTS';
	}
	elsif ( $lib_lsb_status->{"CANDIDATE"}{COUNT} ) {
		if ( $lib_lsb_status->{"FUTURE"}{COUNT} ) {
			$summary->{LSB} = 'CANDIDATE_FUTURE_LIBS';
		} else {
			$summary->{LSB} = 'CANDIDATE_LIBS';
		}
	}
	elsif ( $int_lsb_status->{"CANDIDATE"}{COUNT} ) {
		if ( $int_lsb_status->{"FUTURE"}{COUNT} ) {
			$summary->{LSB} = 'CANDIDATE_FUTURE_INTS';
		} else {
			$summary->{LSB} = 'CANDIDATE_INTS';
		}
	}
	elsif ( $lib_lsb_status->{"FUTURE"}{COUNT} ) {
		$summary->{LSB} = 'FUTURE_LIBS';
	}
	elsif ( $int_lsb_status->{"FUTURE"}{COUNT} ) {
		$summary->{LSB} = 'FUTURE_INTS';
	}
	elsif ( $lib_lsb_status->{"DEPRECATED"}{COUNT} || $int_lsb_status->{"DEPRECATED"}{COUNT} ) {
		$summary->{LSB} = 'DEPRECATED';
	}
	else { #  "IN" or "OPTIONAL"
		$summary->{LSB} = 'ALL_OK';
	}
	
	if ( !%$libs ) {
		$summary->{DISTRS}{STATUS} = 'NO_BINS';
		$summary->{LSB} = 'NO_BINS';
	}
	
	if ( $Analysis::improper_arch ) {
		$summary->{IMPROPER_ARCH} = $Analysis::improper_arch;
	}
	
	return $summary;
}
#-----------------------------------------------------------------------

# Renders a short summary for the header of the results page.
sub Summary {
	my ($selected_lsb_ver) = @_;
	
	my $summary = summary_data($selected_lsb_ver);
	
	## Let's print the summary

	my $nl = "\n";
	my $res = '';

	my $green_image  = '<img src="images/dot_green.png" width="16" height="16" alt="green" /> ';
	my $yellow_image = '<img src="images/dot_yellow.png" width="16" height="16" alt="yellow" /> ';
	my $red_image    = '<img src="images/dot_red.png" width="16" height="16" alt="red" /> ';
	my $warn_image   = '<img src="images/dot_warn.png" width="16" height="16" alt="warning" /> ';

	my $green_table = <<DATA;
<div style="background-color: #40d040; border: 1px solid #7f7f7f; padding: 0.3em; text-align: left; vertical-align: middle; font-size: large; color: #ffffff; margin-bottom: 0;"><b>No compatibility problems detected</b>
</div>
<div id="summary" class="summary" style="background-color: #e6ffe6;">
DATA
	my $yellow_table = <<DATA;
<div style="background-color: #d0a720; border: 1px solid #7f7f7f; padding: 0.3em; text-align: left; vertical-align: middle; font-size: large; color: #ffffff; margin-bottom: 0;"><b>Some compatibility problems detected</b>
</div>
<div id="summary" class="summary" style="background-color: #fff9d4;">
DATA
	my $red_table = <<DATA;
<div style="background-color: #d04040; border: 1px solid #7f7f7f; padding: 0.3em; text-align: left; vertical-align: middle; font-size: large; color: #ffffff; margin-bottom: 0;"><b>Numerous compatibility problems found</b>
</div>
<div id="summary" class="summary" style="background-color: #ffeeeb;">
DATA

	local *_pl = sub {
		my ($num, $mid, $singular, $plural) = @_;
		
		return '<b>'.$num.'</b> '.($mid ? $mid.' ' : '').($num > 1 ? $plural : $singular);
	};
	
	local *refer = sub {
		my ($text_html, $tab, $add, $no_below) = @_;
		
		my $link = tab_link($tab).'&nofilt=1';
		$link .= $add if ( $add );
		return '<a href="'.esc_html($link).'">'.$text_html.'</a>';
	};
	
	local *see_details = sub {
		my ($tab, $add) = @_;
		
		my $link = tab_link($tab);
		$link .= $add if ( $add );
		return ' (<a href="'.esc_html($link).'">details&hellip;</a>)';
	};

	# Summary block
	if ( $summary->{STATUS} eq 'Success' ) {
		$res .= $green_table;
	}
	elsif ( $summary->{STATUS} eq 'Warnings' ) {
		$res .= $yellow_table;
	}
	elsif ( $summary->{STATUS} eq 'Failed' ) {
		$res .= $red_table;
	}

	# Summary on compatibility with distributions
	my $dstat = $summary->{DISTRS};
	my $distr_status = $dstat->{STATUS};
	if ( $distr_status eq 'ALL' ) {
		$res .= '<p>';
		$res .= $green_image; # Green image
		$res .= 'All libraries and interfaces used by the Application';
		$res .= ' are present in <b>all the analyzed distributions</b>';
		$res .= see_details('distrs', '&expand=1');
		$res .= '.</p>'.$nl;
	}
	elsif ( $distr_status eq 'ALL_EXCEPT' ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'All libraries and interfaces required by the Application'
				.' (except some unused or unknown libraries)'
				.' are present in <b>all the analyzed distributions</b>';
		$res .= see_details('distrs', '&expand=1');
		$res .= '.</p>'.$nl;
	}
	elsif ( $distr_status eq 'SOME_GOOD' ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'There '.($dstat->{GOOD} > 1 ? 'are' : 'is' ).' '
				.refer('<b>'.$dstat->{GOOD}.'</b> of <b>'.$dstat->{TOTAL}.'</b> distributions', 'distrs', '&expand=1')
				.' that provide all the required libraries and interfaces.';
		$res .= '</p>'.$nl;
	}
	elsif ( $distr_status eq 'SOME_GOOD_EXCEPT' ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		my $n = $dstat->{GOOD} + $dstat->{WARN};
		$res .= 'There '.($n > 1 ? 'are' : 'is' ).' '
				.refer('<b>'.$n.'</b> of <b>'.$dstat->{TOTAL}.'</b> distributions', 'distrs', '&expand=1')
				.' that provide all the required libraries and interfaces'
				.' except unneeded and unknown.';
		$res .= '</p>'.$nl;
	}
	elsif ( $distr_status eq 'NO_ONE' ) {
		$res .= '<p>';
		$res .= $red_image; # Red image
		$res .= 'The Application is unlikely to run on any of the analyzed distributions.';
		$res .= '</p>'.$nl;
	}
	elsif ( $distr_status eq 'NO_BINS' ) {
		if ( $summary->{IMPROPER_ARCH} ) {
			$res .= '<p>';
			$res .= $yellow_image; # Yellow image
			$res .= 'The Application Checker is unable to test applications for <b>another architecture</b>.';
			$res .= ' Please use the version of the Application Checker packaged for the same architecture as the program you want to check.';
			$res .= '</p>'.$nl;
		} else {
			$res .= '<p>';
			$res .= $yellow_image; # Green image
			$res .= 'The application requires no external libraries or interfaces.';
			$res .= ' Please see the '.$res_tab_names{'cert'}.' tab for the results of script analysis (Perl, Python'.(($appchecker_branch{'STD'} eq 'LSB') ? ', Shell' : '').' scripts).';
			$res .= '</p>'.$nl;
		}
	}
	
	if ( $summary->{IMPROPER_ARCH} && $distr_status ne 'NO_BINS' ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'The application includes binary components compiled for <b>another architecture</b>. These binaries will be ignored.';
		$res .= ' Please use the version of the Application Checker packaged for the same architecture as the program you want to check.';
		$res .= '</p>'.$nl;
	}

	# Summary on LSB/Moblin compatibility
	$res .= '<p>';
	my $lib_lsb_status = $summary->{LSB_LIBS};
	my $int_lsb_status = $summary->{LSB_INTS};
	if ( $summary->{LSB} eq 'NON_LSB_LIBS' ) {
		my $x = $lib_lsb_status->{"NOT"}{COUNT} + $lib_lsb_status->{"WITHDRAWN"}{COUNT}
				+$lib_lsb_status->{"CANDIDATE"}{COUNT} + $lib_lsb_status->{"FUTURE"}{COUNT};
		$res .= $red_image; # Red image
		$res .= 'The Application uses '
			.refer( _pl($x, 'external', 'library', 'libraries'), 'libs', '&filt_show=non-std' )
			.' <b>incompatible with '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'</b>.';
	}
	elsif ( $summary->{LSB} eq 'NON_LSB_INTS' ) {
		my $x = $int_lsb_status->{"NOT"}{COUNT} + $int_lsb_status->{"WITHDRAWN"}{COUNT}
				+$int_lsb_status->{"CANDIDATE"}{COUNT} + $int_lsb_status->{"FUTURE"}{COUNT};
		$res .= $red_image; # Red image
		$res .= 'The Application uses '
			.refer( _pl($x, '', 'interface', 'interfaces'), 'ints', '&filt_show=non-std')
			.' <b>incompatible with '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'</b>.';
	}
	elsif ( $summary->{LSB} eq 'CANDIDATE_FUTURE_LIBS' ) {
		my $cand_version = (sort keys %{$lib_lsb_status->{"CANDIDATE"}{VERS}})[-1]; # max
		if ( $cand_version eq 'Unknown' || $cand_version eq '' ) {
			$cand_version = 'future '.$appchecker_branch{'STD'}.' version';
		} else {
			$cand_version = $appchecker_branch{'STD'}.' '.$cand_version;
		}
		$res .= $yellow_image; # Yellow image
		$res .= 'All libraries used by the Application are either included'
				.' in <b>'.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'</b> or future versions or are candidates'
				.' for '.esc_html($cand_version).see_details('libs', '&show=non-std').'.';
		if ( $lib_lsb_status->{'DEPRECATED'}{COUNT} ) {
			$res .= ' Note that some libraries used by the Application are <b>deprecated</b>'
				.see_details('libs', '&show=deprecated').'.';
		}
	}
	elsif ( $summary->{LSB} eq 'CANDIDATE_LIBS' ) {
		my $cand_version = (sort keys %{$lib_lsb_status->{"CANDIDATE"}{VERS}})[-1]; # max
		if ( $cand_version eq 'Unknown' || $cand_version eq '' ) {
			$cand_version = 'future '.$appchecker_branch{'STD'}.' version';
		} else {
			$cand_version = $appchecker_branch{'STD'}.' '.$cand_version;
		}
		$res .= $yellow_image; # Yellow image
		$res .= 'All libraries used by the Application are either included'
				.' in <b>'.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'</b> or are candidates'
				.' for '.esc_html($cand_version).see_details('libs', '&show=non-std').'.';
		if ( $lib_lsb_status->{'DEPRECATED'}{COUNT} ) {
			$res .= ' Note that some libraries used by the Application are <b>deprecated</b>'
				.see_details('libs', '&show=deprecated').'.';
		}
	}
	elsif ( $summary->{LSB} eq 'CANDIDATE_FUTURE_INTS' ) {
		my $cand_version = (sort keys %{$int_lsb_status->{"CANDIDATE"}{VERS}})[-1]; # max
		if ( $cand_version eq 'Unknown' || $cand_version eq '' ) {
			$cand_version = 'future '.$appchecker_branch{'STD'}.' version';
		} else {
			$cand_version = $appchecker_branch{'STD'}.' '.$cand_version;
		}
		$res .= $yellow_image; # Yellow image
		$res .= 'All interfaces used by the Application are either included'
				.' in <b>'.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'</b> or future versions or are candidates'
				.' for '.esc_html($cand_version).see_details('ints', '&show=non-std').'.';
		if ( $lib_lsb_status->{'DEPRECATED'}{COUNT} ) {
			$res .= ' Note that some libraries used by the Application are <b>deprecated</b>'
				.see_details('libs', '&show=deprecated').'.';
		} elsif ( $int_lsb_status->{'DEPRECATED'}{COUNT} ) {
			$res .= ' Note that some interfaces used by the Application are <b>deprecated</b>'
				.see_details('ints', '&show=deprecated').'.';
		}
	}
	elsif ( $summary->{LSB} eq 'CANDIDATE_INTS' ) {
		my $cand_version = (sort keys %{$int_lsb_status->{"CANDIDATE"}{VERS}})[-1]; # max
		if ( $cand_version eq 'Unknown' || $cand_version eq '' ) {
			$cand_version = 'future '.$appchecker_branch{'STD'}.' version';
		} else {
			$cand_version = $appchecker_branch{'STD'}.' '.$cand_version;
		}
		$res .= $yellow_image; # Yellow image
		$res .= 'All interfaces used by the Application are either included'
				.' in <b>'.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'</b> or are candidates'
				.' for '.esc_html($cand_version).see_details('ints', '&show=non-std').'.';
		if ( $lib_lsb_status->{'DEPRECATED'}{COUNT} ) {
			$res .= ' Note that some libraries used by the Application are <b>deprecated</b>'
				.see_details('libs', '&show=deprecated').'.';
		} elsif ( $int_lsb_status->{'DEPRECATED'}{COUNT} ) {
			$res .= ' Note that some interfaces used by the Application are <b>deprecated</b>'
				.see_details('ints', '&show=deprecated').'.';
		}
	}
	elsif ( $summary->{LSB} eq 'FUTURE_LIBS' ) {
		my $future_version = (sort keys %{$lib_lsb_status->{"FUTURE"}{VERS}})[-1]; # max
		
		my $x = $lib_lsb_status->{"FUTURE"}{COUNT};
		
		$res .= $yellow_image; # Yellow image
		$res .= 'The Application uses '
				.refer( _pl($x, '', 'library', 'libraries'), 'libs', '&filt_show=non-std' )
				.', which appeared in a <b>later '.$appchecker_branch{'STD'}.' version than '.esc_html($selected_lsb_ver).'</b>.';
		$res .= ' Try checking the Application for compatibility with '.$appchecker_branch{'STD'}.' '.esc_html($future_version).'.';
		
		if ( $lib_lsb_status->{'DEPRECATED'}{COUNT} ) {
			$res .= ' Note that some libraries used by the Application are <b>deprecated</b>'
					.' in '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).see_details('libs', '&show=deprecated').'.';
		}
	}
	elsif ( $summary->{LSB} eq 'FUTURE_INTS' ) {
		my $future_version = (sort keys %{$int_lsb_status->{"FUTURE"}{VERS}})[-1]; # max
		
		my $x = $int_lsb_status->{"FUTURE"}{COUNT};
		
		$res .= $yellow_image; # Yellow image
		$res .= 'The Application uses '
				.refer( _pl($x, '', 'interface', 'interfaces'), 'ints', '&filt_show=non-std')
				.', which appeared in a <b>later '.$appchecker_branch{'STD'}.' version than '.esc_html($selected_lsb_ver).'</b>.';
		$res .= ' Try checking the Application for compatibility with '.$appchecker_branch{'STD'}.' '.esc_html($future_version).'.';
		
		if ( $lib_lsb_status->{"DEPRECATED"}{COUNT} ) {
			$res .= ' Note that some libraries used by the Application are <b>deprecated</b>'
					.' in '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).see_details('libs', '&show=deprecated').'.';
		}
		elsif ( $int_lsb_status->{"DEPRECATED"}{COUNT} ) {
			$res .= ' Note that some interfaces used by the Application are <b>deprecated</b>'
					.' in '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).see_details('ints', '&show=deprecated').'.';
		}
	}
	elsif ( $summary->{LSB} eq 'DEPRECATED' ) {
		$res .= $yellow_image; # Yellow image
		$res .= 'All libraries and interfaces used by the Application';
		$res .= ' are <b>included in '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'</b>';
		$res .= ', but some of them are <b>deprecated</b>.';
		$res .= ' You may have compatibility issues with new distributions in the future.';
	}
	elsif ( $summary->{LSB} eq 'ALL_OK' ) {
		$res .= $green_image; # Green image
		$res .= 'All libraries and interfaces used by the Application';
		$res .= ' are <b>included in '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'</b>.';
		$res .= ' You have a good chance to achieve '.refer($appchecker_branch{'STD'}.' compliance', 'cert', '', 1).'.';
	}
	elsif ( $summary->{LSB} eq 'NO_BINS' ) {
		# do nothing
	}
	else {
		$res .= "Unhandled Summary: ".esc_html($summary->{LSB});
	}
	$res .= '</p>'.$nl;

	# Libraries without interfaces used
	if ( $summary->{UNNEEDED_LIBS} ) {
		$res .= '<p>';
		$res .= $warn_image; # Yellow image
		$res .= 'The Application uses '
			.refer( _pl($summary->{UNNEEDED_LIBS}, 'external', 'library', 'libraries'), 'libs', '&filt_show=unneeded' )
			.' <b>without using interfaces</b> from '.(($summary->{UNNEEDED_LIBS} > 1) ? 'those libraries' : 'this library').'.'
			.' The Application would be more portable if it is fixed to not use unneeded libraries.';
		$res .= '</p>';
	}
	
	# Unapproved libraries
	if ( $summary->{UNAPPROVED_LIBS} ) {
		$res .= '<p>';
		$res .= $warn_image; # Yellow image
		$res .= 'The Application uses '
			.refer( _pl($summary->{UNAPPROVED_LIBS}, 'external', 'library', 'libraries'), 'libs', '&filt_show=uncommon' )
			.' that '.($summary->{UNAPPROVED_LIBS}==1?'is':'are').' <b>not common</b> in distributions.'
			.' Please consider providing such libraries as a part of the Application package.';
		$res .= '</p>';
	}

	if ($appchecker_branch{'STD'} eq 'LSB') {
		if ( $summary->{NO_SHT_HASH} ) {
			$res .= '<p>';
			$res .= $warn_image; # Yellow image
			if ( $summary->{NO_SHT_HASH} == 1 ) { # only one
				$res .= 'There is '.refer( '<b>1</b> application component', 'com', '' ).' which has <b>no `.hash\' section</b>. This component would fail to run on older distributions (e.g., RHEL4).';
			} else { # plural
				$res .= 'There are '.refer( '<b>'.$summary->{NO_SHT_HASH}.'</b> application components', 'com', '' )
					.' that have <b>no \'.hash\' section</b>. These components will fail to run on older distributions (e.g., RHEL4).';
			}
			$res .= '</p>';
		}
	}


	$res .= "$nl</div>$nl<br />$nl";
	$res .= tooltip();

	$res .= <<DATA;
<script type="text/javascript" language="javascript">
// <![CDATA[
tooltip_texts['libs_ints_common'] = 'Common libraries are manually selected by the Linux Foundation<br />to represent .so names of those libraries that may be externally required<br />by applications and provided by distributions.<br />Please write to <a href="mailto:lsb-appchk-support\@linuxbase.org">lsb-appchk-support\@linuxbase.org</a> to suggest<br />adding more libraries to the "common libraries" list.';
tooltip_texts['libs_ints_unknown'] = 'Unknown libraries are those not included in the "approved" list<br />of common libraries. This means such libraries are not supposed<br />to be externally required by applications and correspondingly provided<br />by distributions. Sometimes new versions of common libraries<br />fall into the "unknown" category as well. It is recommended to include<br />such libraries as a part of your application (either by statically linking<br />or by supplying the .so in your package).';
tooltip_texts['libs_ext_ints'] = 'There are no interfaces used directly from this library,<br />but there are some interfaces used from additional<br />libraries that are linked by this library.<br />Click the number to see these interfaces.';
tooltip_texts['libs_unneeded'] = 'No interfaces from this library are used<br />by any application component.';
tooltip_texts['libs_normal_zero'] = 'This library isn\\\'t necessary for the selected<br />application component, but is required by<br />other application component.';
tooltip_texts['ints_linkage_failed'] = 'The analyzer has failed to determine which library (among<br />those required by this application component) provides this<br />interface. This may happen if the corresponding library<br />wasn\\\'t found in the database or in the distribution on this<br />machine, or if the list of libraries needed by this application<br />component is incomplete. If this binary is executable, then<br />most likely it will fail if launched on this machine.';
tooltip_texts['com_common_lib'] = 'This library is provided by some distributions.<br />Depending on which distributions provide this library, you can<br />consider excluding this library from the application package<br />(making it an external dependency), thus reducing the<br />package size.';
tooltip_texts['com_no_hash_section'] = 'The ELF binaries have a hash section for runtime symbol<br/>lookup to be used by the dynamic linker. This is the \\'.hash\\'<br/>section in the old \\'sysv\\' format and the \\'.gnu.hash\\' section<br/>in the new \\'gnu\\' format. The latter format is up to 50\%<br/>more effective. In newer distributions, the dynamic linker<br/>can handle both types of hash formats, while older<br/>distributions can only work with the older \\'sysv\\' format<br/>and therefore require a \\'.hash\\' ELF section.<br/>Which section is present in the binary is determined by<br/>the linker switch \\'--hash-style\\' at compile time.';
tooltip_texts['lib_missing_in_needed'] = 'This library name was guessed. It is missing in the component<br/>dependency list, so the linker will fail to find interfaces in it.<br/>Sometimes it is not a problem for "plug-in" components, because<br/>interfaces can be taken from a library that was pulled in by the<br/>main executable (analyzer can\\'t resolve this case automatically).<br/>We recommend to link such components with all required libs.';
// ]]>
</script>
DATA

	return $res;
}

sub ints_data {
	my ($filt_distr, $filt_com, $filt_lib, $filt_int) = @_;
	
	my $ints = {};
	# $ints->{$fullname}{$com}{$lib_key} = {
	#	LIBRARY, # library name (if LIBRARY ne $lib_key)
	#	SONAME, # real library (if SONAME ne LIBRARY)
	#	DISTRS => { $Did => 1 }
	#   UNAPPROVED,
	# };
	
	my $nonfilt = { INTS => {}, LIBS => {} }; # Non-filtered information
	my $filt_lib_impl;
	if ( $filt_lib && $filt_lib =~ /^([^*]*)\*$/ ) { # Library filter
		$filt_lib_impl = $1;
	}
	
	foreach my $com ( keys %$Analysis::app_components ) {
		my $com_obj = $Analysis::app_components->{$com};

		my %needed_libs = map {$_ => 1} @{$com_obj->{NEEDED_LIBS}};

		foreach my $fullname ( keys %{$com_obj->{NEEDED_INTS}} ) {
			$nonfilt->{INTS}{$fullname} = 1; # Non-filtered information
			
			if ( $filt_int && $filt_int ne $fullname ) { next; }
			
			foreach my $Did ( keys %$Analysis::all_distrs ) {
				my $com_did = $com_obj->{$Did};
				if ( my $lib = $com_did->{PROBABLE_LIB}{$fullname} 
							|| $com_did->{FOUND_INTS}{$fullname} ) 
				{
					# Non-filtered information
					$nonfilt->{LIBS}{$lib} = 1;
					
					if ( $filt_com && $com ne $filt_com ) { next; } # Component filter
					if ( $filt_distr && !$filt_distr->{$Did} ) { next; } # Distribution filter
					if ( $filt_lib_impl ) { # Library filter
						# If $filt_lib ends with a '*' then pass all additional libraries
						# that are loaded by the $filt_lib library.
						if ( $filt_lib_impl ne $lib ) {
							my $res = 0;
							Analysis::for_deps_tree $com_did->{LIBS}, $lib, sub {
								my ($soname) = @_;
								
								if ( $soname eq $filt_lib_impl ) {
									$res = 1;
									return 1; # last
								}
								if ( $needed_libs{$lib} ) { # NEEDED lib
									return 0; # skip children
								}
								return;
							};
							next if !$res;
						}
					}
					elsif ( $filt_lib && $lib ne $filt_lib ) { next; } # Library filter

					if ( $com_did->{FOUND_INTS}{$fullname} ) {
						my $lib_key = $lib;
						my $data = $ints->{$fullname}{$com}{$lib_key} ||= {};
						$data->{LIBRARY} = $lib if $lib ne $lib_key;
						$data->{DISTRS}{$Did} = 1;
					} else {
						my $data = $ints->{$fullname}{$com}{$lib} ||= {};
						$data->{DISTRS}{$Did} = 0;
						if ( !$com_did->{LIBS}{$lib}{FOUND} ) {
							if ( $Analysis::approved_libs->{$lib} ) {
								$data->{LIBMISS}++;
								if ( !$needed_libs{$lib}
									&& !defined $com_did->{LIBS}{$lib}{FOUND} # not in needed_impl
								) {
									$data->{NOLIB}++;
								}
							}
						}
						if ( !$Analysis::approved_libs->{$lib} ) {
							$data->{UNAPPROVED} = 1;
						}
					}
				}
				else { # Non-linked for this distro
					my $lib = ""; # library for this interface wasn't found.
					
					# Non-filtered information
					$nonfilt->{LIBS}{"~"} = 1; # symbol '~' goes after a-z in alphabetical order
					
					if ( $filt_com && $com ne $filt_com ) { next; } # Component filter
					if ( $filt_distr && !$filt_distr->{$Did} ) { next; } # Distribution filter
					if ( $filt_lib && $filt_lib ne 'nonlinked' ) { next; } # Library filter
					
					my $data = $ints->{$fullname}{$com}{$lib} ||= {};
					$data->{DISTRS}{$Did} = 0;
					
					my $Iname = $fullname;
					$Iname =~ s/\@.*//;
					if ( $Analysis::unknown_ints->{$Iname} ) {
						$data->{UNAPPROVED} = 1;
					}
				}
			}
		}
	}
	
	if ( wantarray ) {
		return ($ints, $nonfilt);
	}
	return $ints;
}

sub tooltip
{
	my $res = "";

	$res .= <<DATA;
<div id="tooltip" style="display: none; border: solid 1px black; position: absolute; z-index: 1000;" onclick="javascript:event.cancelBubble = true;">
<table cellpadding="0" cellspacing="0" border="0" style="width: 17em;">
  <tr style="background-color: #d0d0d0; color: black;">
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/environment-spacer.gif" width="7" height="1" alt="" /></td>
    <td align="center" width="100%" style="border-bottom: solid 1px black;"><b>Information</b></td>
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/close.png" onclick="javascript:initDialog(null);" width="20" height="20" title="Close" alt="X" style="cursor: pointer;" /></td>
  </tr>
  <tr>
    <td colspan="3" style="padding: 5px; white-space: nowrap; background-color: #ffffd0;" id="tooltip_text"></td>
  </tr>
</table>
</div>

<table cellpadding="0" cellspacing="0" border="0" style="display: none; position: absolute; z-index: 999;" id="shadow">
  <tr>
    <td width="14" height="14" style="background: url('images/shadow_tl.png') no-repeat;"></td>
    <td height="14" style="background: url('images/shadow_tc.png') repeat-x;"></td>
    <td width="14" height="14" style="background: url('images/shadow_tr.png') no-repeat;"></td>
  </tr>
  <tr>
    <td width="14" style="background: url('images/shadow_ml.png') repeat-y;"></td>
    <td style="background: url('images/shadow_mc.png') repeat;" id="shadow_center"></td>
    <td width="14" style="background: url('images/shadow_mr.png') repeat-y;"></td>
  </tr>
  <tr>
    <td width="14" height="14" style="background: url('images/shadow_bl.png') no-repeat;"></td>
    <td height="14" style="background: url('images/shadow_bc.png') repeat-x;"></td>
    <td width="14" height="14" style="background: url('images/shadow_br.png') no-repeat;"></td>
  </tr>
</table>

<script type="text/javascript" language="javascript">
// <![CDATA[

preload_images(new Array('status_collapse.png', 'close.png', 'shadow_bc.png', 'shadow_bl.png', 'shadow_br.png', 'shadow_mc.png', 'shadow_ml.png', 'shadow_mr.png', 'shadow_tc.png', 'shadow_tl.png', 'shadow_tr.png'));

var just_shown_dialog = false;          // Indicates that the dialog was just shown (to prevent its hiding from onBodyClick)
var dialog = null;                      // The dialog itself
var dialog_caller = null;               // Element which caused the dialog to appear
var dialog_caller_class = null;         // Normal class name of the dialog caller (to be restored when the dialog is closed)
var dialog_shadow = document.getElementById('shadow');
var dialog_shadow_center = document.getElementById('shadow_center');

// Hides the previous dialog (if it was shown) and prepare data for
// displaying the new one.
// CALLER - the HTML element which caused the dialog to appear and (possibly)
// will change its appearance because of that.
function initDialog(caller) {
	if (dialog_caller && dialog_caller_class)
		dialog_caller.className = dialog_caller_class;
	dialog_shadow.style.display = 'none';
	if (dialog)
		dialog.style.display = 'none';
	if (!caller || (caller == dialog_caller)) {
		dialog_caller = null;
		dialog_caller_class = null;
		return false;
	}
	else {
		dialog_caller = caller;
		dialog_caller_class = caller.className;
		return true;
	}
}

// Shows the dialog at the specified position and adjusts the shadow below it.
function showDialog(show_dlg, pos) {
	dialog = show_dlg;
	dialog.style.left = pos.x + 'px';
	dialog.style.top = pos.y + 'px';
	dialog.style.display = '';
	just_shown_dialog = true;
	dialog_shadow_center.width = dialog.offsetWidth - 15;
	dialog_shadow_center.height = dialog.offsetHeight - 15;
	dialog_shadow.style.left = pos.x + 'px';
	dialog_shadow.style.top = pos.y + 'px';
	dialog_shadow.style.display = '';
}

function onBodyClick() {
	if (!just_shown_dialog)
		initDialog(null);
	just_shown_dialog = false;
}

var tooltip_texts = new Array();

function show_hide_tooltip(elem_parent, id, event) {
	if (!initDialog(elem_parent))
		return;
	var tt_window = document.getElementById('tooltip');
	if (tt_window) {
		var tt_text = document.getElementById('tooltip_text');
		if (tt_text) {
			if (tooltip_texts[id])
				tt_text.innerHTML = tooltip_texts[id];
			else
				tt_text.innerHTML = '<font color="red"><b>Sorry, no information for this item was found!</b></font><br />Please write to <a href="mailto:lsb-appchk-support\@linuxbase.org">lsb-appchk-support\@linuxbase.org</a>.';
			var scrollTop = document.body.scrollTop;
			if (scrollTop == 0)
			{
			    if (window.pageYOffset)
			        scrollTop = window.pageYOffset;
			    else
			        scrollTop = (document.body.parentElement) ? document.body.parentElement.scrollTop : 0;
			}
			var scrollLeft = document.body.scrollLeft;
			if (scrollLeft == 0)
			{
			    if (window.pageXOffset)
			        scrollLeft = window.pageXOffset;
			    else
			        scrollLeft = (document.body.parentElement) ? document.body.parentElement.scrollLeft : 0;
			}
			var div_top = event.clientY + scrollTop + 10;
			var div_left = event.clientX + scrollLeft + 10;
			tt_window.style.display = '';
			var div_width = (tt_window.style.pixelWidth || tt_window.offsetWidth);
			var wnd_width = (document.body.clientWidth || window.innerWidth);
			if (div_left + div_width > wnd_width + scrollLeft)
				div_left = wnd_width + scrollLeft - div_width;
			if (div_left < scrollLeft)
				div_left = scrollLeft;
			showDialog(tt_window, {x: div_left, y: div_top});
			event.cancelBubble = true;
		}
	}
}

function onBodyClick() {
	initDialog(null);
}

// ]]>
</script>
DATA

	return $res;
}

sub altlib_tooltip_entry {
	my ($alts, $soname, $Did, $key) = @_;

	my $res = "";

	my $tiptext = '<b>'.esc_html($soname).'</b> is missing on <b>'.esc_html($Analysis::all_distrs->{$Did}{FULLNAME}).'</b> by default.<br/>'.esc_html(Analysis::soname_to_libname($soname)).' is present on '.esc_html($Analysis::all_distrs->{$Did}{FULLNAME}).' as:<br />';
	foreach my $alt ( @$alts ) {
		$tiptext .= '<br />&nbsp;<span style="color:#000080">'.esc_html($alt->{SONAME}).'</span>';
		$tiptext .= ' ('.esc_html($alt->{FILE}).')';
		$tiptext .= ', package: <span style="color:#000080">'.esc_html($alt->{PACKAGE}).'</span>' if $alt->{PACKAGE};
	}
	$res .= '<script type="text/javascript" language="javascript">'."\n"
		.'// <![CDATA['."\n"
		."tooltip_texts['$key'] = '".$tiptext."';\n"
		.'// ]]>'."\n"
		.'</script>';
	return $res;
}

# Library compatibility table
sub I_tab {
	my ($selected_lsb_ver, $distr_sel, $filt_com, $filt_lib, $filt_show) = @_;

	my $res = "";
	my $nl = "\n";

	my $filtered = $filt_com || $filt_lib || $filt_show || scalar grep {!$_} values %$distr_sel;
	
	my ($ints, $nonfilt_info) = ints_data( $distr_sel, $filt_com, $filt_lib );
	
	# Check if the table is empty
	if ( !$filtered && !%$Analysis::app_components ) {
		$res .= no_binary_div();
		return $res;
	}
	if ( !$filtered && !%$ints ) {
		$res .= no_binary_div("The application requires no external interfaces.");
		return $res;
	}

	$res .= <<HEREDOC;
The table below lists all external interfaces required by your Application (based on ELF symbols). Distribution presence and $appchecker_branch{'STD'} compliance statuses are provided for each interface.<br /><br />
HEREDOC

	my $link = tab_link('ints');
	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[
function toggle( id )
{
	var v = document.getElementById( id );
	if ( v ) {
		v.style.display = (v.style.display == 'none' ? '' : 'none');
	}
	return false;
}

function popup_win( link )
{
	var w = window.open( link, 'details',
			'toolbar=0,menubar=0,scrollbars=1,height=750,width=850,dependent=yes,dialog=yes'
		);
	w.moveTo(100,0);
	w.focus();

	return false;
}
// ]]>
</script>
HEREDOC
	
	my ($table, $row_n) = I_table($ints, $selected_lsb_ver, $distr_sel, $filt_com, $filt_lib, $filt_show);
	
	# Filters
	my $filt = "";
	
	$filt .= distr_selector( $distr_sel ); # Distribution selector
	
	$filt .= '<form name="filter" method="get" action="">';
	$filt .= '<table><tr><td>';
	
	$filt .= '<table cellspacing="0" class="bartable"><tr>';
	
	# "Component" filter
	$filt .= '<td style="border-right:0"><b>Component:</b></td>'
		.'<td>'
		.'<select name="filt_com" id="filt_com" onchange="onFilter()" style="width: 20em;">'.$nl
		.'<option value="">All</option>'.$nl;
	foreach my $com ( sort keys %$Analysis::app_components ) {
		my ($disp_name, $disp_path) = parse_app_component($com);
		my $selected = "";
		if ( $filt_com && $com eq $filt_com ) {
			$selected = ' selected="selected"';
		}
		$filt .= '<option value="'.esc_html($com).'"'
					.$selected.'>'.esc_html($disp_name).'</option>'.$nl;
	}
	$filt .= '</select>'.$nl
		.'</td>'; # </filt_com>

	# spacer
	$filt .= '<td style="padding:0.2em;border:0;background:white"></td>';

	# "Library" filter
	$filt .= '<td style="border-right:0"><b>Library:</b></td>'
		.'<td>'
		.'<select name="filt_lib" id="filt_lib" onchange="onFilter();" style="width: 12.5em;">'.$nl
		.'<option value="">All</option>'.$nl;
	foreach my $soname ( sort {lc $a cmp lc $b} keys %{$nonfilt_info->{LIBS}} ) {
		my $val = $soname;
		if ( $val eq "~" ) { $val = "nonlinked"; }
		my $display = $val;
		if ( $display eq "nonlinked" ) { $display = "Non-linked"; }
		my $selected = "";
		if ( $filt_lib && $val eq $filt_lib ) {
			$selected = ' selected="selected"';
		}
		$filt .= '<option value="'.esc_html($val).'"'
					.$selected.'>'.esc_html($display).'</option>'.$nl;
	}
	if ( $filt_lib =~ /^([^*]*)\*$/ ) {
		$filt .= '<option value="'.esc_html($filt_lib).'" selected="selected">'.esc_html($filt_lib).'</option>'.$nl;
	}
	$filt .= '</select>'.$nl
		.'</td>'; # </filt_lib>
	
	# spacer
	$filt .= '<td style="padding:0.2em;border:0;background:white"></td>';
	
	# "Show" filter
	$filt .= '<td style="border-right:0"><b>Show:</b></td>'
		.'<td>'
		.'<select name="filt_show" id="filt_show" onchange="onFilter()" style="width: 12.5em;">'.$nl;
	my @filt_show_values = ( ""=>"All", "problems"=>"Compatibility problems",
			"non-std"=>"Non-$appchecker_branch{'STD'}", "deprecated"=>"Deprecated" );
	while ( @filt_show_values ) {
		my $key = shift @filt_show_values;
		my $val = shift @filt_show_values;
		my $selected = "";
		if ( $filt_show && $key eq $filt_show ) {
			$selected = ' selected="selected"'; 
		}
		$filt .= '<option value="'.esc_html($key).'"'
					.$selected.'>'.esc_html($val).'</option>'.$nl;
	}
	$filt .= '</select>'.$nl
		.'</td>'; # </filt_show>
		
	$filt .= '</tr></table>';
	$filt .= '</td>';

	# Filter status
	my $all_n = scalar keys %{$nonfilt_info->{INTS}};
	$filt .= '<td style="padding-left:20pt">';
	$filt .= '<span id="lines_count">';
	if ( $filtered ) {
		$filt .= '<span style="color:red"><b>'.$row_n.'</b></span> of ';
	}
	$filt .= '</span>';
	$filt .= $all_n.' interface'.($all_n!=1?'s':'').' shown.';
	
	$filt .= '</td></tr></table>'; # </filters>
	$filt .= '</form>';
	
	## Ajax stuff
	$res .= show_error_dlg(''); # placeholder for an error box
	
	$res .= ajax_wait(); # ajax hourglass

	my $filt_com_js = esc_js($filt_com);
	my $filt_lib_js = esc_js($filt_lib);
	my $filt_show_js = esc_js($filt_show);
	my $ajax_link = esc_js('details='.esc_url($_GET{'details'}));
	my $details_js = esc_js($_GET{'details'});
	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[

timeout_show_progress = 0;

var cur_ifc = '$filt_com_js';
var cur_ifl = '$filt_lib_js';
var cur_ifs = '$filt_show_js';

function onFilter() {
	var filt_com = document.getElementById('filt_com');
	var filt_lib = document.getElementById('filt_lib');
	var filt_show = document.getElementById('filt_show');
	
	if ( filt_com ) {
		cur_ifc = filt_com.value;
		setCookie('ifc', cur_ifc, null);
		filt_com.blur();
	}
	if ( filt_lib ) {
		cur_ifl = filt_lib.value;
		setCookie('ifl', cur_ifl, null);
		filt_lib.blur();
	}
	if ( filt_show ) {
		cur_ifs = filt_show.value;
		setCookie('ifs', cur_ifs, null);
		filt_show.blur();
	}
	setCookie('if_for', '$details_js', null);
	
	var i_table_wrap = document.getElementById('i_table_wrap');
	
	if ( i_table_wrap ) {
		if ( ajax_req_busy) { onAbort(); }
		ajax_call_get('action=results_table&table=ints&$ajax_link');
	} else {
		setTimeout("onFilter()", 500);
	}
	
	return false;
}

function ajaxProcessResult(responseXML) {
	
	var i_table_html;
	if (responseXML.getElementsByTagName('i_table').length > 0) {
		i_table_html = responseXML.getElementsByTagName('i_table')[0].childNodes[0].nodeValue;
	}
	
	var i_table_wrap = document.getElementById('i_table_wrap');
	
	if ( i_table_html && i_table_wrap ) {
		i_table_wrap.innerHTML = i_table_html;
	} else {
		return -1;
	}
	
	var lines_count = document.getElementById('lines_count');
	if ( lines_count ) {
		var row_n;
		if (responseXML.getElementsByTagName('row_n').length > 0) {
			row_n = responseXML.getElementsByTagName('row_n')[0].childNodes[0].nodeValue;
			lines_count.innerHTML = '<span style="color:red"><b>'+row_n+'</b></span> of&nbsp;';
		} else {
			lines_count.innerHTML = '';
		}
	}
}

// Cookies on focus:
if (typeof(document.addEventListener) != 'undefined') {
 document.addEventListener('focus', doFocus, false); 
} else if (typeof(window.attachEvent) != 'undefined') {
 window.attachEvent('onfocus', doFocus);
}

function doFocus(event) {
	setCookie('ds', cur_ds, null);
	setCookie('ifc', cur_ifc, null);
	setCookie('ifl', cur_ifl, null);
	setCookie('ifs', cur_ifs, null);
	setCookie('if_for', '$details_js', null);
}

// Force AJAX initialization
ajaxInit();

// ]]>
</script>
HEREDOC
	
	## Put the filters
	$res .= $filt;
	
	## Put the table
	$res .= '<div id="i_table_wrap">'.$nl;
	$res .= $table;
	$res .= '</div>'.$nl;
	##
	
	return $res;
}

sub I_table {
	my ($ints, $selected_lsb_ver, $distr_sel, $filt_com, $filt_lib, $filt_show) = @_;
	
	my $filt_distr = scalar grep {!$_} values %$distr_sel;
	my $distrs_all = scalar grep {$_} values %$distr_sel;
	
	my $table = "";
	my $nl = "\n";
	
	# Table Header
	my $table_open = '<table id="i_table" class="wikitable" width="100%">';
	$table .= $table_open.$nl;
	$table .= '<tr>'
		.'<th>Interface</th>'
		.'<th>Version</th>'
		.'<th>Dependent<br/>Application&nbsp;Components</th>'
		.'<th>Library</th>'
		.'<th>Presence in<br />'.($filt_distr ? 'Selected ' : '').'Distributions</th>'
		.'<th id="std_col_header" style="white-space:nowrap">In&nbsp;'.$appchecker_branch{'STD'}.'&nbsp;'.esc_html($selected_lsb_ver).'?</th>'
		.'<th>More Info</th>'
		.'</tr>';

	my $aux_id = 0;
	my $interface_n = 0;
	
	my %name_sort = ();
	foreach my $fullname ( keys %$ints ) {
		my ($Iname, $ver) = split /\@/, $fullname, 2;
		
		$name_sort{$fullname} = lc( $Analysis::unmangled->{$Iname} || $Iname );
		$name_sort{$fullname} .= "@".$ver if $ver;
	}
	
	# Table Body
	my $prev = "";
	foreach my $fullname ( sort {$name_sort{$a} cmp $name_sort{$b}} keys %$ints ) {
		my ($Iname, $ver) = split /\@/, $fullname, 2;
		$ver ||= "";
		
		# $ints->{$fullname}{$com}{$lib_key} = {
		#	LIBRARY, # library name (if LIBRARY ne $lib_key)
		#	SONAME, # real library (if SONAME ne LIBRARY)
		#	DISTRS => { $Did => 1 }
		#   UNAPPROVED,
		# };
		
		my $single_lib = undef;
		
		my $com_group = {};
		foreach my $com ( sort keys %{$ints->{$fullname}} ) {
			my $com_key = int_com_key($ints->{$fullname}{$com});
			push @{$com_group->{$com_key}}, $com;
		}
		next if !%$com_group;
		
		my @com_rows = ();
		foreach my $com_key ( sort keys %$com_group ) {
			my $coms = $com_group->{$com_key};
			my ($com) = @$coms; # Any component
			
			my $libs = $ints->{$fullname}{$com};
			
			my @lib_rows = ();
			foreach my $lib_key ( sort keys %$libs ) {
				my $lib = $libs->{$lib_key};
				my $libname = $lib->{LIBRARY} || $lib_key;

				my $distrs = $lib->{DISTRS};
				my $distrs_total = 0;
				my $distrs_present = 0;
				if ( $distrs ) {
					$distrs_total = keys %$distrs;
					$distrs_present = grep {$_} values %$distrs;
				}

				## Library
				my $col_lib = "";
				if ( $libname eq "" ) { # Non-linked
					$col_lib .= '<span style="color:red">Non-linked</span>';
					# Tooltip:
					$col_lib .= '&nbsp;<a href="tests_help.pl#ints_linkage_failed" onclick="javascript:show_hide_tooltip(this, \'ints_linkage_failed\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a>';
				} else {
					if ( !$Analysis::approved_libs->{$libname} ) {
						# unapproved
						$col_lib .= '<span style="color:#A04020">'.esc_html($lib_key).'</span>';
					}
					elsif ( $lib->{NOLIB} ) {
						$col_lib .= '<span style="color:#A04020">'.esc_html($lib_key).'</span>';
						if ( $lib->{NOLIB} == $distrs_all ) {
							$col_lib .= '&nbsp;<a href="#" onclick="show_hide_tooltip(this, \'lib_missing_in_needed\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a>';
						}
					}
					elsif ( ($lib->{LIBMISS} && $lib->{LIBMISS} == $distrs_total) ) {
						$col_lib .= '<span style="color:#A04020">'.esc_html($lib_key).'</span>';
					}
					else {
						#my $soname_link = $appchecker_branch{'NAVIGATOR'}.'rawlib.php'
							#.'?cmd=list-byrunname&RLrunname='.esc_url($libname) );
						#$col_library = '<a href="'.esc_html($soname_link).'" target="_blank"'
								#.($color ? ' style="color:'.$color.'"' : '').'>'.esc_html($libname).'</a>';
						$col_lib .= esc_html($lib_key);
					}
				}
				
				## Presence in distributions
				
				my $col_distrs = "";
				my $bgcolor;
				my $pr_link = tab_link('ints').'&com='.esc_url($com).'&list='.esc_url($fullname).'&lib='.esc_url($lib_key);
				my $pr_onclick = 'onclick="'.esc_html('return popup_win(\''.esc_js($pr_link).'\')').'"';
				if ( $distrs_present > 0 ) {
					if ( $distrs_present == $distrs_total ) {
						if ( $filt_show && $filt_show eq 'problems' ) { next; }
						$bgcolor = "#CCFFCC"; # green
						$col_distrs .= $distrs_present." of ".$distrs_total;
					} else {
						$bgcolor = "#FEFECC"; # yellow
						$col_distrs .= $distrs_present." of ".$distrs_total;
						if ( $lib->{UNAPPROVED} ) {
							$col_distrs .= ' known';
						}
					}
				} else {
					if ( $lib->{UNAPPROVED} ) {
						$bgcolor = "#FEECCC"; # dark yellow
						$col_distrs .= 'Unknown';
					}
					else {
						if ( $distrs_total < $distrs_all ) {
							$bgcolor = "#FEFECC"; # yellow
						} else {
							$bgcolor = "#FFCCCC"; # red
						}
						$col_distrs .= $distrs_present." of ".$distrs_total;
					}
				}
				$col_distrs .= ' (<a href="'.esc_html($pr_link).'" '.$pr_onclick.'>list&hellip;</a>)';
				
				## LSB
				my $col_lsb = "";
				my $status_info = $Analysis::lsb_int_status->{$fullname}{$libname};
				
				my $lsb_status     = $status_info->{STATUS};
				my $lsb_status_ver = $status_info->{STATUS_VER};
				my $reason         = $status_info->{REASON};
				my $data           = $status_info->{DATA};
				
				my $color = undef;
				if ( $lsb_status eq "NOT" || $lsb_status eq "WITHDRAWN" ) {
					my $txt_html = 'No';
					my $color = 'red';
					if ( $reason ) {
						if ( $reason eq 'WRONG_VER' ) {
							$txt_html = 'Wrong&nbsp;ver';
							$color = '#A04020';
						}
						elsif ( $reason eq 'WRONG_LIB' ) {
							my ($best_lib, $best_status, $best_status_ver) = split /,/, ($data or "");
							unless ( $best_status && $best_status eq 'WITHDRAWN' ) {
								$txt_html = 'Wrong&nbsp;lib';
								$color = '#A04020';
							}
						}
						elsif ( $reason eq 'WRONG_LIB_AND_VER' ) {
							$txt_html = 'Wrong&nbsp;lib and&nbsp;version';
							$color = '#A04020';
						}
					}
					$col_lsb .= '<span style="color:'.$color.'">'.$txt_html.'</span>';
				}
				elsif ( $lsb_status eq "CANDIDATE" ) {
					if ( $lsb_status_ver eq 'Unknown' || $lsb_status_ver eq '' ) {
						$col_lsb .= '<span style="color:#907000">Candidate</span>';
					} else {
						$col_lsb .= '<span style="color:#907000">Candidate for '.esc_html($lsb_status_ver).'</span>';
					}
				}
				elsif ( $lsb_status eq "FUTURE" ) {
					$col_lsb .= '<span style="color:#907000">In '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'</span>';
				}
				elsif ( $lsb_status eq "OPTIONAL" ) {
					if ( $filt_show && $filt_show eq 'non-std' ) { next; }
					$col_lsb .= '<span style="color:blue">Optional</span>';
				}
				elsif ( $lsb_status eq "IN" ) {
					if ( $filt_show && $filt_show eq 'non-std' ) { next; }
					$col_lsb .= '<span style="color:green">Yes</span>';
				}
				elsif ( $lsb_status eq "DEPRECATED" ) {
					if ( $filt_show && $filt_show eq 'non-std' ) { next; }
					$col_lsb .= '<span style="color:blue">Deprecated</span>';
				}
				if ( $lsb_status ne "DEPRECATED" ) {
					if ( $filt_show && $filt_show eq 'deprecated' ) { next; }
				}
				
				## More Info
				my $info_link = tab_link('ints').'&int='.esc_url($fullname).'&lib='.esc_url($lib_key).'&com='.esc_url($com);
				my $info_onclick = 'onclick="'.esc_html('return popup_win(\''.esc_js($info_link).'\')').'"';
				my $col_moreinfo = 'Details&hellip;';
				my $attr = get_int_attrs($Iname, $libname);
				if ( $attr ) {
					$col_moreinfo = '<b>More&nbsp;Info&hellip;</b>';
				}
				$col_moreinfo = '<a href="'.esc_html($info_link).'" '.$info_onclick.'>'.$col_moreinfo.'</a>';
				
				# Compile a row
				my $lib_row = "";
				
				$lib_row .= '<td style="white-space:nowrap">';
				$lib_row .= $col_lib;
				$lib_row .= '</td>'.$nl;
				
				$lib_row .= '<td align="right" style="background:'.$bgcolor.';white-space:nowrap">';
				$lib_row .= $col_distrs;
				$lib_row .= '</td>'.$nl;
				
				$lib_row .= '<td align="center">';
				$lib_row .= $col_lsb;
				$lib_row .= '</td>'.$nl;
				
				$lib_row .= '<td>';
				$lib_row .= $col_moreinfo;
				$lib_row .= '</td>'.$nl;
				
				if ( !defined $single_lib ) {
					$single_lib = $libname;
				} else {
					$single_lib = "";
				}
				
				$com_key .= $lib_key."=".(join ",", map {($distrs->{$_}?"":"-").$_} sort keys%$distrs )."|";
				push @lib_rows, $lib_row;
			}
			next if !@lib_rows;
			
			## App modules expandable list
			my $app_components = "";
			foreach my $com ( sort @$coms ) {
				my ($disp_name, $disp_path) = parse_app_component($com);
				
				$app_components .= '<div class="indentwrap" title="'.esc_html($disp_path).'">'.esc_html($disp_name).'</div>';
			}
			my $div_id = 'cl'.($aux_id++);
			my $app_comp_n = scalar @$coms;
			my $col_components = "";
			$col_components .= '<a href="" onclick="return toggle(\''.$div_id.'\')">'
						.$app_comp_n.' component'.( $app_comp_n > 1 ? 's' : '' )
						.' (&hellip;)'
					.'</a>';
			$col_components .= '<div id="'.$div_id.'" style="display:none">'.$nl;
			$col_components .= $app_components;
			$col_components .= '</div>';
			
			my $rowspan = scalar @lib_rows;
			$col_components = '<td'.($rowspan>1 ? ' rowspan="'.$rowspan.'"' : '').'>'.$col_components.'</td>';
			push @com_rows, $col_components.shift(@lib_rows);
			push @com_rows, @lib_rows;
		}
		$com_group = undef;
		next if !@com_rows;
		
		## Interface name column
		my $unmangled = $Analysis::unmangled->{$Iname} || $Iname;
		my $displayname = $unmangled;
		$displayname =~ s/\(/ \(/g;
		$displayname =~ s/\)/\) /g;
		# wrap unmangled name (at _) if it is too long
		my $tmp = "";
		while ( $displayname =~ s/^(\S{0,40}?\s+|\S{30}[^_]{0,20}|\S{1,40}$)// ) {
			$tmp .= $1;
			$tmp .= "\n" if $displayname && $1 !~ /\s$/;
		}
		my $displayname_html = esc_html($tmp);
		$displayname_html =~ s/\n/<br \/>/g;
		
		my $iname_link =  $appchecker_branch{'NAVIGATOR'}.'int_single.php'
			."?cmd=list-by-name&Iname=".esc_url($Iname);
		if ( $single_lib ) {
			$iname_link .= '&Ilibrary='.esc_url(Analysis::soname_to_libname($single_lib));
		}
		
		my $col_iname = '<a href="'.esc_html($iname_link).'" target="_blank"'
				.'>'.$displayname_html.'</a>';
		
		# Print the row
		$table .= '<tr>'.$nl;
		
		my $rowspan = scalar @com_rows;
		$table .= '<td class="indentwrap"'.($rowspan>1 ? ' rowspan="'.$rowspan.'"' : '' ).'>'.$nl;
		$table .= $col_iname;
		$table .= '</td>'.$nl;
			
		$table .= '<td '.($rowspan>1 ? ' rowspan="'.$rowspan.'"' : '' ).'>';
		$table .= $ver;
		$table .= '</td>'.$nl;
		
		$table .= shift @com_rows;
		
		$table .= '</tr>';
		
		$table .= '<tr>'.$_.'</tr>'.$nl foreach @com_rows;
		
		$interface_n++;
	}
	
	# Table Footer
	$table .= '</table>'.$nl;
	
	my $filtered = $filt_com || $filt_lib || $filt_show || $filt_distr;
	if ( $filtered && $interface_n == 0 ) {
		$table .= '<div style="padding:10pt;text-align:center;font-size:large;color:#009">'
					.'<p>No interfaces match the filter.</p>'
				.'</div>';
	}
	
	return ($table, $interface_n);
}

sub get_int_attrs {
	my ($Iname, $libname) = @_;

	return if !$Analysis::int_attrs->{$Iname};

	my $attr;
	if ( $libname ) {
		$attr = $Analysis::int_attrs->{$Iname}{Analysis::soname_to_libname($libname)};
	} else {
		# Use random lib name
		my $any_lib = (keys %{$Analysis::int_attrs->{$Iname}})[0];
		# TODO: try LSB assigned lib first
		$attr = $Analysis::int_attrs->{$Iname}{$any_lib};
	}
	return $attr;
}

sub int_details {
	my ($fullname, $lib_key, $com, $selected_lsb_ver) = @_;
	
	my $nl = "\n";
	my $res = "";
	
	my $ints = ints_data(undef, undef, undef, $fullname);
	
	my $data = $ints->{$fullname}{$com}{$lib_key}
		or return "Error: Wrong parameters: '$fullname' '$lib_key' '$com'";
	
	my ($Iname, $ver) = split /\@/, $fullname;
	$ver ||= "";
	my $unmangled = $Analysis::unmangled->{$Iname} || $Iname;
	
	my $libname = $data->{LIBRARY} || $lib_key;
	my $Lname = Analysis::soname_to_libname($libname);
	
	# Reference to the LSB Navigator:
	my $navi_link = $appchecker_branch{'NAVIGATOR'}.'int_single.php'
			."?cmd=list-by-name&Iname=".esc_url($Iname).($Lname ? '&Ilibrary='.esc_url($Lname) : '');
	# Keep synchronized with int_distr_list()
	# Title
	$res .= '<h1>Interface <span class="title-highlight">'.esc_html($unmangled).'</span>'
			.' ('.($ver ? esc_html($ver).', ' : '')
				.($libname ? '<a href="'.esc_html(tab_link('libs').'&lib='.esc_url($libname)).'">'.esc_html($libname).'</a>'
				          : '<span style="color:red">non-linked</span>')
			.')'.'</h1>';
	$res .= '<p>Look up this interface <a href="'.esc_html($navi_link).'" target="_blank">in the '.$appchecker_branch{'STD'}.' Navigator</a>.</p>';
	if ( $Analysis::unmangled->{$Iname} ) {
		$res .= '<p><b>C++ mangled name:</b> '.esc_html($Iname).'</i></p>';
	}
	$res .= '<hr />'.$nl;
	
	# Int KB info still not implemented
	
	# Check IntAttr
	my $attr = get_int_attrs($Iname, $libname);
	if ( $attr ) {
		foreach my $type ( sort keys %$attr ) {
			my $attr_html = "";
			my $bgcolor = '#fec';
			if ( $type eq "Recommendation" ) {
				$attr_html .= '<h3>Recommendation</h3>';
				$attr_html .= esc_html($attr->{$type});
			}
			elsif ( $type eq "Rejection" ) {
				$attr_html .= '<h3>Rejection</h3>';
				$attr_html .= esc_html($attr->{$type});
			}
			else { # Custom
				$attr_html .= esc_html($attr->{$type});
			}
			$attr_html =~ s/^\s+//; # remove unnecessary spaces
			$attr_html =~ s/\s+$//; #
			$attr_html .= "." if $attr_html =~ /[^\s\.]\s*$/;
			
			$res .= '<div class="box" style="background:'.$bgcolor.'">'.$attr_html.'</div>';
		}

		$res .= '<hr />'.$nl;
	}

	# Compliance Status
	$res .= '<h2>'.$appchecker_branch{'STD'}.' Compliance Status:</h2>'.$nl;
	
	my $lsb_res = '';
	my $lsb_color = 'red';
	
	my $lsb_status_obj = $Analysis::lsb_int_status->{$fullname}{$libname};
	my ($lsb_status, $lsb_status_ver) = @$lsb_status_obj{qw/STATUS STATUS_VER/};
	
	if ( $lsb_status eq "NOT" ) {
		if ( !$lsb_status_obj->{REASON} ) {
			$lsb_res .= 'Never been in '.$appchecker_branch{'STD'}.'.';
		} else {
			my $reason = $lsb_status_obj->{REASON};
			my $data = $lsb_status_obj->{DATA} || "";
			
			if ( $reason eq 'WRONG_LIB' ) {
				my ($best_lib, $best_status, $best_status_ver) = split /,/, $data;
				
				if ( $best_status && $best_status eq 'CANDIDATE' ) {
					if ( $best_status_ver eq 'Unknown' || $best_status_ver eq '' ) {
						$best_status_ver = 'future '.$appchecker_branch{'STD'}.' versions';
					} else {
						$best_status_ver = $appchecker_branch{'STD'}.' '.$best_status_ver;
					}
					$lsb_res .= "This interface is assigned to the '".esc_html($best_lib)."' library as a candidate for ".esc_html($best_status_ver).".";
				}
				elsif ( $best_status && $best_status eq 'FUTURE' ) {
					$lsb_res .= "This interface appeared in $appchecker_branch{'STD'} ".esc_html($best_status_ver)." in library '".esc_html($best_lib)."'.";
				}
				elsif ( $best_status && $best_status eq 'DEPRECATED' ) {
					$lsb_res .= "This interface is assigned to the '".esc_html($best_lib)."' library and is deprecated since $appchecker_branch{'STD'} ".esc_html($best_status_ver).".";
				}
				elsif ( $best_status && $best_status eq 'WITHDRAWN' ) {
					$lsb_res .= "This interface was assigned to the '".esc_html($best_lib)." library and withdrawn in $appchecker_branch{'STD'} ".esc_html($best_status_ver).".";
				}
				elsif ( $best_lib ) {
					$lsb_res .= "This interface is included in $appchecker_branch{'STD'} but is assigned to the '".esc_html($best_lib)."' library.";
				}
				else {
					$lsb_res .= 'This interface is included in '.$appchecker_branch{'STD'}.' but is assigned to another library.';
				}
				
				$lsb_res .= '<br/>Look up this interface in the '.$appchecker_branch{'STD'}.' Navigator for more details.';
			}
			elsif ( $reason eq 'WRONG_VER' ) {
				my ($best_ver, $best_status, $best_status_ver) = split /,/, $data;
				
				if ( $best_ver eq "" && defined $best_status ) {
					$best_ver = "NO version";
				}
				
				if ( $best_status && $best_status eq 'CANDIDATE' ) {
					if ( $best_status_ver eq 'Unknown' || $best_status_ver eq '' ) {
						$best_status_ver = 'future '.$appchecker_branch{'STD'}.' versions';
					} else {
						$best_status_ver = $appchecker_branch{'STD'}.' '.$best_status_ver;
					}
					if ( $ver eq "" && $best_ver ne "" ) {
						$lsb_res .= "The application doesn't specify the required version of this interface. It's a possible source of errors in future because the specific version of this interface (".esc_html($best_ver).") is a candidate for ".esc_html($best_status_ver).".";
					} else {
						$lsb_res .= "The application requires version '".esc_html($ver)."' of this interface. This may cause problems in future because another version of this interface (".esc_html($best_ver).") is a candidate for ".esc_html($best_status_ver).".";
					}
				}
				elsif ( $best_status && $best_status eq 'FUTURE' ) {
					if ( $ver eq "" && $best_ver ne "" ) {
						$lsb_res .= "The application doesn't specify the required version of this interface. It's a possible source of errors in future because the specific version of this interface (".esc_html($best_ver).") is required by $appchecker_branch{'STD'} ".esc_html($best_status_ver).".";
					} else {
						$lsb_res .= "The application requires version '".esc_html($ver)."' of this interface. This may cause problems in future because another version of this interface (".esc_html($best_ver).") is required by $appchecker_branch{'STD'} ".esc_html($best_status_ver).".";
					}
				}
				elsif ( $best_status && $best_status eq 'IN' ) {
					if ( $ver eq "" && $best_ver ne "" ) {
						$lsb_res .= "The application doesn't specify the required version of this interface. It's a possible source of errors because the specific version of this interface (".esc_html($best_ver).") is required by $appchecker_branch{'STD'} ".esc_html($best_status_ver).".";
					} else {
						$lsb_res .= "The application requires version '".esc_html($ver)."' of this interface. Another version of this interface (".esc_html($best_ver).") is required by the standard since $appchecker_branch{'STD'} ".esc_html($best_status_ver).".";
					}
				}
				elsif ( $best_ver ) {
					if ( $ver eq "" && $best_ver ne "" ) {
						$lsb_res .= "The application doesn't specify the required version of this interface. It's a possible source of errors because the specific version of this interface (".esc_html($best_ver).") is required by $appchecker_branch{'STD'}.";
					} else {
						$lsb_res .= "The application requires version '".esc_html($ver)."' of this interface. Another version of this interface (".esc_html($best_ver).") is included in $appchecker_branch{'STD'}.";
					}
				}
				else {
					$lsb_res .= 'Another version of this interface is required by '.$appchecker_branch{'STD'}.'.';
				}
				
				$lsb_res .= '<br/>Look up this interface in the '.$appchecker_branch{'STD'}.' Navigator for more details.';
			}
			elsif ( $reason eq 'WRONG_LIB_AND_VER' ) {
				my ($best_lib, $best_ver, $best_status, $best_status_ver) = split /,/, $data;

				if ( $ver eq "" && $best_ver ne "" ) {
					$lsb_res .= "The application takes this interface from the '".esc_html($libname)."' library without specifying the version of this interface. ";
				} else {
					$lsb_res .= "The application requires version '".esc_html($ver)."' of this interface from the '".esc_html($libname)."' library. ";
				}

				if ( $best_status && $best_status eq 'CANDIDATE' ) {
					if ( $best_status_ver eq 'Unknown' || $best_status_ver eq '' ) {
						$best_status_ver = 'future '.$appchecker_branch{'STD'}.' versions';
					} else {
						$best_status_ver = $appchecker_branch{'STD'}.' '.$best_status_ver;
					}
					$lsb_res .= "But the standard assigns this interface of version '".esc_html($best_ver)."' to the '".esc_html($best_lib)."' library as a candidate for ".esc_html($best_status_ver).".";
				}
				elsif ( $best_status && $best_status eq 'FUTURE' ) {
					$lsb_res .= "But in $appchecker_branch{'STD'} '".esc_html($best_status_ver)."' this interface of version '".esc_html($best_ver)."' is assigned to the '".esc_html($best_lib)."' library.";
				}
				elsif ( $best_status && $best_status eq 'IN' ) {
					$lsb_res .= "But since $appchecker_branch{'STD'} '".esc_html($best_status_ver)."' this interface of version '".esc_html($best_ver)."' is assigned to the '".esc_html($best_lib)."' library.";
				}
				elsif ( $best_ver && $best_lib ) {
					$lsb_res .= "But the $appchecker_branch{'STD'} standard assigns this interface of version '".esc_html($best_ver)."' to the '".esc_html($best_lib)."' library.";
				}
				else {
					$lsb_res .= "But the $appchecker_branch{'STD'} standard assigns this interface to the '".esc_html($best_lib)."' library.";
				}
				
				$lsb_res .= '<br/>Look up this interface in the '.$appchecker_branch{'STD'}.' Navigator for more details.';
			}
			else {
				# Unhandled reason
				$lsb_res .= 'Never been in '.$appchecker_branch{'STD'}.'.';
			}
		}
	}
	elsif ( $lsb_status eq "FUTURE" ) {
		$lsb_color = 'red';
		$lsb_res .= 'Not in '.$appchecker_branch{'STD'}.' '.esc_html($selected_lsb_ver).'. (Appeared in '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.)';
	}
	elsif ( $lsb_status eq "CANDIDATE" ) {
		if ( $lsb_status_ver eq 'Unknown' || $lsb_status_ver eq '' ) {
			$lsb_status_ver = 'future '.$appchecker_branch{'STD'}.' versions';
		} else {
			$lsb_status_ver = $appchecker_branch{'STD'}.' '.$lsb_status_ver;
		}
		$lsb_color = 'red';
		$lsb_res .= 'Not in '.$appchecker_branch{'STD'}.'. Candidate for '.esc_html($lsb_status_ver).'.';
	}
	elsif ( $lsb_status eq "OPTIONAL" ) {
		$lsb_color = 'blue';
		$lsb_res .= 'Optional since '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.';
	}
	elsif ( $lsb_status eq "IN" ) {
		$lsb_color = 'green';
		$lsb_res .= 'Mandatory since '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.';
	}
	elsif ( $lsb_status eq "DEPRECATED" ) {
		$lsb_color = 'blue';
		$lsb_res .= 'Deprecated since '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.';
	}
	elsif ( $lsb_status eq "WITHDRAWN" ) {
		$lsb_color = 'red';
		$lsb_res .= 'Withdrawn since '.$appchecker_branch{'STD'}.' '.esc_html($lsb_status_ver).'.';
	}
	if ( $lsb_status_obj->{REASON} && $lsb_status_obj->{REASON} eq 'BETTER' ) {
		my ($best_lib, $best_status, $best_status_ver) = split /,/, $lsb_status_obj->{DATA};
		$lsb_res .= " It is better to use this interface from the '".esc_html($best_lib)."' library.";
	}
	
	$res .= '<div style="color:'.$lsb_color.'">';
	$res .= $lsb_res;
	$res .= '</div>'.$nl;
	
	$res .= '<hr />'.$nl;
	
	my $com_key = int_com_key( $ints->{$fullname}{$com} );
	my @coms = grep { int_com_key($ints->{$fullname}{$_}) eq $com_key } keys %{$ints->{$fullname}};
	
	$res .= '<h2>This interface is used by the following aplication components:</h2>';

	$res .= '<ul>';
	foreach my $com ( sort @coms ) {
		my ($disp_name, $disp_path) = parse_app_component($com);
		$res .= '<li>';
		my $com_link = tab_link('com').'&com='.esc_url($com);
		$res .= '<a href="'.esc_html($com_link).'"'
			.' title="'.esc_html($disp_path).'"><b>'.esc_html($disp_name).'</b></a>';
		$res .= '</li>';
	}
	$res .= '</ul>';
	
	$res .= '<hr />'.$nl;
	
	return $res;
} # end of  int_details
#-----------------------------------------------------------------------

sub int_com_key {
	my ($int_com) = @_;
	
	my $com_key = "";
	foreach my $lib_key ( keys %$int_com ) {
		my $distrs = $int_com->{$lib_key}{DISTRS};
		$com_key .= "$lib_key=".(join ",", map {($distrs->{$_}?"":"-").$_} sort keys %$distrs )."|";
	}
	return $com_key;
}
#-----------------------------------------------------------------------

sub int_distr_list {
	my ($fullname, $com, $lib_key, $distr_sel, $filt_com, $filt_lib) = @_;
	
	my $nl = "\n";
	my $res = "";

	$filt_com = undef if $filt_com eq "";
	$filt_lib = undef if $filt_lib eq "";
	my $ints = ints_data($distr_sel, $filt_com, $filt_lib, $fullname);
	
	my ($Iname, $ver) = split /\@/, $fullname, 2;
	$ver ||= "";
	my $unmangled = $Analysis::unmangled->{$Iname} || $Iname;
	my $libname = $ints->{$fullname}{$com}{$lib_key}{LIBRARY} || $lib_key;
	my $Lname = Analysis::soname_to_libname($libname);
	
	# Reference to the LSB Navigator:
	my $navi_link = $appchecker_branch{'NAVIGATOR'}.'int_single.php'
			."?cmd=list-by-name&Iname=".esc_url($Iname).($Lname ? '&Ilibrary='.esc_url($Lname) : '');
	# Keep synchronized with int_details()
	# Title
	$res .= '<h1>Interface <span class="title-highlight">'.esc_html($unmangled).'</span>'
			.' ('.($ver ? esc_html($ver).', ' : '')
				.($libname ? '<a href="'.esc_html(tab_link('libs').'&lib='.esc_url($libname)).'">'.esc_html($libname).'</a>'
				          : '<span style="color:red">non-linked</span>')
			.')'.'</h1>';
	$res .= '<p>Look up this interface <a href="'.esc_html($navi_link).'" target="_blank">in the '.$appchecker_branch{'STD'}.' Navigator</a>.</p>';
	if ( $Analysis::unmangled->{$Iname} ) {
		$res .= '<p><b>C++ mangled name:</b> '.esc_html($Iname).'</i></p>';
	}
	$res .= '<hr />'.$nl;
	
	# Presence in Distributions
	$res .= '<h2>Presence in Distributions</h2>';
	
	my $filtered = scalar grep {!$_} values %$distr_sel;
	$res .= int_distr_list_sub( $ints, $fullname, $com, $lib_key, $filtered );
	
	$res .= '<hr />'.$nl;

	my $com_key = int_com_key( $ints->{$fullname}{$com} );
	my @coms = grep { int_com_key($ints->{$fullname}{$_}) eq $com_key } keys %{$ints->{$fullname}};
	
	$res .= '<h2>This interface is used by the following aplication components:</h2>';

	$res .= '<ul>';
	foreach my $com ( sort @coms ) {
		my ($disp_name, $disp_path) = parse_app_component($com);
		$res .= '<li>';
		my $com_link = tab_link('com').'&com='.esc_url($com);
		$res .= '<a href="'.esc_html($com_link).'"'
			.' title="'.esc_html($disp_path).'"><b>'.esc_html($disp_name).'</b></a>';
		$res .= '</li>';
	}
	$res .= '</ul>';
	
	$res .= '<hr />'.$nl;
}

sub int_distr_list_sub {
	my ($ints, $fullname, $com, $lib_key, $filtered) = @_;
	
	my $nl = "\n";
	my $res = "";
	
	my $data = $ints->{$fullname}{$com}{$lib_key} or return "";
	
	if ( $data->{UNAPPROVED} ) {
		return '<p><i>No information.</i></p>';
	}
	
	my $total_num = 0;
	my %present = ();
	my %missing = ();
	foreach my $Did ( keys %{$data->{DISTRS}} ) {
		my $distr_name = $Analysis::all_distrs->{$Did}{FULLNAME};
		if ( $data->{DISTRS}{$Did} ) {
			$present{$distr_name} = $Did;
		} else {
			$missing{$distr_name} = $Did;
		}
		$total_num++;
	}
	my $present_num = scalar keys %present;
	my $missing_num = scalar keys %missing;
	
	if ( $present_num > 0 ) {
		$res .= '<p>';
		$res .= 'Present in <span style="color:green">'.$present_num.'</span>';
		$res .= ' and missing in <span style="color:red">'.$missing_num.'</span>';
		if ( $filtered ) {
			$res .= ' of '.$total_num.' selected distributions.';
		} else {
			$res .= ' of total '.$total_num.' distributions.';
		}
		$res .= '</p>'.$nl;
	} else {
		$res .= '<p>This interface is missing in '.$total_num;
		if ( $filtered ) {
			$res .= ' selected distributions.</p>'.$nl;
		} else {
			$res .= ' analyzed distributions.</p>'.$nl;
		}
	}
	
	if ( %present ) {
		$res .= '<h3>Present in:</h3>';
		$res .= '<ul>';
		foreach my $distr_name ( sort {lc $a cmp lc $b} keys %present ) {
			my $Did = $present{$distr_name};
			
			my $distr_link = tab_link('distrs').'&id='.$Did;
						
			$res .= '<li><a style="color:green" href="'.esc_html($distr_link).'">'
						.esc_html($distr_name).'</a>';
			$res .= '</li>';
		}
		$res .= '</ul>';
	}
	
	if ( %missing ) {
		$res .= '<h3>Missing in:</h3>';
		$res .= '<ul>';
		foreach my $distr_name ( sort {lc $a cmp lc $b} keys %missing ) {
			my $Did = $missing{$distr_name};
			
			my $distr_link = tab_link('distrs').'&id='.$Did;
			
			$res .= '<li><a style="color:red" href="'.esc_html($distr_link).'">'
						.esc_html($distr_name).'</a></li>';
		}
		$res .= '</ul>';
	}
	
	return $res;
} # end of  int_distr_list

#-----------------------------------------------------------------------

sub C_tab {
	my ($selected_lsb_ver) = @_;
	
	my $res = "";
	my $nl = "\n";

	#my $distr_sel = get_distr_sel();
	my $distr_sel = $Analysis::all_distrs;

	my $app_components = $Analysis::app_components;
	foreach my $com_obj ( values %$app_components ) {
		Analysis::com_data( $com_obj ); # TODO: use distr_sel
		Analysis::com_data_lsb( $com_obj );
	}
	
	# Check if the table is not empty
	if ( !%$app_components ) {
		$res .= no_binary_div();
		return $res;
	}
	
	##
	
	my $link = tab_link('com');
	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[

function popup_win( link )
{
	var w = window.open( link, 'details',
			'toolbar=0,menubar=0,scrollbars=1,height=750,width=850,dependent=yes,dialog=yes'
		);
	w.moveTo(100,0);
	w.focus();

	return false;
}
// ]]>
</script>
HEREDOC

	$res .= <<HEREDOC;
<p>The table below lists all the binary components of your Application with their individual compatibility statuses.</p>
HEREDOC

	local *_pl = sub {
		my ($num, $mid, $singular, $plural) = @_;
		
		return '<b>'.$num.'</b> '.($mid ? $mid.' ' : '').($num > 1 ? $plural : $singular);
	};
	
	local *refer = sub {
		my ($com, $text_html, $tab, $add, $color) = @_;
		
		my $link = tab_link($tab).'&nofilt=1&filt_com='.esc_url($com);
		$link .= $add if ( $add );
		return '<a href="'.esc_html($link).'"'
			.($color?' style="color:'.esc_html($color).'"':'')
			.'>'.$text_html.'</a>';
	};
	
	local *add_br = sub {
		$_[0] .= '<br/>' if $_[0];
	};

	$res .= '<table id="i_table" class="wikitable" width="100%">'.$nl;
	
	$res .= '<tr>'
			.'<th>Component</th>'
			.'<th>Distribution<br/>Compatibility</th>'
			.'<th>Missing<br/>Libraries</th>'
			.'<th>Missing<br/>Interfaces</th>'
			.'<th>Unneeded<br/>Libraries</th>'
			.'<th>'.$appchecker_branch{'STD'}.'&nbsp;'.esc_html($selected_lsb_ver).'<br/>Compatibility</th>'
			.'<th>Comments</th>'
		.'</tr>'.$nl;
	
	foreach my $com ( sort keys %$app_components ) {
		my $com_obj = $app_components->{$com};
		
		$res .= '<tr>'.$nl;
		
		# Component
		$res .= '<td class="indentwrap">';
		my ($disp_name, $disp_path) = parse_app_component($com);
		$res .= '<span title="'.esc_html($disp_path).'">'.esc_html($disp_name).'</span>';
		$res .= '</td>'.$nl;
		
		# Distribution Compatibility
		my $all = scalar keys %$Analysis::all_distrs;
		
		my ($missing, $unknown, $ok) = (0) x 4;
		foreach my $Did ( keys %$Analysis::all_distrs ) {
			Analysis::com_distr_status( $com_obj, $Did );
			my $dstatus = $com_obj->{$Did}{STATUS};
			if ( $dstatus eq 'MISSING' || $dstatus eq 'UNNEEDED' ) { $missing++ }
			elsif ( $dstatus eq 'UNKNOWN' || $dstatus eq '' ) { $unknown++ }
			elsif ( $dstatus eq 'OK' ) { $ok++ }
		}
		my $bgcolor = ( $ok + $unknown == 0 ? '#FFCCCC' : $ok == $all ? '#CCFFCC' : '#FEFECC' );
		my $info_link = tab_link('com').'&com='.esc_url($com);
		my $info_onclick = 'onclick="'.esc_html('return popup_win(\''.esc_js($info_link).'\')').'"';
		$res .= '<td style="text-align:right;background:'.$bgcolor.'">';
		if ( $ok == 0 && $unknown > 0 ) {
			$res .= '<span style="white-space:nowrap">'.$unknown.' of '.$all.' unknown';
			$res .= ' (<a href="'.esc_html($info_link).'" '.$info_onclick.'>list&hellip;</a>)';
			$res .= '</span>';
		} else {
			$res .= '<span style="white-space:nowrap">'.$ok.' of '.$all.'';
			$res .= ' (<a href="'.esc_html($info_link).'" '.$info_onclick.'>list&hellip;</a>)';
			$res .= '</span>';
			if ( $unknown ) { $res .= ',<br/>+'.$unknown.' unknown'; }
		}
		$res .= '</td>'.$nl;
		
		# Missing Libraries
		$all = @{$com_obj->{NEEDED_LIBS}};
		my $x = 0; my $un = 0;
		foreach my $status ( values %{$com_obj->{LIB_STATUS}} ) {
			if ( $status eq 'MISSING' || $status eq 'UNNEEDED' ) { $x++ }
			elsif ( $status eq 'UNKNOWN' ) { $un++ }
		}
		$bgcolor = ( $x + $un == 0 ? '#CCFFCC' : $x == 0 ? '#FEFECC' : '#FFCCCC' );
			$res .= '<td style="text-align:right;background:'.$bgcolor.';white-space:nowrap">';
		if ( $x + $un == 0 ) {
			$res .= refer($com, 'none', 'libs', '&nofilt=1' );
		} elsif ( $x == 0 ) {
			$res .= $un.' unknown ('.refer($com, '&hellip;', 'libs', '&filt_show=problems').')';
		} else {
			$res .= $x.' of '.$all.' ('.refer($com, '&hellip;', 'libs', '&filt_show=problems').')';
			if ( $un ) { $res .= ',<br/>+'.$un.' unknown'; }
		}
		$res .= '</td>'.$nl;
		
		# Missing Interfaces
		$x = 0; $un = 0;
		$all = keys %{$com_obj->{NEEDED_INTS}};
		foreach my $status ( values %{$com_obj->{NEEDED_INTS}} ) {
			if ( $status eq 'MISSING' || $status eq 'UNNEEDED' ) { $x++ }
			elsif ( $status eq 'UNKNOWN' ) { $un++ }
		}
		$bgcolor = ( $x + $un == 0 ? '#CCFFCC' : $x == 0 ? '#FEFECC' : '#FFCCCC' );
		$res .= '<td style="text-align:right;background:'.$bgcolor.';white-space:nowrap">';
		if ( $x + $un == 0 ) {
			$res .= refer($com, 'none', 'ints', '&nofilt=1' );
		} elsif ( $x == 0 ) {
			$res .= $un.' unknown ('.refer($com, '&hellip;', 'ints', '&filt_show=problems').')';
		}
		 else {
			$res .= $x.' of '.$all.' ('.refer($com, '&hellip;', 'ints', '&filt_show=problems').')';
			if ( $un ) { $res .= ',<br/>+'.$un.' unknown'; }
		}
		$res .= '</td>'.$nl;
		
		# Unneeded libraries
		$x = grep {$_} values %{$com_obj->{LIBS_UNNEEDED}};
		$res .= '<td style="text-align:center">';
		if ( $x ) {
			$res .= '<span style="color:red"><b>'.$x.'</b></span>&nbsp;';
			$res .= '('.refer($com, '&hellip;', 'libs', '&filt_show=unneeded' ).')';
		}
		$res .= '</td>'.$nl;
		
		## LSB status
		my $lsb_status = "";
		# LSB status for Libraries
		my $lib_lsb_status = $com_obj->{LSB_STATUS}{LIBS};
		my %statuses;
		@statuses{qw/NOT CANDIDATE FUTURE IN OPTIONAL DEPRECATED WITHDRAWN/} = (0) x 7; # initialize
		foreach my $status_obj ( values %$lib_lsb_status ) {
			$statuses{ $status_obj->{STATUS} }++;
		}
		if ( $statuses{'NOT'} || $statuses{'WITHDRAWN'} ) {
			my $x = $statuses{'NOT'} + $statuses{'WITHDRAWN'} + $statuses{'CANDIDATE'} + $statuses{'FUTURE'};
			my $text = _pl($x, 'non-'.$appchecker_branch{'STD'}, 'library', 'libraries');
			add_br( $lsb_status );
			$lsb_status .= refer($com, $text, 'libs', '&filt_show=non-std', '#DD0000'); # Red
		}
		elsif ( $statuses{'CANDIDATE'} ) {
			#my $cand_version = (sort map {$_->{STATUS_VER}} grep {$_->{STATUS} eq 'CANDIDATE'} values %$lib_lsb_status)[-1]; # max
			my $text = "";
			if ( $statuses{'FUTURE'} ) {
				my $x = $statuses{'FUTURE'} + $statuses{'CANDIDATE'};
				$text = _pl($x, '', 'library', 'libraries')
						.' appeared in future '.$appchecker_branch{'STD'}.' versions or are candidate'.($x==1?'':'s').'.';
			} else {
				my $x = $statuses{'CANDIDATE'};
				$text = _pl($x, '', 'library is a candidate', 'libraries are candidates')
						.' for future '.$appchecker_branch{'STD'}.' versions';
			}
			add_br( $lsb_status );
			$lsb_status .= refer( $com, $text, 'libs', '&filt_show=non-std', '#907000'); # Yellow
		}
		elsif ( $statuses{'FUTURE'} ) {
			my $future_version = (sort map {$_->{STATUS_VER}} grep {$_->{STATUS} eq 'FUTURE'} values %$lib_lsb_status)[-1]; # max
			my $x = $statuses{'FUTURE'};
			my $text = _pl($x, '', 'library is', 'libraries are')
				.' since '.$appchecker_branch{'STD'}.'&nbsp;'.esc_html($future_version);
			add_br( $lsb_status );
			$lsb_status .= refer( $com, $text, 'libs', '&filt_show=non-std', '#907000'); # Yellow
		}
		if ( $statuses{'DEPRECATED'} ) {
			my $x = $statuses{'DEPRECATED'};
			my $text = _pl($x, 'deprecated', 'library', 'libraries');
			add_br( $lsb_status );
			$lsb_status .= refer( $com, $text, 'libs', '&filt_show=deprecated', '#907000'); # Yellow
		}
		# LSB status for Interfaces
		my $int_lsb_status = $com_obj->{LSB_STATUS}{INTS};
		@statuses{qw/NOT CANDIDATE FUTURE IN OPTIONAL DEPRECATED WITHDRAWN/} = (0) x 7; # initialize
		foreach my $status_obj ( values %$int_lsb_status ) {
			$statuses{ $status_obj->{STATUS} }++;
		}
		if ( $statuses{'NOT'} || $statuses{'WITHDRAWN'} ) {
			my $x = $statuses{'NOT'} + $statuses{'WITHDRAWN'} + $statuses{'CANDIDATE'} + $statuses{'FUTURE'};
			my $text = _pl($x, 'non-'.$appchecker_branch{'STD'}, 'interface', 'interfaces');
			add_br( $lsb_status );
			$lsb_status .= refer( $com, $text, 'ints', '&filt_show=non-std', '#DD0000'); # Red
		}
		elsif ( $statuses{'CANDIDATE'} ) {
			#my $cand_version = (sort map {$_->{STATUS_VER}} grep {$_->{STATUS} eq 'CANDIDATE'} values %$int_lsb_status)[-1]; # max
			my $text = "";
			if ( $statuses{'FUTURE'} ) {
				my $x = $statuses{'FUTURE'} + $statuses{'CANDIDATE'};
				$text = _pl($x, '', 'interface', 'interfaces')
						.' appeared in future '.$appchecker_branch{'STD'}.' versions or are candidate'.($x==1?'':'s').'.';
			} else {
				my $x = $statuses{'CANDIDATE'};
				$text = _pl($x, '', 'interface is a candidate', 'interfaces are candidates')
						.' for future '.$appchecker_branch{'STD'}.' versions';
			}
			add_br( $lsb_status );
			$lsb_status .= refer( $com, $text, 'ints', '&filt_show=non-std', '#907000'); # Yellow
		}
		elsif ( $statuses{'FUTURE'} ) {
			my $future_version = (sort map {$_->{STATUS_VER}} grep {$_->{STATUS} eq 'FUTURE'} values %$int_lsb_status)[-1]; # max
			my $x = $statuses{'FUTURE'};
			my $text = _pl($x, '', 'interface is', 'interfaces are')
				.' since '.$appchecker_branch{'STD'}.'&nbsp;'.esc_html($future_version);
			add_br( $lsb_status );
			$lsb_status .= refer( $com, $text, 'ints', '&filt_show=non-std', '#907000'); # Yellow			
		}
		if ( $statuses{'DEPRECATED'} ) {
			my $x = $statuses{'DEPRECATED'};
			my $text = _pl($x, 'deprecated', 'interface', 'interfaces');
			add_br( $lsb_status );
			$lsb_status .= refer( $com, $text, 'ints', '&filt_show=deprecated', '#907000'); # Yellow
		}
		if ( !$lsb_status ) {
			$lsb_status = '<span style="color:green">OK</span>';
		}
		$res .= '<td style="text-align:center">';
		$res .= $lsb_status;
		$res .= '</td>'.$nl;
		
		# Comments
		my $comments = "";
		if ($appchecker_branch{'STD'} eq 'LSB') {
			if ( $com_obj->{NO_SHT_HASH} ) {
				$comments .= '<br/>' if $comments;
				$comments .= '<span style="white-space:nowrap">No .hash section!';
				$comments .= '&nbsp;<a href="" onclick="javascript:show_hide_tooltip(this, \'com_no_hash_section\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a></span>';
			}
		}
		
		Analysis::com_common_data($com_obj);
		if ( $com_obj->{COMMON} ) {
			$comments .= '<br/>' if $comments;
			$comments .= '<span style="white-space:nowrap">A common library</span>';
			$comments .= '<span style="white-space:nowrap">';
			my $x = scalar grep { $distr_sel->{$_} } keys %{$com_obj->{COMMON_DISTRS}};
			my $all = scalar grep { $distr_sel->{$_} } keys %$distr_sel;
			my $link = tab_link('com').'&com='.esc_url($com).'&common=1';
			my $onclick = 'onclick="'.esc_html('return popup_win(\''.esc_js($link).'\')').'"';
			$comments .= ' (<a href="'.esc_html($link).'" '.$onclick.'>'.$x.' of '.$all.'</a>)';
			# Tooltip:
			$comments .= ' <a href="" onclick="javascript:show_hide_tooltip(this, \'com_common_lib\', event); return false;"><img src="themes/'.$appchecker_branch{'DIR'}.'/images/question.png" width="16" height="16" alt="?" /></a>';
			$comments .= '</span>';
		}
		$res .= '<td align="right">';
		$res .= $comments;
		$res .= '</td>'.$nl;
		
		$res .= '</tr>'.$nl;
	}
	
	$res .= '</table>'.$nl;
	
	return $res;
}

# Contents of a details popup window for Components table
sub com_details {
	my ($com) = @_;

	my $nl = "\n";
	my $res = "";

	my $com_obj = $Analysis::app_components->{$com};
	if ( !$com_obj ) {
		$res .= "Error: there is no such component: '".esc_html($com)."'";
		return $res;
	}
	Analysis::com_data( $com_obj );
	
	my ($disp_name, $disp_path) = parse_app_component($com);
	
	$res .= '<h1>Application&nbsp;Component<br/><span class="title-highlight">'.esc_html($disp_name).'</span></h1>'.$nl;
	
	if ( $disp_path ne $disp_name ) {
		$res .= '<p><b>Full path:</b> '.esc_html($disp_path).'</p>';
	}
	
	$res .= '<hr/>'.$nl;
	
	if ($appchecker_branch{'STD'} eq 'LSB') {
		if ( $com_obj->{NO_SHT_HASH} ) {
			$res .= '<h2>No \'.hash\' section!</h2>'.$nl;
			$res .= '<p>This application component has no <b>.hash</b> section. It would fail to run on older distributions (e.g., RHEL4).</p>'.$nl;
			$res .= '<p>Which section is present in the binary is determined by the linker switch \'--hash-style\' at compile time.</p>'.$nl;
			$res .= '<hr/>';
		}
	}
	
	$res .= '<h2>Distribution Compatibility</h2>'.$nl;
	$res .= '<p>The table below shows the compatibility information for this application component.</p>'.$nl;
	$res .= '<table style="border-collapse:collapse" border="1">'.$nl;
	$res .= '<tr>'
			.'<th rowspan="2">Distribution</th>'
			.'<th colspan="2">Libraries</th>'
			.'<th colspan="2">Interfaces</th>'
			.((($appchecker_branch{'STD'} eq 'LSB') && $com_obj->{NO_SHT_HASH}) ? '<th rowspan="2">Comments</th>' : '' )
		.'</tr>'.$nl;
	$res .= '<tr>'
			.'<th>Missing</th>'
			.'<th>Unknown</th>'
			.'<th>Missing</th>'
			.'<th>Unknown</th>'
		.'</tr>'.$nl;
	my $all_distrs = $Analysis::all_distrs;
	my @distrs = sort {lc $all_distrs->{$a}{FULLNAME} cmp lc $all_distrs->{$b}{FULLNAME}} keys %$all_distrs;
	foreach my $Did ( @distrs ) {
		Analysis::com_distr_status( $com_obj, $Did );
		
		my $status = $com_obj->{$Did}{STATUS};
		my $bgcolor = "";
		if ( $status eq 'OK' ) {
			$bgcolor = "#ccffcc"; # green
		} elsif ( $status eq 'UNNEEDED' ) {
			$bgcolor = "#fefecc"; # yellow
		} elsif ( $status eq 'UNKNOWN' ) {
			$bgcolor = "#fefecc"; # yellow
		} elsif ( $status eq 'MISSING' ) {
			$bgcolor = "#ffdddd"; # red
		}
		my $link = tab_link('com').'&com='.esc_url($com).'&distr='.esc_url($Did);
		
		# Component Info
		my ($missing_libs, $unneeded_libs, $unknown_libs, $missing_ints, $unknown_ints) = (0) x 5;
		foreach my $req_soname ( @{$com_obj->{NEEDED_LIBS}} ) {
			my $val = $com_obj->{$Did}{LIBS}{$req_soname}{STATUS};
			if ( $val eq 'MISSING' ) { $missing_libs++; }
			if ( $val eq 'UNNEEDED' ) { $unneeded_libs++; }
			if ( $val eq 'UNKNOWN' ) { $unknown_libs++; }
		}
		foreach my $val ( values %{$com_obj->{$Did}{INT_STATUS}} ) {
			if ( $val eq 'MISSING' ) { $missing_ints++ }
			if ( $val eq 'UNKNOWN' ) { $unknown_ints++; }
		}
		
		$res .= '<tr style="background:'.$bgcolor.'">';
		$res .= '<td>';
		$res .= '<a href="'.esc_html($link).'">'.esc_html($all_distrs->{$Did}{FULLNAME}).'</a>';
		$res .= '</td>';
		
		$res .= '<td>';
		if ( $missing_libs ) {
			$res .= '<span style="color:red"><b>'.$missing_libs.'</b></span>';
			$res .= ' (+<span style="color:red"><b>'.$unneeded_libs.'</b></span> unneeded)' if $unneeded_libs;
		} elsif ( $unneeded_libs ) {
			$res .= '<span style="color:#907000">';
			$res .= '<b>'.$unneeded_libs.'</b> unneeded';
			$res .= '</span>';
		}
		$res .= '</td>';
		$res .= '<td><span style="color:#907000"><b>'.($unknown_libs or "").'</b></span></td>';
		$res .= '<td><span style="color:red"><b>'.($missing_ints or "").'</b></span></td>';
		$res .= '<td><span style="color:#907000"><b>'.($unknown_ints or "").'</b></span></td>';
		
		if ($appchecker_branch{'STD'} eq 'LSB') {
			if ( $com_obj->{NO_SHT_HASH} ){
				$res .= '<td>';
				if ( $all_distrs->{$Did}{NO_GNU_HASH} ) {
					$res .= 'No .gnu.hash support';
				}
				$res .= '</td>';
			}
		}
		
		$res .= '</tr>'.$nl;
	}
	$res .= '</table>'.$nl;
	
	$res .= '<hr />'.$nl;

	return $res;
}


# Contents of a details popup window for common lib comment at the Components tab
sub com_common_details {
	my ($com) = @_;

	my $nl = "\n";
	my $res = "";
	
	my $distr_sel = $Analysis::all_distrs;
	
	my $com_obj = $Analysis::app_components->{$com};
	Analysis::com_common_data($com_obj); # TODO: use distr_sel
	
	if ( !$com_obj ) {
		$res .= "<p>Error: there is no such component: '".esc_html($com)."'</p>";
		return $res;
	}
	
	if ( !$com_obj->{COMMON} ) {
		$res .= "<p>No information.</p>";
		return $res;
	}

	my $link = $appchecker_branch{'NAVIGATOR'}.'rawlib.php'
			.'?cmd=list-byrunname&RLrunname='.esc_url($com_obj->{SONAME});
	$res .= '<h1>Library <span class="title-highlight">'.esc_html($com_obj->{SONAME}).'</span></h1>'.$nl;

	# Explanation
	$res .= '<p>This library is provided by some distributions. Depending on which distributions provide this library (see below), you can consider excluding this library from the application package (making it an external dependency) thus reducing your package size.</p>'.$nl;

	$res .= '<p>Look up the <a href="'.esc_html($link).'" target="_blank"><b>'.esc_html($com_obj->{SONAME}).'</b> library in the '.$appchecker_branch{'STD'}.' Navigator</a>.</p>'.$nl;
	$res .= '<hr/>'.$nl;
	
	# Presence in distributions
	my %present = ();
	my %missing = ();
	foreach my $Did ( grep {$distr_sel->{$_}} keys %{$com_obj->{COMMON_DISTRS}} ) {
		my $distr_name = $Analysis::all_distrs->{$Did}{FULLNAME};
		$present{$distr_name} = $Did;
	}
	foreach my $Did ( grep {$distr_sel->{$_}} keys %$distr_sel ) {
		my $distr_name = $Analysis::all_distrs->{$Did}{FULLNAME};
		next if $present{$distr_name};
		$missing{$distr_name} = $Did;
	}
	
	$res .= '<h2><b>'.esc_html($com_obj->{SONAME}).'</b> presence in distributions:</h2>';
	
	$res .= '<p>';
	$res .= 'Present in <span style="color:green">'.(scalar keys %present).'</span>';
	$res .= ' and missing in <span style="color:red">'.(scalar keys %missing).'</span>';
	$res .= ' of '.(scalar grep {$distr_sel->{$_}} keys %$distr_sel).' distributions.';
	$res .= '</p>'.$nl;
	
	if ( %present ) {
		$res .= '<h3>Present in:</h3>';
		$res .= '<ul>';
		foreach my $distr_name ( sort {lc $a cmp lc $b} keys %present ) {
			my $Did = $present{$distr_name};
			
			my $distr_link = tab_link('distrs').'&id='.$Did;
			
			$res .= '<li><a style="color:green" href="'.esc_html($distr_link).'">'
						.esc_html($distr_name).'</a></li>';
		}
		$res .= '</ul>';
	}

	if ( %missing ) {
		$res .= '<h3>Missing in:</h3>';
		$res .= '<ul>';
		foreach my $distr_name ( sort {lc $a cmp lc $b} keys %missing ) {
			my $Did = $missing{$distr_name};
			
			my $distr_link = tab_link('distrs').'&id='.$Did;
			
			$res .= '<li><a style="color:red" href="'.esc_html($distr_link).'">'
						.esc_html($distr_name).'</a></li>';
		}
		$res .= '</ul>';
	}
	
	$res .= '<hr />'.$nl;

	return $res;
}

sub com_distr_details {
	my ($com, $Did) = @_;

	my $nl = "\n";
	my $res = "";

	my $com_obj = $Analysis::app_components->{$com};
	if ( !$com_obj ) {
		$res .= "<p>Error: there is no such component: '".esc_html($com)."'</p>";
		return $res;
	}
	my $distr = $Analysis::all_distrs->{$Did};
	if ( !$distr ) {
		$res .= "<p>Error: there is no such distribution: '".esc_html($Did)."'</p>";
		return $res;
	}
	
	Analysis::com_distr_data( $com_obj, $Did, $com_obj->{LIBS_UNNEEDED} );
	
	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[
function toggle( id, obj )
{
	var v = document.getElementById( id );
	if ( v ) {
		if ( v.style.display == 'none' ) {
			v.style.display = '';
			if ( obj ) obj.innerHTML = '&ndash;';
		} else {
			v.style.display = 'none';
			if ( obj ) obj.innerHTML = '+';
		}
	}
	return false;
}
// ]]>
</script>
HEREDOC
	
	my ($disp_name, $disp_path) = parse_app_component($com);
	# Header
	$res .= '<h1>Application&nbsp;Component<br/><a href="'.esc_html(tab_link('com').'&com='.esc_url($com)).'" title="'.esc_html($disp_path).'">'.esc_html($disp_name).'</a>'
			.' on&nbsp;<a href="'.esc_html(tab_link('distrs').'&id='.$Did).'" style="white-space:nowrap">'.esc_html($distr->{FULLNAME}).'</a></h1>'.$nl;
	if ( $disp_path ne $disp_name ) {
		$res .= '<p><b>Full path:</b> '.esc_html($disp_path).'</p>';
	}
	$res .= '<hr />'.$nl;
	
	if ($appchecker_branch{'STD'} eq 'LSB') {
		if ( $com_obj->{NO_SHT_HASH} && $distr->{NO_GNU_HASH} ) {
			$res .= '<h2 style="color:#C00000">Section .gnu.hash is not supported by this distribution</h2>'.$nl;
			$res .= '<p>Binaries targeting this distribution must use the \'.hash\' section instead.</p>'.$nl;
			$res .= '<hr />'.$nl;
		}
	}

	$res .= tooltip();

	# Gather data about libraries and interface linkage
	my $libs = {};
	my $order = 1;
	foreach my $soname ( @{$com_obj->{NEEDED_LIBS}}, @{$com_obj->{$Did}{NEEDED_LIBS_IMPL}} ) {
		my $com_lib = $com_obj->{$Did}{LIBS}{$soname};
		my $lib = $libs->{$soname} ||= {};
		$lib->{ORDER} = $order++;

		$lib->{STATUS} = $com_lib->{STATUS} if $com_lib->{STATUS};
		$lib->{FILE} = $com_lib->{FILE} if $com_lib->{FILE};
		$lib->{PACKAGE} = $com_lib->{PACKAGE} if $com_lib->{PACKAGE};
		
		if ( $com_lib->{VIA} ) {
			$lib->{VIA} = $com_lib->{VIA};
			for my $parent ( keys %{$com_lib->{VIA}} ) {
				my $parent_lib = $libs->{$parent} ||= {};
				push @{$parent_lib->{LINKED_LIBS}}, $soname;
			}
		}
	}
	
	# Interfaces
	foreach my $fullname ( keys %{$com_obj->{NEEDED_INTS}} ) {
		if ( my $soname = $com_obj->{$Did}{FOUND_INTS}{$fullname} ) {
			my $int = {};
			
			$int->{STATUS} = 'OK';
			my $lib = $libs->{$soname} ||= {}; $lib->{ORDER} ||= $order++;
			$lib->{INTERFACES}{$fullname} = $int;
			
			$soname = $com_obj->{$Did}{PROBABLE_LIB}{$fullname};
			if ( $soname ) {
				$lib = $libs->{$soname} ||= {}; $lib->{ORDER} ||= $order++;
				$lib->{INTERFACES_MORE}{$fullname} = 1;
			}
		} else {
			my $soname = $com_obj->{$Did}{PROBABLE_LIB}{$fullname} || 'NON-LINKED';
			
			my $int = {};
			$int->{STATUS} = 'MISSING';
			
			my $lib = $libs->{$soname} ||= {}; $lib->{ORDER} ||= $order++;
			$lib->{INTERFACES}{$fullname} = $int;
		}
	}
	
	my $n = 0;
	local *print_lib = sub {
		my ($soname) = @_;
		
		my $lib = $libs->{$soname};
		my $int_count = 0;
		my $int_mis_count = 0;
		if ( $lib->{INTERFACES} ) {
			$int_count = scalar keys %{$lib->{INTERFACES}};
			$int_mis_count = scalar grep {$lib->{INTERFACES}{$_}{STATUS} eq 'MISSING'} keys %{$lib->{INTERFACES}};
		}
		
		$n++;
		
		$res .= '<p>';
		
		my $block_id = 'b'.$n;
		$res .= '[<a href="#" onclick="toggle(\''.$block_id.'\', this);return false;">+</a>] ';
		
		my $color = undef;
		my $add = "";
		if ( !defined $lib->{STATUS} ) {
			# Linked or guessed library
			if ( !defined $lib->{VIA} && $soname ne 'NON-LINKED' ) { # not linked, hence guessed
				$color = 'red';
				$add .= 'Guessed library'
			}
			$add .= 'Runname: '.$lib->{FILE} if $lib->{FILE};
		}
		elsif ( $lib->{STATUS} eq 'OK' ) {
			$color = 'green';
			$add .= 'Runname: '.$lib->{FILE} if $lib->{FILE};
		}
		elsif ( $lib->{STATUS} eq 'UNKNOWN' ) {
			$color = '#907000'; # yellow
			$add .= 'Status is unknown';
		}
		elsif ( $lib->{STATUS} eq 'UNNEEDED' ) {
			$color = '#907000'; # yellow
			$add .= 'Missing library';
		}
		elsif ( $lib->{STATUS} eq 'MISSING' ) {
			$color = 'red';
			$add .= 'Missing library' if $soname ne 'NON-LINKED';
		}

		if ( defined $lib->{STATUS} && $lib->{STATUS} ne 'OK'
			&& (my $alts = $Analysis::alt_libs->{$Did}{$soname})
		) {
			$add .= ' [<a href="#" onclick="javascript:show_hide_tooltip(this, \''.$soname.'\', event); return false;" >see alternatives</a>]';
			$add .= altlib_tooltip_entry($alts, $soname, $Did, $soname);
		}

		$add .= ', ' if $add;
		if ( $int_count == 0 ) {
			if ( defined $lib->{STATUS} ) { # a needed lib
				# <yellow>0</yellow> interfaces
				$add .=  '<span style="color:#907000"><b>0</b></span> interfaces';
			} else {
				$add .=  '0 interfaces';
			}
		}
		elsif ( $lib->{STATUS} && $lib->{STATUS} eq 'MISSING' ) {
			# <red> N interfaces </red>
			$add .=  '<span style="color:red"><b>'.$int_count.'</b>'
				.' interface'.($int_count==1?'':'s').'</span>';
		}
		elsif ( $lib->{STATUS} && $lib->{STATUS} eq 'UNKNOWN'
				|| $soname ne 'NON-LINKED' && !$Analysis::approved_libs->{$soname}
		) {
			# <yellow> N interfaces unknown </yellow>
			$add .= '<span style="color:#907000"><b>'.$int_mis_count.'</b> interfaces - no information</span>';
		}
		else {
			# N interfaces
			if ( $int_count == $int_mis_count && $int_count > 0 ) {
				# <red> M missing interfaces</red>
				$add .= '<span style="color:red"><b>'.$int_mis_count.'</b> missing interface'.($int_mis_count==1?'':'s').'</span>';
			} else {
				my $tmp = '<b>'.$int_count.'</b> interface'.($int_count==1?'':'s');
				if ( !$lib->{STATUS} ) { $tmp = '<span style="color:green">'.$tmp.'</span>'; }
				$add .= $tmp;
				
				if ( $int_mis_count ) {
					$add .= ', ' if $add;
					# <red> M missing </red>
					$add .= '<span style="color:red"><b>'.$int_mis_count.'</b> missing</span>';
				}
				#else {
					#$add .= ', ' if $add;
					#$add .= '0 missing';
				#}
			}
		}
		
		if ( $lib->{INTERFACES_MORE} ) {
			my $int_linked_count = scalar grep {!$lib->{INTERFACES}{$_}} keys %{$lib->{INTERFACES_MORE}};
			if ( $int_linked_count ) {
				$add .= ', ' if $add;
				$add .= ' <b>+'.$int_linked_count.'</b> interface'.($int_linked_count==1?'':'s').' in linked libs';
			}
		}

		if ( $lib->{PACKAGE} ) {
			$add .= ', ' if $add;
			$add .= 'package: '.esc_html($lib->{PACKAGE});
		}

		if ( $color ) {
			$res .= '<span style="color:'.$color.'"><b>'.esc_html($soname).'</b></span>';
		} else {
			$res .= '<b>'.esc_html($soname).'</b>';
		}
		$res .= ' &nbsp; ('.$add.')' if $add;
		$res .= '</p>'.$nl;
		
		$res .= '<div id="'.$block_id.'" style="display:none">'.$nl;
		
		# Interfaces of the library
		if ( $lib->{INTERFACES} ) {
			$res .= '<ul>'.$nl;
			foreach my $fullname ( sort {lc $a cmp lc $b} keys %{$lib->{INTERFACES}} ) {
				my $int = $lib->{INTERFACES}{$fullname};
				$res .= '<li>';
				my $color = "blue";
				my $add = "";
				if ( $int->{STATUS} eq 'MISSING' ) {
					if ( $lib->{STATUS} && $lib->{STATUS} eq 'UNKNOWN'
						|| $soname ne 'NON-LINKED' && !$Analysis::approved_libs->{$soname}
					) {
						$color = '#907000'; # Yellow
					} else {
						$color = 'red';
					}
				}
				
				my ($Iname, $ver) = split /\@/, $fullname;
				my $displayname = $Analysis::unmangled->{$Iname} || $Iname;
				$displayname .= ' @'.$ver if $ver;
				if ( $color ) {
					$res .= '<span style="color:'.$color.'">'.esc_html($displayname).'</span>';
				} else {
					$res .= esc_html($displayname);
				}
				
				# TODO: other versions?
				$res .= '</li>'.$nl;
			}
			$res .= '</ul>'.$nl;
		}
		elsif ( defined $lib->{STATUS} ) { # check this is a NEEDED library
			if ( $Analysis::libs_unneeded->{$soname} ) { # Don't use $lib->{STATUS} here, because it may be 'OK' for an unneeded library.
				$res .= '<p style="padding-left:20pt">Unneeded library</p>';
			}
			elsif ( !$lib->{INTERFACES} ) {
				$res .= '<p style="padding-left:20pt">No interfaces from this library are used by the selected component on the selected distribution. This library isn\'t marked as an unneeded because it is still used on another distribution or by another component.</p>';
			}
		}
		
		if ( !$lib->{STATUS} && $lib->{VIA} || $lib->{LINKED_LIBS} ) {
			$res .= '<table style="margin-left:20pt">'.$nl;
			$res .= '<tr><td align="middle">Linked by:</td>'
					.'<td align="middle">this library</td>'
					.'<td align="middle">Links:</td></tr>'.$nl;
			$res .= '<tr>';
			# Linked by
			$res .= '<td>';
			if ( $lib->{VIA} ) {
				foreach my $soname ( sort keys %{$lib->{VIA}} ) {
					$res .= '<b>'.esc_html($soname).'</b><br/>';
				}
			} else {
				$res .= '<span style="color:gray">none</span>';
			}
			$res .= '</td>';
			$res .= '<td>';
			$res .= '-> '.esc_html($soname).' ->';
			$res .= '</td>';
			# Links
			$res .= '<td>';
			if ( $lib->{LINKED_LIBS} ) {
				foreach my $soname ( @{$lib->{LINKED_LIBS}} ) {
					$res .= '<b>'.esc_html($soname).'</b><br/>';
				}
			} else {
				$res .= '<span style="color:gray">none</span>';
			}
			$res .= '</td>';
			$res .= '</tr></table>'.$nl;
		}
		$res .= '</div>'.$nl;
	};
	
	# Needed libs
	$res .= '<p><b>NEEDED libraries:</b><br/>Below are listed the libraries specified in the NEEDED section of the application component selected.</p>'.$nl;
	foreach my $soname ( @{$com_obj->{NEEDED_LIBS}} ) {
		print_lib($soname);
	}
	if ( !@{$com_obj->{NEEDED_LIBS}} ) {
		# There are no libs in the NEEDED list
		$res .= '<p style="padding-left:20pt"><b>No libraries.</b></p>';
	}
	
	# Linked libs
	my %linked_libs = ();
	foreach my $soname ( sort {$libs->{$a}{ORDER}<=>$libs->{$b}{ORDER}} keys %$libs ) {
		next if defined $libs->{$soname}{STATUS};
		next if $soname eq "NON-LINKED";
		next if !$libs->{$soname}{VIA}; # skip guessed libs
		$linked_libs{$soname} = 1;
	}
	if ( %linked_libs ) {
		$res .= '<hr />'.$nl;
		$res .= '<p><b>Linked libraries:</b><br/>Below are listed the libraries that are additionally linked by the libraries from the NEEDED section of the component.</p>'.$nl;
		foreach my $soname ( sort {$libs->{$a}{ORDER}<=>$libs->{$b}{ORDER}} keys %linked_libs ) {
			print_lib($soname);
		}
	}
	
	# Guessed libs
	my %guessed_libs = ();
	foreach my $soname ( sort {$libs->{$a}{ORDER}<=>$libs->{$b}{ORDER}} keys %$libs ) {
		next if defined $libs->{$soname}{STATUS};
		next if $soname eq "NON-LINKED";
		next if $libs->{$soname}{VIA}; # skip linked libs
		$guessed_libs{$soname} = 1;
	}
	if ( %guessed_libs ) {
		$res .= '<hr />'.$nl;
		$res .= '<p><b>Guessed libraries:</b><br/>Below are listed the libraries that were guessed for some of the interfaces. The Application Checker had to guess library names for the interfaces that weren\'t found in the linked libraries.</p>'.$nl;
		foreach my $soname ( sort {$libs->{$a}{ORDER}<=>$libs->{$b}{ORDER}} keys %guessed_libs ) {
			print_lib($soname);
		}
	}
	
	# Non-linked interfaces
	if ( $libs->{"NON-LINKED"} ) {
		$res .= '<hr />'.$nl;
		print_lib("NON-LINKED");
	}
	
	$res .= '<hr />'.$nl;
	
	return $res;
}
#-----------------------------------------------------------------------

sub ajax_wait {
	return <<HEREDOC;
<div id="ajax_loading" style="display: none;"><div align="center">
Please wait&hellip;<br />
<img src="images/environment-spacer.gif" width="1" height="5" alt="" /><br />
<img src="themes/$appchecker_branch{'DIR'}/images/ajax_progress.gif" width="16" height="16" alt="" /><br />
<img src="images/environment-spacer.gif" width="1" height="7" alt="" /><br />
<input type="button" name="ajax_abort" id="ajax_abort" value="Abort" onclick="javascript:onAbort();" />
</div></div>
HEREDOC

}

1; # return value
